#!/usr/bin/python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.

import exceptions
import parameterlist
import getopt
import sys
import userinterface
from logger import logger

__version__ = "$Rev: 35275 $"

class CliInterface(userinterface.UserInterface):

    ''' Class for creating an abstraction of the Command line interface '''


    def __init__(self, argv, parameter_list, tool_name, tool_description, 
        tool_version):

        ''' Initializes the Cli Interface with the parameters passed through
            command line and the list of acceptable parameters
        '''

        if not isinstance(parameter_list, parameterlist.ParameterList):
            logger.error("Expected ParameterList type")
            raise exceptions.TypeError() 

        self.argv = argv
        self.parameter_list = parameter_list
        self.tool_name = tool_name
        self.tool_description = tool_description
        self.tool_version = tool_version

        self.param_dict = {} #Were the values will be stored
        #Default options for help and version modes
        default_short_opt = 'hv'

        long_opts = ['help']

        for section in parameter_list.get_sections():
            for  long_option in parameter_list.get_section_longopts(section):
                long_opts.append( long_option + "=" )


        try:
            opts, args = getopt.getopt(self.argv[1:], default_short_opt, \
                long_opts)

        except getopt.GetoptError, exp:          
            logger.error("Command line arguments error: " + str(exp))
            self.help()
            raise userinterface.InvalidParameters("The parameters are invalid")

        for opt, arg in opts:
            if opt in ('-h', '--help'):
                self.help()
                raise userinterface.UserInterfaceExit()

                
            for section in self.parameter_list.get_sections():
                
                for parameter in self.parameter_list.get_section_parameters\
                    (section):
                   
                     
                    argument = self.parameter_list.\
                         get_parameter_longopts(section,parameter)
                    
                    #only parse parameters with long options

                    if argument:

                        if opt == "--" + argument:
                            
                            if not self.param_dict.has_key(section):
                                # If the section was first used here, add
                                # the section to the list
                                self.param_dict[section] = {}
                                
                            self.param_dict[section][parameter] = arg


    def get_sections (self):
        ''' Gets the sections that has values in their parameters '''
        return self.param_dict.keys()

    def get_parameter_list(self, section):
        ''' Obtained the list of defined parameters '''
        if self.param_dict.has_key(section):
            parameter_list = self.param_dict[section].keys()
        else:
            parameter_list = []
            
        return parameter_list

    def help(self):
        ''' Displays help message 
        
        '''

        print self.tool_name + "  Version: " + self.tool_version 
        print self.tool_description
        print "\nAccepted parameters:"

        for section in self.parameter_list.get_sections():
            for parameter in self.parameter_list.get_section_parameters\
                (section):
                
                argument = self.parameter_list.\
                    get_parameter_longopts(section,parameter)
                argument_desc = self.parameter_list.\
                    get_parameter_description(section,parameter)

                if argument:
                    print "\t--" + argument + "\t\t" + argument_desc



         
    def exit(self):
        '''Exits from the User Interface'''
        sys.exit()

    def __run__(self):
        '''Executes the interface '''
        # Nothing to be done on execution
        pass


    def get_parameter(self, section, item):
        ''' Return the value of a parameter '''

        try:
            if not self.param_dict.has_key(section):
                #fail if section is not available
                raise userinterface.SectionNotFound(section)
            
            parameter_value = self.param_dict[section][item]  
        except exceptions.KeyError:
            #fail if parameter is not present in section
            raise userinterface.ParameterNotFound(item)

        return parameter_value


