#!/usr/bin/env python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.

import exceptions
from logger import logger
import fileparameterparser
import userinterface
import parameterlist
import os.path

__version__ = "$Rev: 35275 $"

class ParameterNotConfigured(exceptions.Exception):
    ''' Raised when a predefined parameter was not configured in the file
        or command line
     '''

    def __init__(self, parameter):
        exceptions.Exception.__init__(self)
        self.parameter = parameter

    def __str__(self):
        return "Missing parameter %s" % self.parameter

class DuplicatedLongOpts(exceptions.Exception):
    ''' Raised when a long option was duplicated in two different sections
        or the same section
    '''

    def __init__(self, duplicated_parameters_list):
        exceptions.Exception.__init__(self)
        self.duplicated = duplicated_parameters_list

    def __str__(self):
        return "Duplicated long options were found: %s" % str(self.duplicated)

class ParameterNotFound(exceptions.Exception):
    ''' Raised when a parameter was not defined but requested '''

    def __init__(self, parameter):
        exceptions.Exception.__init__(self)
        self.parameter = parameter

    def __str__(self):
        return "Parameter %s was not defined" % self.parameter

class NotConfigured(exceptions.Exception):
    ''' Raised if the recipe config object was not configured yet '''

    def __init__(self):
        exceptions.Exception.__init__(self)

    def __str__(self):
        return "Initialize the recipe config object before using" 


def get_default_file_config_name():
    ''' Returns which of the files is present for configuration  

        example 1 - Find a file when it is not defined

    '''

    files = ['./icr.ini',
             '/opt/intel/icr/etc/icr.ini'
        ] 

    return _get_existent_file(files)

def _get_existent_file(files):
    ''' Returns which of the files is present for configuration  

        example 1 - Find a file when it is not defined
        >>> filelist = [ '/opt/intel/icr/recipe_config.ini',\
                      '/opt/intel/recipe_config.ini'] 
        >>> print _get_existent_file(filelist)
        None
        
        example 2 - Find a defined file
        >>> filelist = [ '/opt/intel/icr/recipe_config.ini',\
                      '/opt/intel/recipe_config.ini',\
                      '/etc/hosts'] 
        >>> print _get_existent_file(filelist)
        /etc/hosts

    '''

    file_iterator = iter(files)
    found = False

    try:
        while found == False:
            file_name = file_iterator.next() 
            found = os.path.isfile(file_name)
        file_name = os.path.abspath(file_name)
    except exceptions.StopIteration:
        file_name = None

    return file_name


def _check_required_parameters(user_interface, file_config, parameter_list):
    ''' Check that all the required parameters are defined in the file 
        or passed through the user interface
    '''

    if not isinstance(user_interface, userinterface.UserInterface):
        logger.error("Expected UserInterface type")
        raise exceptions.TypeError()

    if not isinstance(file_config, fileparameterparser.FileParameterParser):
        logger.error("Expected FileParameterParser type")
        raise exceptions.TypeError()

    if not isinstance(parameter_list, parameterlist.ParameterList):
        logger.error("Expected ParameterList type")
        raise exceptions.TypeError()

    for section in parameter_list.get_sections():
        for parameter in parameter_list.get_section_parameters(section):
            if parameter_list.is_parameter_required(section, parameter):
                try:
                    user_interface.get_parameter(section, parameter)

                except userinterface.UserInterfaceErrors:

                    try:
                        file_config.get(section, parameter)
                    except fileparameterparser.SectionNotFound:
                        logger.error("Missing section %s" % (section))
                        raise ParameterNotConfigured(parameter)
                    except fileparameterparser.ParameterNotFound:
                        logger.error("Missing parameter %s in config file" \
                            % (parameter))
                        raise ParameterNotConfigured(parameter)
                        
    
    

    
class RecipeConfiguration(object):
    ''' Object that unifies recipe configuration gathering from
        user or files
    '''
    def __init__(self):
        ''' Initialize the recipe configuration 
             
        '''
        self.uint = None
        self.file_interface = None
        self.parameter_list = None
        self.cached_parameters = {} 
        


    def configure(self, uint, file_config, parameter_list):
        ''' Configures the recipeconfiguration object 
            Example 1 - Correct initialization 
            >>> import iniparameterparser
            >>> import cliuserinterface
            >>> file_config = iniparameterparser.IniParameterParser\
                ("test_files/recipeconf1.ini")
            >>> param_list = parameterlist.ParameterList('Infiniband', \
                {'IB0_IP' : ['ib0_ip','ib0 infiniband ip address',True],\
                'IB0_NM': [None, 'ib0 netmask address', False],\
                'IB1_NM': [None, 'ib1 netmask address', False],\
                'IB1_IP': [None, 'ib1 ip address', False]})
            >>> dict2 = parameterlist.ParameterList('Ethernet', \
                {'ETH0_IP' : ['eth0_ip','eth0 ip address',True],\
                'ETH0_NM': [None, 'eth0 netmask address', False],\
                'ETH1_NM': [None, 'eth1 netmask address', False],\
                'ETH1_IP': ['eth1_ip', 'eth1 ip address', True]})
            >>> param_list.concat(dict2)
            >>> args=['userinterface.py']
            >>> userinterface = cliuserinterface.CliInterface( args, \
                param_list, "Test", "Demo", "0.01")
            >>> recp_conf = RecipeConfiguration()
            >>> recp_conf.configure(userinterface, file_config, param_list)

            Example 2 - Adding a required parameter not defined anyhere 
            >>> import iniparameterparser
            >>> import cliuserinterface
            >>> file_config = iniparameterparser.IniParameterParser\
                ("test_files/recipeconf1.ini")
            >>> param_list = parameterlist.ParameterList('Infiniband', \
                {'IB0_IP' : ['ib0_ip','ib0 infiniband ip address',True],\
                'IB0_NM': [None, 'ib0 netmask address', False],\
                'IB1_NM': [None, 'ib1 netmask address', False],\
                'IB1_IP': [None, 'ib1 ip address', False]})
            >>> dict2 = parameterlist.ParameterList('Ethernet', \
                {'ETH0_IP' : ['eth0_ip','eth0 ip address',True],\
                'ETH0_NM': [None, 'eth0 netmask address', False],\
                'ETH1_NM': [None, 'eth1 netmask address', False],\
                'ETH2_IP': ['eth2_ip', 'eth2 ip address', True]})
            >>> param_list.concat(dict2)
            >>> args=['userinterface.py']
            >>> userinterface = cliuserinterface.CliInterface( args, \
                param_list, "Test", "Demo", "0.01")
            >>> recp_conf = RecipeConfiguration()
            >>> recp_conf.configure(userinterface, file_config, param_list)
            Traceback (most recent call last):
            ParameterNotConfigured: Missing parameter ETH2_IP
            
            Example 3 - Adding a required parameter only defined through 
            command line (Modifies the file)
            >>> import iniparameterparser
            >>> import cliuserinterface
            >>> file_config = iniparameterparser.IniParameterParser\
                ("test_files/recipeconf2.ini")
            >>> param_list = parameterlist.ParameterList('Infiniband', \
                {'IB0_IP' : ['ib0_ip','ib0 infiniband ip address',True],\
                'IB0_NM': [None, 'ib0 netmask address', False],\
                'IB1_NM': [None, 'ib1 netmask address', False],\
                'IB1_IP': [None, 'ib1 ip address', False]})
            >>> dict2 = parameterlist.ParameterList('Ethernet', \
                {'ETH0_IP' : ['eth0_ip','eth0 ip address',True],\
                'ETH0_NM': [None, 'eth0 netmask address', False],\
                'ETH1_NM': [None, 'eth1 netmask address', False],\
                'ETH3_IP': ['eth3_ip', 'eth3 ip address', True]})
            >>> param_list.concat(dict2)
            >>> args=['userinterface.py', '--eth3_ip=192.168.0.1']
            >>> userinterface = cliuserinterface.CliInterface( args, \
                param_list, "Test", "Demo", "0.01")
            >>> recp_conf = RecipeConfiguration()
            >>> recp_conf.configure(userinterface, file_config, param_list)

            Example 4 - Adding a non required parameter not defined 
            >>> import iniparameterparser
            >>> import cliuserinterface
            >>> file_config = iniparameterparser.IniParameterParser\
                ("test_files/recipeconf3.ini")
            >>> param_list = parameterlist.ParameterList('Infiniband', \
                {'IB0_IP' : ['ib0_ip','ib0 infiniband ip address',True],\
                'IB0_NM': [None, 'ib0 netmask address', False],\
                'IB1_NM': [None, 'ib1 netmask address', False],\
                'IB1_IP': [None, 'ib1 ip address', False]})
            >>> dict2 = parameterlist.ParameterList('Ethernet', \
                {'ETH0_IP' : ['eth0_ip','eth0 ip address',True],\
                'ETH0_NM': [None, 'eth0 netmask address', False],\
                'ETH1_NM': [None, 'eth1 netmask address', False],\
                'ETH3_IP': ['eth3_ip', 'eth3 ip address', False]})
            >>> param_list.concat(dict2)
            >>> args=['userinterface.py']
            >>> userinterface = cliuserinterface.CliInterface( args, \
                param_list, "Test", "Demo", "0.01")
            >>> recp_conf = RecipeConfiguration()
            >>> recp_conf.configure(userinterface, file_config, param_list)

        ''' 
        if not isinstance(uint, userinterface.UserInterface):
            logger.error("Expected UserInterface type")
            raise exceptions.TypeError()
            
        
        if not isinstance(file_config, fileparameterparser.FileParameterParser):
            logger.error("Expected FileParameterParser type")
            raise exceptions.TypeError()

        if not isinstance(parameter_list, parameterlist.ParameterList):
            logger.error("Expected ParameterList type")
            raise exceptions.TypeError()


        
        _check_required_parameters(uint, file_config, parameter_list)

        
        self.uint = uint 
        self.file_interface = file_config
        self.parameter_list = parameter_list

        
    def is_configured(self):
        ''' Returns True if it was already configured 

            Example 1 - Not configured
            >>> rc = RecipeConfiguration()
            >>> rc.is_configured()
            False
        '''

        return self.uint is not None and self.file_interface is not None



    def get_parameter(self, section, parameter):
        ''' Gets a configured parameter. Searches first if it was defined
            in an user interface, then in the File. 
            
            Example 1 - get a parameter from the file 
            >>> import iniparameterparser
            >>> import cliuserinterface
            >>> file_config = iniparameterparser.IniParameterParser\
                ("test_files/recipeconf1.ini")
            >>> param_list = parameterlist.ParameterList('Infiniband', \
                {'IB0_IP' : ['ib0_ip','ib0 infiniband ip address',True],\
                'IB0_NM': [None, 'ib0 netmask address', False],\
                'IB1_NM': [None, 'ib1 netmask address', False],\
                'IB1_IP': [None, 'ib1 ip address', False]})
            >>> dict2 = parameterlist.ParameterList('Ethernet', \
                {'ETH0_IP' : ['eth0_ip','eth0 ip address',True],\
                'ETH0_NM': [None, 'eth0 netmask address', False],\
                'ETH1_NM': [None, 'eth1 netmask address', False],\
                'ETH1_IP': ['eth1_ip', 'eth1 ip address', True]})
            >>> param_list.concat(dict2)
            >>> args=['userinterface.py']
            >>> userinterface = cliuserinterface.CliInterface( args, \
                param_list, "Test", "Demo", "0.01")
            >>> recp_conf = RecipeConfiguration()
            >>> recp_conf.configure(userinterface, file_config, param_list)
            >>> recp_conf.get_parameter('ETH0_IP')
            '192.168.24.11'

            Example 2 - get a parameter overwritten through command line 
            (Modifies the file)
            >>> import iniparameterparser
            >>> import cliuserinterface
            >>> file_config = iniparameterparser.IniParameterParser\
                ("test_files/recipeconf4.ini")
            >>> param_list = parameterlist.ParameterList('Infiniband', \
                {'IB0_IP' : ['ib0_ip','ib0 infiniband ip address',True],\
                'IB0_NM': [None, 'ib0 netmask address', False],\
                'IB1_NM': [None, 'ib1 netmask address', False],\
                'IB1_IP': [None, 'ib1 ip address', False]})
            >>> dict2 = parameterlist.ParameterList('Ethernet', \
                {'ETH0_IP' : ['eth0_ip','eth0 ip address',True],\
                'ETH0_NM': [None, 'eth0 netmask address', False],\
                'ETH1_NM': [None, 'eth1 netmask address', False],\
                'ETH1_IP': ['eth1_ip', 'eth1 ip address', True]})
            >>> param_list.concat(dict2)
            >>> args=['userinterface.py', '--eth0_ip=10.1.1.1']
            >>> userinterface = cliuserinterface.CliInterface( args, \
                param_list, "Test", "Demo", "0.01")
            >>> recp_conf = RecipeConfiguration()
            >>> recp_conf.configure(userinterface, file_config, param_list)
            >>> recp_conf.get_parameter('ETH0_IP')
            '10.1.1.1'

        '''
        if not self.is_configured():
            raise NotConfigured()

        if parameter in self.cached_parameters:
            value = self.cached_parameters[parameter]
        else:
            try:
                value = self.uint.get_parameter(section,parameter)
            except userinterface.UserInterfaceErrors:
                try:
                    value = self.file_interface.get(section,\
                            parameter)
                except fileparameterparser.FileParserErrors:
                    raise ParameterNotFound(parameter)

        return value

    def get_multi_parameter(self, section, parameter):
        ''' Gets a configured multiple line parameter in the file. 
        '''
        if not self.is_configured():
            raise NotConfigured()

        if parameter in self.cached_parameters:
            value = self.cached_parameters[parameter]
        else:
            try:
                value = self.file_interface.get_multi(section, parameter)
            except fileparameterparser.FileParserErrors:
                raise ParameterNotFound(parameter)

        return value

    def set_parameter(self, section, parameter, value):
        ''' Sets a parameter in the configuration section
            
            method must be called (Modifies the file)
            
            Example 1 - set a parameter  
            >>> import iniparameterparser
            >>> import cliuserinterface
            >>> file_config = iniparameterparser.IniParameterParser\
                ("test_files/recipeconf5.ini")
            >>> param_list = parameterlist.ParameterList('Infiniband', \
                {'IB0_IP' : ['ib0_ip','ib0 infiniband ip address',True],\
                'IB0_NM': [None, 'ib0 netmask address', False],\
                'IB1_NM': [None, 'ib1 netmask address', False],\
                'IB1_IP': [None, 'ib1 ip address', False]})
            >>> dict2 = parameterlist.ParameterList('Ethernet', \
                {'ETH0_IP' : ['eth0_ip','eth0 ip address',True],\
                'ETH0_NM': [None, 'eth0 netmask address', False],\
                'ETH1_NM': [None, 'eth1 netmask address', False],\
                'ETH1_IP': ['eth1_ip', 'eth1 ip address', True]})
            >>> param_list.concat(dict2)
            >>> args=['userinterface.py']
            >>> userinterface = cliuserinterface.CliInterface( args, \
                param_list, "Test", "Demo", "0.01")
            >>> recp_conf = RecipeConfiguration()
            >>> recp_conf.configure(userinterface, file_config, param_list)
            >>> recp_conf.set_parameter('Ethernet','ETH0_IP', '10.0.0.1')
            >>> recp_conf.get_parameter('ETH0_IP')
            '10.0.0.1'
        '''
        if not self.is_configured():
            raise NotConfigured()

        self.cached_parameters[parameter] = value
        

def _test():
    """ Inline doctest. """
    import doctest
    doctest.testmod()


if __name__ == "__main__":
    _test()


