""" Logger module for Intel(R) Cluster Ready scripts """
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.


import logging
import logging.handlers
import os
import datetime

logger = logging.getLogger('clck')

__version__ = "$Rev: 35275 $"

#Get this from the configuration module
LOGDIR = '/var/log/icr'
LOG_TIME_STAMP=datetime.datetime.now().strftime("%Y%m%d.%H%M%S")
LOGFILENAME = 'icr-%s.log' % (LOG_TIME_STAMP)
LOGFILE = os.path.join(LOGDIR, LOGFILENAME)

FILE_PERMISSION = True # Flag to check if we have access to log file

# Try to create the icr log directory if it didn't exist
if not os.path.exists(LOGDIR):
    try:
        os.mkdir(LOGDIR)
    except OSError:
        FILE_PERMISSION = False
    
# Some environment variable to enable DEBUG log level when set
if os.getenv("ICR_SCRIPTS_DEBUG"):
    LEVEL = logging.DEBUG
else:
    LEVEL = logging.INFO

# Use DEBUG level always. So it is easier to send output to the log and not
# to the screen, by using logger.debug
LEVEL = logging.DEBUG

try:
    filehdlr = logging.FileHandler(LOGFILE)
    formatter = \
        logging.Formatter('%(asctime)s %(levelname)s %(module)s %(message)s')
    filehdlr.setFormatter(formatter)
except IOError:
    FILE_PERMISSION = False

conshdlr = logging.StreamHandler()
conshdlr.setLevel(logging.INFO) #Do not log debug messages to the console
                                #(just info and higher)

###
###
# This is not nice but I couldn't find a better way.
# Currently the logger has two handlers. One logs to a file (filehdlr) and the
# other outputs to screen (conshdlr).
# That is because in the script we usually want to output to screen and log
# the same messages. Is that correct?
# If we wanted to print to screen a fancier message for example using
# display_text and we also want that message to be logged, we would call
# display_text and log with the same text, but we will have it output to
# screen twice. Once formatted by display_text and the other raw by the log
# console handler.
# With this set_conshdlr_emit we can set display_text as the emit method
# for the console handler of the logger, and setting it back afterward
# if we want to stop using display_text and continue outputting raw to screen
# For example
# 
# >>> from logger import logger
# >>> logger.critical('This will be logged and outputted raw')
# >>>
# >>> from logger import set_conshdlr_emit
# >>> from recipeutils import display_text
# >>> set_conshdlr_emit(display_text)
# >>> logger.critical('This will be logged and displayed in color')
# >>> set_conshdlr_emit()
# >>> logger.critical('This will be logged and displayed raw again')

std_emit = conshdlr.emit

def set_conshdlr_emit(emit=std_emit):
    """ Changes the emit method of a logger handler for the function given
        in the emit parameter. If no emit argument is given it is used the
        emit method that conshdlr handler had, which is to use stderr.
    """
    handler = conshdlr

    if emit is std_emit:
        handler.emit = emit
    else:
        def myemit(msg):
            """ Mock emit functions. Just calls the emit function passed
                to set_conshdlr_emit with the message contained in the object
                it receives.
            """
            if isinstance(msg, basestring):
                emit(msg)
            else:
                emit(msg.getMessage())
        handler.emit = myemit
###
###

prev_emit = conshdlr.emit

def stop_conshdlr():
    """ Replaces the emit method of the console logger handler with a void
        function, so output to console is ignored but still logs to a file.
    """
    global prev_emit
    prev_emit = conshdlr.emit
    set_conshdlr_emit(emit=lambda x: 0)

def resume_conshdlr():
    """ Restores the emit method for the console logger with the value prior
        to 'stop_conshdlr' invocation.
    """
    global prev_emit
    set_conshdlr_emit(prev_emit)

if FILE_PERMISSION:
    logger.addHandler(filehdlr)
    
logger.addHandler(conshdlr)
logger.setLevel(LEVEL)

if not FILE_PERMISSION:
    logger.warning("Could not start file logger. Logging to %s disabled." % \
                   LOGFILE)

