#!/usr/bin/python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.


""" This module defines a common API for many actions to be executed by the
    scripts that are performend differently according to the Operating
    System being used.
"""

# System Modules
import exceptions
import re
import os
import shutil
import tempfile
import time

# Internal Modules
from common.logger import logger
from configrecipe import ParameterNotFound, RecipeConfiguration 
import recipeutils as ru
import common.parameterlist as parameterlist
import common.ipaddr as ipaddr

__version__ = "$Rev: 37520 $"

##############################################################################
#         Exception Classes                                                  #
##############################################################################

class OSALException(exceptions.Exception):
    """ Exceptions for the Operating System Abstraction Layer base class """
    def __init__(self, reason):
        exceptions.Exception.__init__(self)
        self.reason = reason

    def __str__(self):
        return self.reason


class OSNotFound(OSALException):
    """ Exception raised when a specific operating system could not be found
        in the local system
    """

    def __init__(self, name=None):
        if name == None:
            OSALException.__init__(self, "No supported operating "\
                                       "system was detected")
        else:
            OSALException.__init__(self, "The " + name + " operating" \
                                       " system was not detected")

class ServiceError(OSALException):
    """ Exception raised when a problem occurs managing a service. """

    def __init__(self, message):
        OSALException.__init__(self, message)

class PackageError(OSALException):
    """ Exception raised when a problem occurs managing a package. """

    def __init__(self, message):
        OSALException.__init__(self, message)

class UserCreationError(OSALException):
    """ Exception raised when a problem occurred when creating a user. """

    def __init__(self, message):
        OSALException.__init__(self, message)

class UserAlreadyExists(OSALException):
    """ Exception raised when an already existing user was created. """

    def __init__(self, message):
        OSALException.__init__(self, message)

class UserDoesNotExists(OSALException):
    """ Exception raised when an already existing user was created. """

    def __init__(self, message):
        OSALException.__init__(self, message)

class ModReloadError(OSALException):
    """ Exception raised when the module could not be reloaded. """

    def __init__(self, message):
        OSALException.__init__(self, message)

class ModLoadError(OSALException):
    """ Exception raised when the module could not be loaded. """

    def __init__(self, message):
        OSALException.__init__(self, message)

class ModUnloadError(OSALException):
    """ Exception raised when the module could not be unloaded. """

    def __init__(self, message):
        OSALException.__init__(self, message)

class RefreshRepoException(OSALException):
    """ Exception raised when the repository could not be refreshed. """

    def __init__(self, message):
        OSALException.__init__(self, message)

class SymLinkException(OSALException):
    """ Exception raised when a symlink could not be created"""

    def __init__(self, message):
        OSALException.__init__(self, message)

class MissingParameterException(OSALException):
    """ Exception raised when a method is called with wrong qty of parameters"""

    def __init__(self, message):
        OSALException.__init__(self, message)

class FileCopyException(OSALException):
    """ Exception raised when a file could not be copied"""

    def __init__(self, message):
        OSALException.__init__(self, message)

class DirCreationException(OSALException):
    """ Exception raised when a dir could not be created"""

    def __init__(self, message):
        OSALException.__init__(self, message)

class DirRemovalException(OSALException):
    """ Exception raised when a dir could not be removed"""

    def __init__(self, message):
        OSALException.__init__(self, message)

class FileCreationException(OSALException):
    """ Exception raised when a file could not be created"""
    
    def __init__(self, message):
        OSALException.__init__(self, message)

class FileNotFoundException(OSALException):
    """ Exception raised when a method try to do an operation with an\
    non existing file"""

    def __init__(self, message):
        OSALException.__init__(self, message)

class FileNotWritable(OSALException):
    """ Exception raised when a method try to write a non writable file"""

    def __init__(self, message):
        OSALException.__init__(self, message)

class UnzipException(OSALException):
    """ Exception raised when unzip command returns non zero exit status"""

    def __init__(self, message):
        OSALException.__init__(self, message)

class UntarException(OSALException):
    """ Exception raised when tar command returns non zero exit status"""

    def __init__(self, message):
        OSALException.__init__(self, message)

class TempFileNotCreated(OSALException):
    """ Exception raised when a temporary file could not be created. """

    def __init__(self, message):
        OSALException.__init__(self, message)

#############################################################################
#         Operating System Classes                                          #
#############################################################################

class _OSAL(object):
    """ Base class for operating system abstraction layer.
        Implements common methods. Derived classes implements specific ones
        or overrides the generic ones.
    """

    def __init__(self, name = None, forced = False):

        if name is None:
            #If the name was not already provided the name will be the
            #name of the class
            self.name = self.__class__.__name__
        else:
            self.name = name

        # Version is not defined until self.probe() is called
        self.version = None

        if forced == False:
            # Probe for the existence of the OS
            logger.debug('Probing for ' + self.name +
                         ' operating system presence')
            self.probe()
        else:
            self.version = 'Unknown'

        # Set INI parameters used by the OS class
        self._defaultdict = {}
        self._section = "OS"
        self._dict = {'PASSWD':[None, "passwd file location", False],
                      'OPENIB':[None, "openib.conf file location", False],
                      'IFCFG':[None, "ifcfg-* files location", False],
                      'DATCNF':[None, "dat.conf file location", False],
                      'MODPRB':[None, "modprobe.conf file location", False],
                      'LIMITS':[None, "limits.conf file location", False],
                      'SYSCTL':[None, "sysctl.conf file location", False],
                      'PROFILE': [None, "user profile file location" , False],
                      }

        self.SYSCTL_FILE = None
        self.LIMITS_FILE = None
        self.MODPRB_FILE = None
        self.DATCNF_FILE = None
        self.IFCFG_FILE = None
        self.OPENIB_FILE = None
        self.PASSWD_FILE = None
        self.PROFILE_FILE = None
    
        # Define the infiniband services
        self.ib_opensm_service = "opensmd"
        self.ib_openib_service = "openibd" 

    def __str__(self):
        return self.name + " " + self.version

    def probe(self):
        """ This method should be implemented in the child classes"""
        error = "Probe method should be implemented in child class"
        raise NotImplementedError(error)

    def get_parameter_list(self):
        """ This method return a list with all the variables that Config Recipe
        should understand.
        """
        if self._section == '' or self._dict == {}:
            raise OSALException('Section or vars not defined.')
        else:
            params = parameterlist.ParameterList(self._section, self._dict)
            if self._defaultdict != {}:
                defparams = parameterlist.ParameterList('Default',
                                                        self._defaultdict)
                params.concat(defparams)

            return params

    def set_conf_files(self, config):
        """ This method sets the OS configuration files """

        if isinstance(config, RecipeConfiguration):
            # Sets sysctl.conf file path
            try:
                self.SYSCTL_FILE = config.get_parameter(self._section,"SYSCTL")
            except ParameterNotFound:
                self.SYSCTL_FILE = "/etc/sysctl.conf"

            # Sets limits.conf file path
            try:
                self.LIMITS_FILE = config.get_parameter(self._section,"LIMITS")
            except ParameterNotFound:
                self.LIMITS_FILE = "/etc/security/limits.conf"

            # Sets modprobe.conf file path
            try:
                self.MODPRB_FILE = config.get_parameter(self._section,"MODPRB")
            except ParameterNotFound:
                self.MODPRB_FILE = "/etc/modprobe.conf"

            # Sets dat.conf file path
            try:
                self.DATCNF_FILE = config.get_parameter(self._section,"DATCNF")
            except ParameterNotFound:
                self.DATCNF_FILE = "/etc/dat.conf"

            # Sets ifcfg network scripts path and prefix
            try:
                self.IFCFG_FILE = config.get_parameter(self._section,"IFCFG")
            except ParameterNotFound:
                self.IFCFG_FILE = "/etc/sysconfig/network-scripts/ifcfg-"

            # Sets openib.conf file path
            try:
                self.OPENIB_FILE = config.get_parameter(self._section,"OPENIB")
            except ParameterNotFound:
                self.OPENIB_FILE = "/etc/ofed/openib.conf"

            # Sets passwd file path
            try:
                self.PASSWD_FILE = config.get_parameter(self._section,"PASSWD")
            except ParameterNotFound:
                self.PASSWD_FILE = "/etc/passwd"
            
            # Sets the profile file path
            try:
                self.PROFILE_FILE = config.get_parameter(self._section,"PROFILE")
            except ParameterNotFound:
                self.PROFILE_FILE = "/etc/profile"
        else:
            error = 'RecipeConfiguration type expected but _%s_ received' % \
                    str(type(config)).split("'")[1]
            logger.error(error)
            raise exceptions.TypeError(error)

    @property
    def board_name(self):
        """ Gets the Platform Name from the FRU using dmidecode or lshal. """
        try:
            for line in ru.execute_command('dmidecode'):
                if "Product Name" in line:
                    return line.strip().split()[2]
        except ru.CommandException:
            for line in ru.execute_command('lshal -l -u ' \
                '/org/freedesktop/Hal/devices/computer'):
                line = line.strip()
                if line.startswith('system.product'):
                    fullname = line.split("'")[1]
                    if "not specified" in fullname.lower():
                        break
                    return fullname.split()[0]
            raise OSALException("Could not get board name using dmidecode " \
                                 "or lshal.")

        raise OSALException("Could not get board name using dmidecode")

    def pci_devices(self):
        """ Returns a list of pci devices.
        """
        return ru.execute_command('lspci')

    def eth_devices(self):
        """ Returns the subset of pci_devices which are Ethernet devices
        """
        return [dev for dev in self.pci_devices() \
                if re.search('Ethernet Controller', dev, re.IGNORECASE)]

    def eth_10gb_devices(self):
        """ Returns the subset of eth_devices which are of 10gb
        """
        return [dev for dev in self.eth_devices() \
                if re.search('10-Gigabit', dev, re.IGNORECASE)]

    def ib_devices(self):
        """ Returns the subset of pci_devices which are InfiniBand devices.
        """
        return [dev for dev in self.pci_devices() \
                if re.search('InfiniBand', dev, re.IGNORECASE)]

    def device_info(self, device):
        """ Returns a dictionary with configuration values of a network device
            by parsing the output of ifconfig.
        """
        found = False
        info = {}

        try:
            output = ru.execute_command('ifconfig %s' % device)
        except ru.CommandException, ex:
            logger.debug(ex)
            logger.debug('error executing ifconfig %s' % device)
            return info

        for line in output:
            line = line.strip()
            if re.match('%s\s+' % device, line):
                found = True
            elif found:
                if not line:
                    break
                else:
                    for pattern in [('HWaddr ([^\s]+)', 'HWaddr'),
                                    ('inet addr:([^\s]+)', 'inet addr'),
                                    ('inet6 addr: ([^\s]+)', 'inet6 addr'),
                                    ('Bcast:([^\s]+)', 'Bcast'),
                                    ('Mask:([^\s]+)', 'Mask')]:
                        match = re.search(pattern[0], line)
                        if match:
                            info[pattern[1]] = match.group(1)
        return info

    def configure_device(self, dev, ip_addr='', netmask='', filename='',
                         netname='', disable=False):
        """ Configures a network device.
            Parameters:
            dev      -- network device, e.g. ib0, eth1
            ip_addr  -- ip address for the device
            netmask  -- subnet mask for the device
            filename -- name of the configuration file where to write the
                        configuration.
                        Defaults to ...network-scripts/ifcfg-<dev>
            netname  -- rocks+ parameter to be ignored
            disable  -- Writes the configuration to disable the device.
                        In this case it does not need ip_addr or netmask.
        """
        if not filename:
            filename = self.IFCFG_FILE + dev

        if disable:
            file_contents = """\
#Modified by Intel(R) Cluster Ready Reference Recipe scripts
DEVICE=%s
BOOTPROTO=none
ONBOOT=no""" % dev

            _fd = open(filename, 'w')
            _fd.write(file_contents)
            _fd.close()

        else:
            if not ip_addr or not netmask:
                raise ValueError('No ip_addr or netmask values provided')
            
            # Calculate the prefix for ip address/netmask
            network = ipaddr.IPv4Network("%s/%s" % (ip_addr, netmask))
            prefix = network.prefixlen
            
            file_contents = """\
#Modified by Intel(R) Cluster Ready Reference Recipe scripts
DEVICE=%s
IPADDR=%s
PREFIX=%s
BOOTPROTO=static
ONBOOT=yes""" % (dev, ip_addr, prefix)

            _fd = open(filename, 'w')
            _fd.write(file_contents)
            _fd.close()


    def openibd_status(self):
        """ Returns the status of openibd driver, since its output from
            'service openibd status' is different from other services
            it is parsed in a different method.
        """
        status = 'other'
        try:
            cmd = 'service %s status' % self.ib_openib_service
            output = ru.execute_command(cmd)
        except:
            return status

        for line in output:
            if 'HCA driver' in line:
                status = line.strip()
                status = status.replace('HCA driver is ', '')
                status = status.replace('HCA driver ', '')
                return status
            if 'Currently active IPoIB interfaces:' in line:
                # This is another status format, in which if the service is
                # down, it will return with status != 0, so we would have
                # caught the exception and already returned 'other'
                return 'loaded'
        return status


    def service_running(self, service):
        """ Parses the output of service status for a given service and
            returns True if it is running, False if it is stopped or
            raises an exception if it can't get its status.
        """
        out = ru.execute_command('service %s status' % service,
                                 ignore_errors=True)
        if 'unrecognized service' in out[-1]:
            logger.error('service %s was not recognized' % service)
            raise ServiceError(out[-1])
        elif 'is stopped' in out[-1]:
            return False
        elif 'is running...' in out[-1]:
            return True
        else:
            logger.debug('Could not get service status from output: %s' % \
                               '\n'.join(out))
            raise ServiceError('Could not get service %s status' % service)

    def service_start(self, service, print_output=False):
        """ Starts a given service. If print_output is set, it will display
            on the standard output the messages printed by service start.
        """
        try:
            ru.execute_command('service %s start' % service,
                               print_output=print_output)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise ServiceError("Error starting service " + service)

    def service_stop(self, service, print_output=False):
        """ Stops a given service. If print_output is set, it will display
            on the standard output the messages printed by service stop.
        """
        try:
            ru.execute_command('service %s stop' % service,
                               print_output=print_output)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise ServiceError("Error stopping service " + service)

    def service_restart(self, service, print_output=False):
        """ Restarts a given service. If print_output is set, it will display
            on the standard output the messages printed by service restart.
        """
        try:
            ru.execute_command('service %s restart' % service,
                               print_output=print_output)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise ServiceError("Error restarting service " + service)

    def service_enable(self, service):
        """ Configures the given service to be started at boot. """
        try:
            ru.execute_command('chkconfig %s on' % service)
        except ru.CommandException:
            raise ServiceError('Error enabling service %s' % service)

    def service_disable(self, service):
        """ Configures the given service to not be started at boot. """
        try:
            ru.execute_command('chkconfig %s off' % service)
        except ru.CommandException:
            raise ServiceError('Error disabling service %s' % service)

    def service_init_status(self, service):
        """ Gets the current init.d configuration for boot time execution.
        """
        try:
            out = ru.execute_command('chkconfig --list %s' % service)
            out_text = " ".join(out)
        except ru.CommandException, ex:
            raise ServiceError('Error getting service %s init status: %s' % \
                              (service, ex))


        # parse an output of the kind
        # dhcpd     0:off   1:off   2:on    3:on    4:on    5:on    6:off
        # returning ['off', 'off', 'on', 'on', 'on', 'on', 'off']

        status_list = out_text.split(':')

        if len(status_list) != 8:
            raise ServiceError('Can\'t get boot init status for service %s ' \
                               'from the line %s' % (service, out_text))

        # skip the first useless item
        status_list = status_list[1:]

        result = []

        for status in status_list:
            if 'on' in status:
                result.append('on')
            elif 'off' in status:
                result.append('off')
            else:
                raise ServiceError("Can't get init status in the segment " \
                                   "'%s' of the line %s" % (status, out_text))

        return result

    def has_package(self, package):
        """ Checks if a package was installed using current distribution
        package system. """

        try:
            ru.execute_command('rpm -q --info %s' % package)
            result = True
        except ru.CommandException:
            result = False

        return result

    def remove_package(self, package):
        """ Removes a package using current distribution package system. """
        try:
            ru.execute_command('yum -y remove %s' % package)
        except ru.CommandException, ex:
            raise PackageError('Error removing package %s: %s' % \
                               (package, ex))
    
    def refresh_repo(self):
        """ Refresh the yum repository """
        try:
            ru.execute_command("yum clean all")
        except ru.CommandException, err:
            raise RefreshRepoException("Error refreshing repository %s" %
                                       str(err))

    def install_package(self, package, extra_args=''):
        """ Installs a package using current distribution package system. """
        #Check if the package is available first as yum install
        #will just ignore failing packages
 
        if not self.has_package_in_repo(package):
            raise PackageError('Package %s missing in repository' % package)
        
        try:
            ru.execute_command('yum -y %s install %s' % (extra_args, package))
        except ru.CommandException, ex:
            raise PackageError('Error installing package %s: %s' % \
                               (package, ex))

    def has_package_in_repo(self, package):
        """ Checks that the package is available in the repository """
        try:
            ru.execute_command('yum info %s' % package)
            haspackage = True
        except ru.CommandException:
            haspackage = False
        
        return haspackage


    def add_user(self, user_name):
        """ Creates a new user in the system
        """
        try:
            ru.execute_command('useradd %s' % (user_name))
        except ru.CommandException, ex:
            if ex.status == 9:
                raise UserAlreadyExists("User %s already exists" % (user_name))
            else:
                raise UserCreationError(str(ex))

    def has_user(self, user_name):
        """ Returns true if a user was created
        """

        try:
            ru.execute_command('cat /etc/passwd | grep %s:' % (user_name))
            exists = True
        except ru.CommandException:
            exists = False

        return exists

    def del_user(self, user_name):
        """ Removes a user from the system """

        try:
            ru.execute_command('userdel %s' % (user_name))
        except ru.CommandException, ex:
            logger.warn("User %s could not be deleted: %s", (user_name, \
                str(ex)))


    def has_module_loaded(self, module):
        """ Returns true if the module is loaded in the os """

        try:
            ru.execute_command('lsmod | grep %s' % (module))
            loaded = True
        except ru.CommandException:
            loaded = False

        return loaded

    def load_module(self, module):
        """ Loads a kernel module """

        try:
            logger.info("- Loading module %s" % module)
            ru.execute_command('modprobe %s'% module)
        except ru.CommandException:
            raise ModLoadError("Error loading %s" % (module))

    def unload_module(self, module):
        """ Unloads a kernel module """

        if self.has_module_loaded(module):
            try:
                logger.info("- Unloading module %s" % module)
                ru.execute_command('modprobe -r %s' % module)
            except ru.CommandException:
                raise ModUnloadError("Error unloading %s" % (module))
        else:
            logger.debug('Module already unloaded: %s' % module)
    
    def reload_module(self, module):
        """ Reloads a kernel module """

        # Unload the module if it is loaded
        if self.has_module_loaded(module):
            try:
                self.unload_module(module)
            except ModUnloadError:
                raise ModReloadError("Error reloading %s" % (module))
            # Give some time to proper unload the module
            time.sleep(2)

        # Load the module
        try:
            self.load_module(module)
        except ModLoadError:
                raise ModReloadError("Error reloading %s" % (module))

    def has_sshkeys(self, user_name):
        """ Check if a user has SSH keys generated """
        try:
            logger.info("Checking SSH keys for user %s" % (user_name))
            ru.execute_command('[ -f /home/%s/.ssh/id_rsa ]' % (user_name))
            result = 1
        except ru.CommandException:
            logger.warn("SSH keys are not generated for user %s"
                        % (user_name))
            result = 0
        return result

    def user_check(self, uid=0):
        """ Checks if current user has the given uid. If uid is not given, its
            default value is 0, so it checks if the user is root.
        """
        if os.geteuid() == uid:
            return True
        return False

    def safe_cp(self, orig, dest):
        """ Copies file 'orig' to file 'dest' leaving original 'dest' file in
            'dest.old'
        """

        if ( orig and dest ) is None:
            error = 'The file "%s" or destination "%s" are not valid' % \
                (str(orig),str(dest))
            raise FileCopyException(error)

        if not os.path.isfile(orig):
            error = '%s is not a file' % orig
            logger.error(error)
            raise FileCopyException(error)

        if os.path.isfile(dest):
            shutil.copyfile(dest, dest + '.old')
        elif os.path.isdir(dest):
            orig_filename = orig.split(os.path.sep)[-1]
            path = os.path.sep.join( [dest, orig_filename] )
            if os.path.isfile(path):
                shutil.copyfile(path, path + '.old')

        shutil.copy(orig, dest)

    def get_full_path(self, dirpath, filetmp):
        """ Generate a full path for a File
        """
        return (("%s/%s") % (dirpath, filetmp))

    def get_last_file_modified(self, dirpath, pattern_file):
        """ Get the latest file pattern modified in dirpath
        """
        files =  self.files_in(dirpath)
        mod = 0
        last_file_modified = ''
        for filename in files:
            if pattern_file in filename:
                modified = os.stat(("%s/%s" % (dirpath, filename))).st_mtime
                if modified > mod:
                    mod = modified
                    last_file_modified = filename

        return last_file_modified

    def files_in(self, dirpath):
        """ Get the files at any folder
        """
        logger.debug("Reading the files at %s " % (dirpath))
        logsfiles = os.listdir(dirpath)
        logger.debug("Reading the files at %s " % (logsfiles))
        return logsfiles

    def create_dir(self, fname):
        """ Create a dir Folder. """
        try:
            os.mkdir(fname)
            logger.info("The folder %s was created" % (fname))
        except os.error, ex:
            logger.critical("The folder %s could not be created " % (fname))
            raise DirCreationException(str(ex))

    def remove_dir(self, fname):
        """ Remove a dir Folder. """
        try:
            os.rmdir(fname)
            logger.info("The folder %s was removed" % (fname))
        except os.error, ex:
            logger.critical("The folder %s could not be removed " % (fname))
            raise DirRemovalException(str(ex))
    
    def create_file(self, filename):
        """ Create a File"""
        try:
            _file = file(filename, 'w')
            logger.debug("The file %s was created" % (filename))
            _file.close()
        except exceptions.IOError, ex:
            logger.critical("The file %s could not be created " % (filename))
            raise FileCreationException(ex)

    def check_file(self, filename):
        """ Checks for the existence of a file. """
        logger.debug("Checking the existence of %s " % (filename))
        if os.path.isfile(filename):
            return True
        logger.critical("The filename %s does not exist " % (filename))
        return False

    def check_folder(self, folder):
        """ Checks for the existence of a Folder. """
        logger.debug("Checking the existence of %s " % (folder))
        if os.path.isdir(folder):
            return True
        logger.debug("The Folder %s does not exist " % (folder))
        return False

    def link_file_to(self, filename, linkname):
        """ Make symlink from filename to linkname
        """
        logger.info("Checking the existence of %s " % (filename))
        try:
            if not os.path.islink(linkname):
                if os.path.isdir(linkname):
                    filename = filename.split('/')[-1]
                    linkname = linkname + '/' + filename
                os.symlink(filename, linkname)
            else:
                logger.info("The link %s already exist" % (linkname))
        except os.error, ex:
            textto = ("The link %s to %s was not created " % \
                (filename, linkname))
            logger.critical(textto)
            textto = textto + str(ex)
            raise SymLinkException(textto)

    def missing_files(self, filelist):
        """ Given a list of filenames it returns a list with the missing files.
        """
        missing  = []

        for rqfile in filelist:
            if not self.check_file(rqfile):
                missing.append(rqfile)

        return missing

    def set_file_parameter(self, filename, parameter, value, uncomment=False):
        """ Set a parameter in a conf file """
        if filename is None or parameter is None or value is None:
            error = "Please check the following parameters: %s, %s, %s" % \
                    (filename,parameter,value)
            raise MissingParameterException(error)

        if self.check_file(filename):
            _fd = open(filename)
            file_contents = _fd.readlines()
            _fd.close()

            if uncomment:
                new_value_str = parameter + '=' + str(value)
            else:
                new_value_str = parameter + '=' + str(value) + \
                    '  #Modified by ICR Recipe Configurator'

            pattern = re.compile("^\s*" + parameter + "\s*=")

            _fd = open(filename, 'w')

            modified = False

            for line in file_contents:
                matched = pattern.match(line)

                if matched:
                    line = line.replace(line.rstrip('\n'), new_value_str)
                    modified = True

                _fd.write(line)

            if not modified:
                _fd.write(new_value_str + '\n')
            _fd.close()
        else:
            error = "The file %s does not exist or you don't have read access" \
                % filename
            raise FileNotFoundException(error)

    def get_file_parameter(self, filename, parameter):
        """ Get a parameter from a conf file """

        if filename is None:
            error = "File name is required"
            raise MissingParameterException(error)

        if parameter is None:
            error = "Parameter is required"
            raise MissingParameterException(error)

        ret_value = None
        matched = None

        if self.check_file(filename):
            _fd = open(filename)
            file_contents = _fd.readlines()
            _fd.close()

            # Compile the regular expression
            regex = str(parameter + "=" + "(.*)")
            pattern = re.compile(regex)

            for line in file_contents:
                matched = pattern.match(line)
                if matched:
                    ret_value = matched.group(1)
                    break
        else:
            error = "The file %s does not exist or you don't have read access" \
                % filename
            raise FileNotFoundException(error)

        return ret_value

    def unzip_file(self, file_name, dest):
        '''Unzips a zip file'''
        if not file_name:
            _msg = "File name not defined"
            raise UnzipException(_msg)
        try:
            ru.execute_command("unzip " + file_name + " -d " +\
                dest)
        except ru.CommandException, ex:
            raise UnzipException(str(ex))

    def zip_file(self, file_name, files, highcomp=None):
        """ Zip files in file_name. Optional highcomp=str(-9) """

        try:
            # Create a temporary file
            fname = tempfile.mkstemp()

        except exceptions.OSError, ex:
            if 'Errno 13' in str(ex):
                raise TempFileNotCreated(str(ex))
            raise OSALException(ex)

        try:
            # Open the file to store the filenames to zip
            _fd = open(fname[1],'w')

        except exceptions.IOError, ex:
            raise FileNotWritable(str(ex))


        try:
            # Write the files to be tarred
            for ufile in files:
                _fd.write(ufile + '\n')
            _fd.close()

        except exceptions.IOError, ex:
            raise FileNotWritable(str(ex))

        try:
            # Create the file
            if highcomp is None:
                ru.execute_command("zip -q %s -@ < %s " % (file_name, fname[1]))
            else:
                ru.execute_command("zip -q %s %s -@ < %s " % 
                                   (highcomp, file_name, fname[1]))

        except ru.CommandException, ex:
            raise UnzipException(str(ex))

        try:
            # Remove the temporary file
            os.remove(fname[1])

        except exceptions.OSError, ex:
            if 'Errno 2' in str(ex):
                raise FileNotFoundException(str(ex))
            if 'Errno 13' in str(ex):
                raise TempFileNotCreated(str(ex))
            raise OSALException(ex)

    def untar_file(self, file_name, dest, compress='z'):
        """Untar a tar ball file by default compressed with gzip"""
        if not file_name:
            _msg = "File name not defined"
            raise UntarException(_msg)

        try:
            ru.execute_command("tar -%sxvf " % compress + file_name + \
                                   " -C " + dest)
        except ru.CommandException, ex:
            raise UntarException(str(ex))

    def tar_file(self, file_name, files, compress='z'):
        """ Tar a ball file by default compressed with gzip"""

        try:
            # Create a temporary file
            fname = tempfile.mkstemp()

        except exceptions.OSError, ex:
            if 'Errno 13' in str(ex):
                raise TempFileNotCreated(str(ex))
            raise OSALException(ex)

        try:
            # Open the file to store the filenames to tar
            _fd = open(fname[1],'w')

        except exceptions.IOError, ex:
            raise FileNotWritable(str(ex))


        try:
            # Write the files to be tarred
            for ufile in files:
                _fd.write(ufile + '\n')
            _fd.close()

        except exceptions.IOError, ex:
            raise FileNotWritable(str(ex))

        try:
            # Create the file
            ru.execute_command('tar -%scf %s.tgz -T %s 2>/dev/null' %
                               (compress,file_name,fname[1]))

        except ru.CommandException, ex:
            raise UntarException(str(ex))

        try:
            # Remove the temporary file
            os.remove(fname[1])

        except exceptions.OSError, ex:
            if 'Errno 2' in str(ex):
                raise FileNotFoundException(str(ex))
            if 'Errno 13' in str(ex):
                raise TempFileNotCreated(str(ex))
            raise OSALException(ex)


    def get_home(self, user):
        """ Gets the user's home directory"""

        passwd_file = '/etc/passwd'
        home_index = 5
        user_index = 0

        try:
            _fd = open(passwd_file)
        except IOError, ex:
            raise OSALException("Could not open file %s\n" % passwd_file)
        
        try:
            passwd_content = _fd.read().split('\n')
        except exceptions.IOError, ex:
            _msg = "Could not read file %s: %s\n" % (passwd_file, str(ex))
            raise OSALException(_msg)

        _fd.close()

        passwd_home = None

        for line in passwd_content: 

            user_array = line.split(":")
            passwd_user = user_array[user_index]

            if user == passwd_user:
                # The user was defined in the line
                passwd_home = user_array[home_index]

        if passwd_home == None:
            raise UserDoesNotExists("User %s not present %s" % user) 

        return passwd_home

    def has_module_param(self, module, param):
        """ Gets the parameters for module and return a list with them """

        cmd = 'modinfo %s | grep ^parm | grep %s' % (module, param)
        try:
            ru.execute_command(cmd)
        except ru.CommandException:
            return 0

        return 1

    def get_module_version(self, module):
        """ Return module version """

        cmd = 'modinfo %s | grep ^version | awk \'{print $2}\'' % module
        try:
            output = ru.execute_command(cmd)
        except ru.CommandException:
            return None

        return output.pop()
    
    def make_os_developer(self):
        """Checks if the installation includes the development packages"""

        # Define the minimal list of packages to make it developer
        devel_packages = ["kernel-devel", "kernel-headers", "gcc",
                          "glibc-devel", "glibc-headers"]

        # Define the list of packages to install
        install_packages = []

        # If all packages are installed then the os can develop
        for package in devel_packages:
            if not self.has_package(package):
                install_packages.append(package)
        
        return install_packages

#############################################################################
#            Operating System detection                                     #
#############################################################################

_supported_operating_systems = list()


def get_os():
    """ Gets current installed OS by probing each registered OS.
    """

    opsys_it = iter(_supported_operating_systems)

    obj = None

    try:
        while obj is None:

            try:
                opsys = opsys_it.next()
                obj = opsys()
                logger.info('Operating system detected: ' + str(obj))
            except OSALException, exception:
                logger.debug(str(exception))

    except StopIteration:
        #Iteration ended
        raise OSNotFound()

    return obj


def register_os(operating_system):
    """ Registers the given operating system as a supported OS to be analyzed
        for its presence in the computer.
    """

    if not issubclass(operating_system, _OSAL):
        raise TypeError, "Invalid parameter"

    _supported_operating_systems.append(operating_system)


if __name__ == "__main__":
    print "This file should not be run alone"
