#!/usr/bin/env python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.

import exceptions
from logger import logger

#   The parameter list dictionary is conformed of a hash with the following form
#   { 
#       'Section name' :  {
#               'Parameter' : [
#                       'long opts',
#                       'Parameter descriptions',
#                       <Parameter is required?>
#               ]
#       }
#   }           
#   
#   For instance
#
#   { 
#       'Infiniband' :  {
#               'IB0_IP' : [
#                       'ib0_ip_address',
#                       'Configures the ip address for the ib0 interface',
#                       False
#               ]
#       }
#   }           
#
#
#
#

# Numerical constants

PARAM_ARRAY_LEN = 3

LONG_OPTS_IDX = 0   #Index for accessing the long opts variable of the 
                    #parameter array
DESC_IDX = 1        #Index for accessing the desc variable of the 
                    #parameter array
REQ_IDX = 2         #Index for accessing the required variable of the 
                    #parameter array

__version__ = "$Rev: 35275 $"

class ParameterListError (exceptions.Exception):
    """ Parameter list Errors """
    
    def __init__(self, str):
        exceptions.Exception.__init__(self)
        self.reason = str
    
    def __str__(self):
        return self.reason

class BadParameterList(ParameterListError):
    """ The parameter list is incorrectly formatted """

    def __init__(self):
        reason = "The parameter list was incorrectly formatted" 
        ParameterListError.__init__(self, reason)
        
    
class MissingSection(ParameterListError):
    """ The Section is missing """
    def __init__(self, section_name):
        self.section = section_name
        reason = "The section %s was not defined" % (self.section) 
        ParameterListError.__init__(self, reason)
        
class MissingParameter(ParameterListError):
    """ The Section is missing """
    def __init__(self, parameter):
        
        self.parameter = parameter
        reason = "The parameter %s was not defined" % (self.parameter) 
        ParameterListError.__init__(self, reason)
        
    
class DuplicatedLongOpts(ParameterListError):
    ''' Raised when a long option was duplicated in two different sections
        or the same section
    '''

    def __init__(self, duplicated_parameters_list):
        
        self.duplicated = duplicated_parameters_list
        reason = "Duplicated long options were found: %s" % str(self.duplicated)
        ParameterListError.__init__(self, reason)

    

class DuplicatedParameterMissmatch(ParameterListError):
    ''' Raised when a long option was duplicated in two different sections
        or the same section
    '''
    def __init__(self,param, first_definition_array, second_definition_array):
        ''' Accepts the parameter that is duplicated. The second argument
        is the array that was first defined, and the third argument is the
        third array that was defined'''
        exceptions.Exception.__init__(self)
        
        string1 = "The definition of the parameter %s is duplicated, "\
                  % param
        string2 = "it was defined as %s and later as %s\n"\
                  % (str(first_definition_array), str(second_definition_array))
        reason = string1 + string2
        ParameterListError.__init__(self, reason)
        
    
        

def _check_longopts_uniqueness( parameter_list):
    ''' Check that there are not two long options with the same name         

    '''

    if not isinstance(parameter_list, ParameterList):
        logger.error("Expected ParameterList type")
        raise exceptions.TypeError()
    
    param_list = parameter_list.get_all_longopts()

    sort_list = sorted(param_list)
    duplicated_list = []

    for i in range(len(sort_list)-1):
        if sort_list[i] == sort_list[i+1]:
            if sort_list[i] not in duplicated_list:
                duplicated_list.append(sort_list[i])

    if duplicated_list:
        raise DuplicatedLongOpts(duplicated_list)



class ParameterList:
    ''' Type that represents the parameter lists that the configuration
        requires '''

    def __init__(self, section=None, parameters_dict=None):       
        ''' Initializes the parameter list. The arguments should be the section
            to initilize and the dictionary of parameters as described in 
            check parameter.
        '''

        self.param_dict = {}

        if section is not None and parameters_dict is not None:

            self.add_section_parameters(section, parameters_dict)

        elif section is None and parameters_dict is None:
            #Initialize an empty Parameter List
            pass

        else:
            #Should never happen, if the section is defined the parameter_dict
            #must be defined
            raise exceptions.TypeError()



    def concat(self, parameter_list_dict):
        ''' Concatenates to parameter dictionaries 
        '''

        if not isinstance(parameter_list_dict, ParameterList):
            raise exceptions.TypeError()

        sections = parameter_list_dict.get_sections()

        for section in sections:
            self.add_section_parameters(section, 
            parameter_list_dict._get_section_parameters_dict(section))

    def _check_parameter_array(self, parameter_array):
        ''' Checks that the parameter array has the syntax described in the
        description
        '''
        if not isinstance(parameter_array, list):
            logger.warn("Parameter %s is not an array" % \
                (str(parameter_array)))
            raise BadParameterList()

        if len(parameter_array) != PARAM_ARRAY_LEN:
            logger.warn("Wrong number of elements in  %s" % \
                (str(parameter_array)))
            raise BadParameterList()

    def check_duplicated_parameters(self, section, parameters):
        ''' Check that duplicated parameters are defined exactly the same'''
        
        if section in self.param_dict:
            for param in parameters:
                if param in self.param_dict[section]:
                    for i in range(len(parameters[param])):
                        if self.param_dict[section][param][i] \
                            != parameters[param][i]:
                            raise DuplicatedParameterMissmatch(param, \
                                    self.param_dict[section][param], \
                                    parameters[param])
                        

    def add_section_parameters(self, section, parameters):
        '''Adds a section with its corresponding parameters. If the section
           was already present it will override old parameters

        '''
        if section in self.param_dict:
            
            # Validate that all the duplicated parameters in the section
            # are defined exactly the same
            self.check_duplicated_parameters(section, parameters)
        
        self._check_parameter_dict(parameters)
                
        for parameter in parameters:
            self._check_parameter_array(parameters[parameter])
            if section not in self.param_dict:
                self.param_dict[section] = { } 
            self.param_dict[section][parameter] = parameters[parameter]
        
        _check_longopts_uniqueness(self)

    def _get_section_parameters_dict(self, section):
        ''' Internal function for obtaining the parameters dictionary of the 
            section '''

        if section in self.param_dict:
            param_dict =  self.param_dict[section]
        else: 
            raise MissingSection(section) 

        return param_dict


    def get_section_parameters(self, section):
        ''' Gets the list of parameters for the specified section 
        '''
        if section not in self.param_dict:
            raise MissingSection(section)

        return self.param_dict[section].keys()

    def get_parameter_longopts(self, section, param):
        ''' Gets the parameters long options 
            Example 1 - Adding to sections and obtaining all the parameters

            >>> dictP = ParameterList('sectionA', \
                {'IB0_IP' : ['ib0_ip','ib0 infiniband ip address',False],\
                 'IB0_NM': [None, 'ib0 netmask address', False]})
            >>> dict2 = ParameterList('sectionB', \
                {'IB1_IP' : ['ib1_ip','ib1 infiniband ip address', False],\
                 'IB1_NM': ['ib1_netmask', 'ib1 netmask address', False]})
            >>> dictP.concat(dict2)
            >>> print dictP.get_parameter_longopts('sectionA', 'IB0_IP')
            ib0_ip

        '''

        long_opts = '' 

        if section not in self.param_dict:
            raise MissingSection(section)

        if param not in self.param_dict[section]:
            raise MissingParameter(param)
        
        long_opts = self.param_dict[section][param][LONG_OPTS_IDX]

        return long_opts 


    def get_section_longopts(self, section):
        ''' Gets the list of long options for the specified section 
            Example 1 - Adding to sections and obtaining all the parameters

            >>> dictP = ParameterList('sectionA', \
                {'IB0_IP' : ['ib0_ip','ib0 infiniband ip address',False],\
                 'IB0_NM': [None, 'ib0 netmask address', False]})
            >>> dict2 = ParameterList('sectionB', \
                {'IB1_IP' : ['ib1_ip','ib1 infiniband ip address', False],\
                 'IB1_NM': ['ib1_netmask', 'ib1 netmask address', False]})
            >>> dictP.concat(dict2)
            >>> print sorted(dictP.get_section_longopts('sectionA'))
            ['ib0_ip']

        '''

        long_opts = []

        if section not in self.param_dict:
            raise MissingSection(section)

        for param in self.param_dict[section].keys():
            if self.get_parameter_longopts(section, param):
                long_opts.append( self.get_parameter_longopts(section, param) )

        return long_opts 


    def get_all_longopts(self):
        ''' Gets the list of parameters for the specified section 
            Example 1 - Adding to sections and obtaining all the parameters

            >>> dictP = ParameterList('sectionA', \
                {'IB0_IP' : ['ib0_ip','ib0 infiniband ip address',False],\
                 'IB0_NM': [None, 'ib0 netmask address', False]})
            >>> dict2 = ParameterList('sectionB', \
                {'IB1_IP' : ['ib1_ip','ib1 infiniband ip address', False],\
                 'IB1_NM': ['ib1_netmask', 'ib1 netmask address', False]})
            >>> dictP.concat(dict2)
            >>> print sorted(dictP.get_all_longopts())
            ['ib0_ip', 'ib1_ip', 'ib1_netmask']

        '''

        long_opts = []

        for section in self.get_sections():
            for param in self.param_dict[section].keys():
                if self.get_parameter_longopts(section, param):
                    long_opts.append( self.get_parameter_longopts(section, \
                        param) )

        

        return long_opts 

    def get_parameter_description(self, section, param):
        ''' Gets the description for the specified section 
            Example 1 - Adding to sections and obtaining all the parameters

            >>> dictP = ParameterList('sectionA', \
                {'IB0_IP' : ['ib0_ip','ib0 infiniband ip address',False],\
                 'IB0_NM': [None, 'ib0 netmask address', False]})
            >>> dict2 = ParameterList('sectionB', \
                {'IB1_IP' : ['ib1_ip','ib1 infiniband ip address', False],\
                 'IB1_NM': ['ib1_netmask', 'ib1 netmask address', False]})
            >>> dictP.concat(dict2)
            >>> print dictP.get_parameter_description('sectionA', 'IB0_IP')
            ib0 infiniband ip address

        '''

        description = ''

        if section not in self.param_dict:
            raise MissingSection(section)

        if param not in self.param_dict[section]:
            raise MissingParameter(param)
        
        description = self.param_dict[section][param][DESC_IDX]

        return description 


    def is_parameter_required(self, section, param):
        ''' Return true if the parameter must be configured in the User 
            interface 
            Example 1 - Adding to sections and obtaining all the parameters

            >>> dictP = ParameterList('sectionA', \
                {'IB0_IP' : ['ib0_ip','ib0 infiniband ip address',False],\
                 'IB0_NM': [None, 'ib0 netmask address', False]})
            >>> dict2 = ParameterList('sectionB', \
                {'IB1_IP' : ['ib1_ip','ib1 infiniband ip address', False],\
                 'IB1_NM': ['ib1_netmask', 'ib1 netmask address', False]})
            >>> dictP.concat(dict2)
            >>> print dictP.is_parameter_required('sectionA', 'IB0_IP')
            False

        '''

        description = ''

        if section not in self.param_dict:
            raise MissingSection(section)

        if param not in self.param_dict[section]:
            raise MissingParameter(param)
        
        description = self.param_dict[section][param][REQ_IDX]

        return description 


    def get_sections(self):
        ''' Obtains the list of sections '''
        return self.param_dict.keys()

    def get_all_parameters(self):
        ''' Returns a list with all the parameter 
            
            Example 1 - Adding to sections and obtaining all the parameters

            >>> dictP = ParameterList('sectionA', \
                {'IB0_IP' : ['ib0_ip','ib0 infiniband ip address',False],\
                 'IB0_NM': [None, 'ib0 netmask address', False]})
            >>> dict2 = ParameterList('sectionB', \
                {'IB1_IP' : ['ib1_ip','ib1 infiniband ip address', False],\
                 'IB1_NM': ['ib1_netmask', 'ib1 netmask address', False]})
            >>> dictP.concat(dict2)
            >>> print sorted(dictP.get_all_parameters())
            ['IB0_IP', 'IB0_NM', 'IB1_IP', 'IB1_NM']

        '''

        new_list = []
        
        for section in sorted(self.get_sections()):
            new_list.extend(self.get_section_parameters(section))
            
        return new_list 


    def __str__(self):
        ''' Returns a string that represents the object '''
        ret_string = ""
        for section in sorted(self.get_sections()):
            ret_string += "%s\n" % (section)
            for parameter in sorted(self.get_section_parameters(section)):
                ret_string += "    %s\n" % (parameter)

        return ret_string.rstrip('\n')


    def _check_parameter_dict(self, parameter_dict):
        '''Checks that the parameter dictionary syntax is correct 
        The correct syntax is a dictionary with variable as keys and
        long options name as values. If no long option is accepted
        use None.
        {Variable Name : Long options commandline name} 
    
        So the parameter_list should have the following form:
        {'IB0_IP' : 'ib0_ip_address', 'IB0_NM': 'ib0_net_mask' ...}

        Example 1 - Correct parameter_dict
        >>> dictP = ParameterList('sectionA', \
            {'IB0_IP' : ['ib0_ip','ib0 infiniband ip address',False],\
            'IB0_NM': [None, 'ib0 netmask address', False]})
        >>> print dictP
        sectionA
            IB0_IP
            IB0_NM

        Example 2 - Incorrect parameter_dict
        >>> dict = ParameterList('sectionA', "parameterA, parameterB") 
        Traceback (most recent call last):
        ...
            raise BadParameterList()
        BadParameterList: The parameter list was incorrectly formatted

        '''

        if not isinstance(parameter_dict, dict):
            raise BadParameterList()








def _test():
    """ Inline doctest. """
    import doctest
    doctest.testmod()


if __name__ == "__main__":
    _test()


