""" This module adds support for the PCM for intel provisioning system. """
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.

# Python imports
import re
import exceptions
import os

# Custom imports
import common.ipaddr as ipaddr
import common.recipeutils as ru
import common.oslayer as oslayer
from common.logger import logger
from common.oslayer import FileCopyException
from common.provisioning import _Provisioning
from common.provisioning import ProvisioningException
from common.provisioning import ProvisioningNotFound
from common.provisioning import ProvisioningVersionNotFound
from common.provisioning import DatabaseError
from common.provisioning import ServiceError
from common.provisioning import PackageAddError


__version__ = "$Rev: 37503 $"

#############################################################################
#     Main PCM Class (all PCM classes should inherit from this one)         #
#############################################################################

class PCM(_Provisioning):
    """ Adds support for PCM (Intel version) provisioning system
        >>> from provisioning import register_provisioning
        >>> from provisioning import get_provision

        Example_1 - Register this provisioning
        >>> register_provisioning(PCM)

        Example_2 - Try to get the provisioning, expect raise an
                    exception because the provisioning could not be found
        >>> get_provision()
        Traceback (most recent call last):
        ...
            raise NoProvisioningFound()
        NoProvisioningFound: No supported provisioning system was detected
    """

    def __init__(self, name = None):
        self.version = ''
        self.build = ''
        self.opsys = None
        _Provisioning.__init__(self, name)
        return


    def probe(self):
        """ Probes if the provisioning is really PCM
        """
        raise NotImplementedError

    def __str__(self):
        return self.name + " version " + self.version + " Build " + self.build

    def _exec_sql(self, sql):
        """ Executes a query with sqlrunner using execute_command.
            The returned value is a list with the lines returned by sqlrunner.
        """
        cmd = 'sqlrunner -q \"%s\"' % sql
        try:
            return ru.execute_command(cmd)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise DatabaseError('Error executing command ' + cmd)

    def get_nodes(self):
        """
        Gets nodes list by using provisioning specific commands
        """
        return self._get_nodes_from_command('genconfig nodes')

    def get_profiles(self):
        """
        Gets profiles by using provisioning specific commands
        """
        return ru.execute_command('genconfig nodegroups')

    @property
    def node_installer_group(self):
        """ Gets the installer nodegroup """

        for prof in self.get_profiles():
            if prof.startswith('installer-'):
                return prof
        return None

    @property
    def compute_nodes_group(self):
        """ Gets the compute nodegroup """

        installer = self.node_installer_group

        if installer:
            compute = installer.replace('installer-', 'compute-')
            if compute in self.get_profiles():
                return compute
        return None

    def has_group(self, group):
        """ Returns true if the group is present in the provisioning system
        """
        if self.get_group_id(group) != None:
            hasgroup = True
        else:
            hasgroup = False
        return hasgroup
        

    def get_group_id(self, group):
        """ Gets the group id for a given group name."""

        cmd = "select ngid from nodegroups where ngname='%s'" % group

        try:
            num = self._exec_sql(cmd)[0]
            return int(num)
        except DatabaseError:
            return None

    def get_component_id(self, component):
        """ Gets the component id for a given group name."""

        cmd = "select cid from components where cname='%s'" % component

        try:
            num = self._exec_sql(cmd)[0]
            return int(num)
        except DatabaseError:
            return None

    def add_component_to_group(self, component_id, node_group_id):
        """ Adds the given component to the given group in the KUSU db. """
        cmd = "insert into ng_has_comp(ngid,cid) values(%s,%s)" % \
            (str(node_group_id), str(component_id))
        self._exec_sql(cmd)

    def get_networks(self):
        """ Gets networks information from the database """

        output = ru.execute_command('netedit -l')
        result = []

        for line in output:
            match = \
            re.match('(\d+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)',
                     line)

            if match:
                result.append( {'id': match.group(1),
                                'device': match.group(2),
                                'network': match.group(3),
                                'subnet': match.group(4),
                                'description': match.group(5),
                                'type': match.group(6),
                                } )
        return result

    def get_private_net(self):
        """ Gets the private network """
        for net in self.get_networks():
            if net.get('type') == 'provision':
                return net


    def get_interfaces(self):
        """ Gets networks interfaces from the database """
        
        nics = self._exec_sql('select * from nics')

        ifaces = []

        for nic in nics.split('|'):
                
            ifaces.append( {'subnet': '',
                                'device': '',
                                'mac': nic[3],
                                'ip': nic[4],
                                'module': '',
                                'name': ''
                                } )

        return ifaces


    def get_netdev_id(self, dev):
        """ Gets the network device id from the database """
        output = ru.execute_command('netedit -l')
        for line in output:
            if dev in line:
                return line.split()[0]
        return ''

    def add_device(self, dev, ip_addr=None, netmask=None, netname=None,
                   mac=None, module=None, name=None):
        """ Adds a device to the database.
            Parameters:
            dev      -- network device, e.g. ib0, eth1
            ip_addr  -- ip address for the device
            netmask  -- subnet mask for the device
            netname  -- netname needed for rocks.
            mac      -- MAC address
            module   -- kernel module used
            name     -- hostname for the device
        """

        if not ip_addr or not netmask or not netname:
            raise ValueError('No ip_addr, netmask or netname values ' \
            'provided (ip_addr=%s, netmask=%s, netname=%s)' % \
            (str(ip_addr), str(netmask), str(netname)))

        installer_group = self.node_installer_group

        if not installer_group:
            logger.critical("Provisioning instance has no installer group " \
                            "set")
            raise Exception("Provisioning instance has no installer group " \
                            "set")

        # Adding the network to the installer nodegroup

        # Retrieves the Net id of the infiniband network
        netid_n = self.get_netdev_id(dev)

        # Retrieves the group id of the installer node
        cmd = "select ngid from nodegroups where ngname='%s'" % installer_group
        ngid_n = self._exec_sql(cmd)[0]

        if netid_n not in ('0', '') and ngid_n not in ('0', ''):
            # Associate the infiniband network with the installer node group
            logger.info("Adding %s network to Installer node group" % dev)

            cmd = "insert into ng_has_net(ngid,netid) values (%s,%s);" % \
            (ngid_n,netid_n)

            self._exec_sql(cmd)

        else:
            if ngid_n in ('0', ''):
                raise ProvisioningException("Unexpected error, " \
                                "Installer nodegroup not available")
            else:
                raise ProvisioningException("Unexpected error, " \
                                "%s network not created" % dev)

        # Retrieves the group id of the installer node
        cmd = "select ngid from nodegroups where ngname='%s'" % installer_group
        ngid_n = self._exec_sql(cmd)[0]

        if netid_n not in ('0', '') and ngid_n not in ('0', ''):
            # Add the head node's ip address to the infiniband network
            logger.info("Adding head node's ip to %s network" % dev)
            cmd = "insert into nics(nid,netid,ip,boot) values " \
                  "(1,%s,\\\"%s\\\",true)" % (netid_n, ip_addr)

            self._exec_sql(cmd)
        else:
            if ngid_n in ('0', ''):
                raise ProvisioningException("Unexpected error, " \
                                "Installer nodegroup not available")
            else:
                raise ProvisioningException("Unexpected error, " \
                                "%s network not created" % dev)

    def edit_device(self, dev, ip_addr=None, netmask=None, netname=None,
                    mac=None, module=None, name=None):
        """ Edits the device information in the database """

    def remove_device(self, dev):
        """ Removes a device from the database """

    def configure_device(self, *args, **kwargs):
        """ Configures a network device.
            Just calls the OS abstraction layer method with the same params.
        """
        try:
            self.opsys.configure_device(*args, **kwargs)
        except Exception, ex:
            logger.debug(ex)
            raise ProvisioningException('Error configuring network device')

    def add_network(self, name, ip_addr, netmask, device, subnet='', desc='',
                    suffix=''):
        """ Adds a network to the database """
        net = ipaddr.IPv4Network('%s/%s' % (ip_addr, netmask))

        if not subnet:
            subnet = net.network.exploded

        next_ip = ipaddr.IPv4Address(int(ipaddr.IPv4Address(ip_addr)) + 1)

        if next_ip not in net:
            raise ValueError('Computed start ip %s is not in network %s' % \
            (next_ip.exploded, net.exploded))

        startip = next_ip.exploded

        if not self.opsys:
            logger.critical("Provisioning instance has no associated OS")
            raise Exception("Provisioning instance has no associated OS")

        installer_group = self.node_installer_group

        if not installer_group:
            logger.critical("Provisioning instance has no installer group " \
                            "set")
            raise Exception("Provisioning instance has no installer group " \
                            "set")

        # Create the net in kusu databases for the infiniband network.
        logger.info("Creating kusu %s network with net %s and netmask %s" % \
            (device, subnet, netmask))

        if desc == '':
            if re.match('ib\d+$', device):
                desc = 'Infiniband Net'
            else:
                raise ValueError('No desc value provided or inferred')

        if suffix == '':
            suffix = '-' + device

        cmd = 'netedit -a -n %s -s %s -i %s -t %s -e "%s" -x "%s" -z' % \
        (subnet, netmask, device, startip, desc, suffix)

        try:
            ru.execute_command(cmd)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise ProvisioningException("Error executing command " + cmd)

    def ib_dev_configured(self, dev, ip_addr, netmask):
        """ Checks if ib device 'dev' is already configured with ip 'ip_addr'
            and netmask 'netmask'
        """

        ib0_info = self.opsys.device_info(dev)

        # Check if is up and with the specified ip and netmask
        if ib0_info.get('inet addr') != ip_addr or \
            ib0_info.get('Mask') != netmask:
            return False 

        net = ipaddr.IPv4Network('%s/%s' % (ip_addr, netmask))

        # Check if is stored in database
        for net_info in self.get_networks():
            net_dev = net_info.get('device')

            if net_dev == dev:
                net_ip = net_info.get('network')
                mask_ip = net_info.get('subnet')
                if net_ip == net.network.exploded and mask_ip == netmask:
                    return True

        return False

        
    def link_network(self, network_device, node_group):
        """ Links a network device to a node group """

        # Adding the network to the compute 

        
        if not isinstance(network_device, str):
            raise TypeError("Invalid network device")

        if not isinstance(node_group, str):
            raise TypeError("Invalid node group")

        # Retrieves the Net id of the infiniband network
        netid_n = self.get_netdev_id(network_device)
        
        # Retrieves the group id of the node group
        cmd = "select ngid from nodegroups where ngname='%s'" % node_group
        ngid_n = self._exec_sql(cmd)[0]

        if netid_n not in ('0', '') and ngid_n not in ('0', ''):
            # Associate the infiniband network with the node group
            cmd = "insert into ng_has_net(ngid,netid) values (%s,%s);" % \
            (ngid_n,netid_n)

            self._exec_sql(cmd)

            logger.info("Adding %s network to %s group" % (network_device\
                , node_group))

        else:
            if ngid_n in ('0', ''):
                raise Exception("Unexpected error, " \
                                "%s group not available" % node_group)
            else:
                raise Exception("Unexpected error, " \
                                "%s network not created" % network_device)

    def has_network_device(self, network_device):
        """ Returns True if the network device is enabled """

        netid_n = self.get_netdev_id(network_device)
        
        if netid_n in ('0', ''):
            has_net = False
        else:
            has_net = True
        
        return has_net    
         

    def setup_ib_files(self):
        """ Configures the necessary OS files for InfiniBand
        """
        if not self.opsys:
            logger.critical("Provisioning instance has no associated OS")
            raise Exception("Provisioning instance has no associated OS")

        #Disable IPoIB connected mode
        self.opsys.set_file_parameter(self.opsys.OPENIB_FILE, 'SET_IPOIB_CM','no')
        #Disable RDMA_CM module
        self.opsys.set_file_parameter(self.opsys.OPENIB_FILE, 'RDMA_CM_LOAD','no')

    def setup_ib_services(self):
        """ Configures, starts, stops, restarts InfiniBand and network
            services as needed for configuration.
        """
        if not self.opsys:
            logger.critical("Provisioning instance has no associated OS")
            raise Exception("Provisioning instance has no associated OS")

        try:
            if self.opsys.service_running(self.opsys.ib_opensm_service):
                self.opsys.service_stop(self.opsys.ib_opensm_service)
        except oslayer.ServiceError, ex:
            logger.debug(ex)
            msg = 'Error stopping service %s' % self.opsys.ib_opensm_service
            raise ServiceError(msg)
            
        try:
            self.opsys.service_restart(self.opsys.ib_openib_service)
        except oslayer.ServiceError, ex:
            logger.debug(str(ex))
            msg = 'Error restarting service %s' % self.opsys.ib_openib_service
            raise ServiceError(msg)

        try:
            self.opsys.service_enable(self.opsys.ib_opensm_service)
        except Exception, ex:
            raise ServiceError(ex)

        # Verify that openib could be loaded successfully
        if self.opsys.openibd_status() != 'loaded':
            msg = "Service %s could not be started" % self.opsys.ib_openib_service
            raise ServiceError(msg)

        try:
            self.opsys.service_restart(self.opsys.ib_opensm_service)
        except oslayer.ServiceError, ex:
            logger.debug(ex)
            msg = 'Error restarting service %s' % self.opsys.ib_opensm_service
            raise ServiceError(msg)

        # Verify that opensmd could be loaded
        if not self.opsys.service_running(self.opsys.ib_opensm_service):
            msg = "Service %s could not be started" % self.opsys.ib_opensm_service
            raise ServiceError(str(msg))

        try:
            self.opsys.service_restart('network')
        except oslayer.ServiceError, ex:
            logger.debug(ex)
            raise ServiceError('Error restarting network service')

    def sync_network(self, *args, **kwargs):
        """ Synchronizes the network configuration """
        self.sync(mode='config')

    def bring_up_device(self, device):
        """ ifup device """

        cmd = "ifup " + device
        try:
            ru.execute_command(cmd)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise ProvisioningException('Error bringing up ' + device)

    def network_configured(self, netname):
        """ Returns True if the network for current host has been added to
            the provisioning system database.
        """
        return True


    def sync(self, nodegroups=None, mode='complete'):
        """ Syncs the provisioning system database. It will do a complete sync
            on all nodes by default. Providing a list of nodegroups will sync
            only those nodegroups. If mode is different from complete it will
            only sync configuration files, and no packages.
        """
        command = 'cfmsync -f'

        if mode == 'complete':
            command += ' -p'

        if isinstance(nodegroups, basestring):
            nodegroups = [nodegroups]
        elif nodegroups != None and not isinstance(nodegroups, list):
            raise ValueError('nodegroups must be string, list or None. ' \
                             'Got %s' % str(type(nodegroups)))

        if nodegroups:
            for ngrp in nodegroups:
                ru.execute_command('%s -n "%s"' % (command, ngrp))
        else:
            ru.execute_command(command)

    def component_installed(self, comp):
        """ Checks if the given component is installed. """

        cmd = "select count(*) from components,ng_has_comp,nodegroups where " \
              "components.cid=ng_has_comp.cid and " \
              "ng_has_comp.ngid = nodegroups.ngid and components.cname like " \
              "'%%%s' and nodegroups.ngname = '%s'" % \
              (comp, self.node_installer_group)

        out = self._exec_sql(cmd)

        if out[0].strip() == '0':
            cmd = "select count(*) from components where " \
                  "components.cname like '%%%s'" % comp
            out = self._exec_sql(cmd)
            if out[0].strip() == '0':
                return False

        return True

    def add_script(self, script, nodegroup):
        """ This method add a script into a node group """
    
        scriptdir = '/depot/repos/custom_scripts/'

        nodegroupid = self.get_group_id(nodegroup)

        if nodegroupid is None:
            raise ProvisioningException("Could not get id for nodegroup " + \
                                        str(nodegroup))
       
        # If it is the installer group, besides adding the script to the
        # database it should be executed
        if nodegroup == self.node_installer_group: 
            self._execute_head_script(script)

        script_filename = script.split(os.path.sep)[-1]
        
        if not self._has_script(nodegroupid, script_filename):
            self._exec_sql("insert into scripts(ngid, script) values('%s','%s');"
                       % (str(nodegroupid), str(script_filename)))

        try:
            self.opsys.safe_cp(script, scriptdir)
        except FileCopyException, ex:
            raise ProvisioningException(str(ex))

        dest_script = "%s/%s" % (scriptdir, script_filename)

        try:
            scriptfile = open(dest_script, "a")
            scriptfile.write("\nrm $0\n")
            scriptfile.close()
        except IOError, ex:
            raise ProvisioningException("Could not update post install"\
                + " scripts: %s" % (str(ex)))
        
        os.system('chmod a+r %s' % dest_script)
        os.system('chmod a+x %s' % dest_script)
    
    def _has_script(self, nodegroupid, script_filename):
        """Method to check the existence of a script in the database"""
        
        cmd = "select count(*) from scripts where " + \
              "ngid='%s' and script like '%%%s'" % \
              (nodegroupid, script_filename)
        out = self._exec_sql(cmd)
        if out[0].strip() == '0':
            has_script = False
        else:
            has_script = True

        return has_script


    def _execute_head_script(self, script):
        """ Executes a script in the Head Node """

        logger.info('Executing ' + script + ' in Head Node')

        if not os.path.isfile(script):
            raise ProvisioningException('Could not locate file %s' \
                    % script)

        os.system('chmod +x %s' % script)

        try:
            ru.execute_command(script)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise ProvisioningException('Error executing %s.' % script)


    def add_package(self, nodegroup, packagename):
        """ This method add a package to a nodegroup"""
        ngid = self.get_group_id(nodegroup)
        packagename = str(packagename)
        
        # If package contains the architecture it is extracted 

        architectures = [".i386", ".i586", ".i686", ".i486", ".x86_64", ".noarch"]
                 
        for arch in architectures:
            if re.search(arch, str(packagename)):
                logger.debug("Package name %s contained the architecture"
                           % (packagename))
                packagename = packagename.split(arch, 1)[0]
        
        if not self.opsys.has_package_in_repo(packagename):
            raise PackageAddError(nodegroup, packagename)
       
        already_installed = self._exec_sql("select ngid from packages where "
                                           "packagename='%s' and ngid='%s';"
                                           % (packagename, str(ngid)))
        logger.info("Trying to add package \"%s\" to nodegroup \"%s\"" 
                    % (packagename, nodegroup))
        if already_installed == ['']:
            try:
                self._exec_sql("insert into packages(ngid, packagename) "
                               "values('%s','%s');"
                               % (str(ngid), packagename))
                logger.info("Package added successfully")
            except:
                error_msg = "Error: an unknown error occurred trying to add %s"\
                             % packagename
                logger.debug(error_msg)
                raise PackageAddError(nodegroup, packagename)
        else:
            logger.info("Warning: The package has already been added.")

    def copy_nodegroup(self, ng_orig, ng_dest):
        """ This method duplicate a nodegroup """
        ru.execute_command('ngedit -c %s -n %s' % (ng_orig, ng_dest))

    def init_script(self, init_script):
        """ This method run KUSU custom init scripts"""
        ru.execute_command('kusurc %s' % init_script)

    def ng_has_comp(self, ngid, cid):
        """ This method checks if a given component is added into a nodegroup.
        """
        has_comp = self._exec_sql("select * from ng_has_comp where "\
                                "cid='%s' and ngid='%s'" % (cid, ngid))
        if has_comp == ['']:
            return False
        else:
            return True

    def add_package_to_repo(self, package_file):
        """ Adds an rpm file package to the repository """
       
        repo_folder = "/depot/contrib/1000/"

        if not os.path.isdir(repo_folder):
            try:
                ru.execute_command('mkdir -p %s' % repo_folder)
            except:
                pass

        if isinstance(package_file, str) and\
            self.opsys.check_file(package_file):
           
            try:
                ru.execute_command("cp " + package_file + " " + repo_folder)
            except ru.CommandException, ex:
                logger.debug(str(ex))
                raise ProvisioningException("Could not add package " + 
                    package_file + " to repository")

    def _get_repo_name(self):
        """ Gets the name of the repositories """
        reponame = None
        try:
            cmd = "repoman -l | grep \"Repo name\" | awk -F\":\" {'print $2'}"\
                + " | awk -F\" \" {'print $1'}"

            reponame = ru.execute_command(cmd)

        except ru.CommandException:
            pass

        return reponame[0]

    def update_repo(self):
        """ Updates the packages repository """

        repo_name = self._get_repo_name()
        
        if not repo_name:
            
            raise ProvisioningException("Could not find repository")

        try:
            cmd = "repoman -u -r %s" % repo_name
            ru.execute_command(cmd)
        except ru.CommandException:
            raise ProvisioningException("Could not update the repository") 
           
        try:
            ru.execute_command("yum makecache")
        except ru.CommandException:
            raise ProvisioningException("Could not update the yum cache")
                                                     
                    
    def set_bkm_ethernet(self, drivers, modprobefile):
        """ Adds the parameter option to the driver in the modprobe file """

        param = "InterruptThrottleRate=0,3"
        for drv in drivers:
            must_add = True

            for line in open(modprobefile):
                if line.strip() == "options %s %s" % (drv, param):
                    must_add = False
                    logger.info("%s already configured for %s." % \
                    (modprobefile, drv))
                    break

            if must_add:
                mpfile = open(modprobefile, 'a')
                mpfile.write("#Line added by Intel(R) Cluster Ready Reference"+\
                             " Recipe scripts\n")
                mpfile.write("options %s %s\n" % (drv, param))
                mpfile.close()
                logger.info("%s configured for %s." % \
                (modprobefile, drv))

    def fix_10gb_order(self, ifaces_10gb):
        """ Attempts to fix the 10gb interfaces order """

    def add_sshkeys(self, user_name):
        """ Add SSH keys for a user """
        logger.info("Generating SSH keys...")
        try:
            ru.execute_command("su - %s -c exit" % (user_name))
        except ru.CommandException, ex:
            logger.critical("Failed creating SSH keys")
            logger.critical("ERROR: %s" % (ex))
            raise ru.CommandException(ex)

#############################################################################
#     PCM 1.2                                                               #
#############################################################################

class PCM12(PCM):
    """ Adds support for PCM 1.2
    """

    def __init__(self):
        self.version = ''
        self.build = ''
        self.opsys = None
        self.file_name = '/etc/kusu-release'
        PCM.__init__(self, "Platform Cluster Manager")
        return


    def probe(self):
        """ Probes if the provisioning is really PCM 1.2
        """

        # Pattern for parsing the following string:
        # Kusu "Platform Cluster Manager for Intel" 1.2  (build 2357)

        prov_pattern = '^Kusu "Platform Cluster Manager for Intel"\s+' \
            '(\d+.\d+)\s+\(build (\d+)\)'

        try:
            fileh = open(self.file_name, 'r')
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise ProvisioningNotFound(self.name)

        try:
            file_contents = fileh.readline()
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise ProvisioningVersionNotFound(self.name)

        matchs = re.match(prov_pattern, file_contents)
        if matchs is None:
            logger.debug ('The expected string was not found in ' + self.file_name)
            raise ProvisioningVersionNotFound(self.name)

        self.version = matchs.group(1)
        self.build = matchs.group(2)

        return

#############################################################################
#     PCM 2.0                                                               #
#############################################################################

class PCM20(PCM):
    """ Adds support for PCM 2.0
    """

    def __init__(self):
        self.version = ''
        self.build = ''
        self.opsys = None
        self.file_name = '/etc/kusu-release'
        PCM.__init__(self, "Platform Cluster Manager")
        return


    def probe(self):
        """ Probes if the provisioning is really PCM 2.0
        """

        # Pattern for parsing the following string:
        # Kusu "Platform Cluster Manager" 2.0 (build 3629)

        prov_pattern = '^Kusu "Platform Cluster Manager"\s+' \
                       '(\d+.\d+)\s+\(build (\d+)\)'

        try:
            fileh = open(self.file_name, 'r')
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise ProvisioningNotFound(self.name)

        try:
            file_contents = fileh.readline()
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise ProvisioningVersionNotFound(self.name)

        matchs = re.match(prov_pattern, file_contents)
        if matchs is None:
            logger.debug ('The expected string was not found in ' + self.file_name)
            raise ProvisioningVersionNotFound(self.name)

        self.version = matchs.group(1)
        self.build = matchs.group(2)

        return


#############################################################################
#     PCM 2.1                                                               #
#############################################################################

class PCM21(PCM):
    """ Adds support for PCM 2.1
    """

    def __init__(self):
        self.version = ''
        self.build = ''
        self.opsys = None
        self.file_name = '/etc/hpc-release'
        PCM.__init__(self, "Platform HPC")
        return


    def probe(self):
        """ Probes if the provisioning is really PCM 2.1
        """

        # Pattern for parsing the following strings:
        # "Platform HPC" 2.1 (build 4613)

        prov_pattern = '^"Platform HPC" (2.1) \(build (4613)\)'

        try:
            fileh = open(self.file_name, 'r')
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise ProvisioningNotFound(self.name)

        try:
            file_contents = fileh.readline()
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise ProvisioningVersionNotFound(self.name)

        matchs = re.match(prov_pattern, file_contents)
        if matchs is None:
            logger.debug ('The expected string was not found in ' + self.file_name)
            raise ProvisioningVersionNotFound(self.name)

        self.version = matchs.group(1)
        self.build = matchs.group(2)

        return

#############################################################################
#     PCM 3.0                                                               #
#############################################################################

class PCM30(PCM):
    """ Adds support for PCM 3.0
    """

    def __init__(self):
        self.version = ''
        self.build = ''
        self.opsys = None
        self.file_name = '/etc/hpc-release'
        PCM.__init__(self, "Platform HPC")
        return


    def probe(self):
        """ Probes if the provisioning is really PCM 3.0
        """

        # Pattern for parsing the following strings:
        # "Platform HPC" 3.0 (build 5388)

        prov_pattern = '^"Platform HPC" (3.0) \(build (5388)\)'

        try:
            fileh = open(self.file_name, 'r')
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise ProvisioningNotFound(self.name)

        try:
            file_contents = fileh.readline()
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise ProvisioningVersionNotFound(self.name)

        matchs = re.match(prov_pattern, file_contents)
        if matchs is None:
            logger.debug ('The expected string was not found in ' + self.file_name)
            raise ProvisioningVersionNotFound(self.name)

        # Due from version 2.0 PCM is a component inside Platform HPC we
        # validate the version of Platform HPC with the build and here we
        # need to hard-code the version because is not the same as the PCM
        self.version = matchs.group(1)
        self.build = matchs.group(2)

        return

#############################################################################
#     PCM 3.0.1                                                               #
#############################################################################

class PCM301(PCM):
    """ Adds support for PCM 3.0.1
    """

    def __init__(self):
        self.version = ''
        self.build = ''
        self.opsys = None
        self.file_name = '/etc/hpc-release'
        PCM.__init__(self, "Platform HPC")
        return


    def probe(self):
        """ Probes if the provisioning is really PCM 3.0.1
        """

        # Pattern for parsing the following strings:
        # "Platform HPC" 3.0.1 (build <build version>)

        prov_pattern = '^"Platform HPC" (3.0.1) \(build (\d+)\)'

        try:
            fileh = open(self.file_name, 'r')
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise ProvisioningNotFound(self.name)

        try:
            file_contents = fileh.readline()
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise ProvisioningVersionNotFound(self.name)

        matchs = re.match(prov_pattern, file_contents)
        if matchs is None:
            logger.debug ('The expected string was not found in ' + self.file_name)
            raise ProvisioningVersionNotFound(self.name)

        self.version = matchs.group(1)
        self.build = matchs.group(2)

        return

def _test():
    """ Inline doctest. """
    import doctest
    doctest.testmod()


if __name__ == "__main__":
    _test()
