#!/usr/bin/python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.


""" Provisioning system abstraction base class and common functions. """

# System Modules
import exceptions

# Internal Modules
from logger import logger
from oslayer import _OSAL
import ipaddr
import recipeutils

__version__ = "$Rev: 37503 $"

##############################################################################
#         Exception Classes                                                  #
##############################################################################

class ProvisioningException(exceptions.Exception):
    """ Exceptions for the Provisioning systems base class. """ 
    def __init__(self, reason):
        self.reason = reason
        return
      
    def __str__(self):
        return self.reason


class ProvisioningNotFound(ProvisioningException):
    """ Exception raised when a specific provisioning system could not be
        found in the local system.
    """ 

    def __init__(self, name):
        ProvisioningException.__init__(self, "The provisioning " + name + \
                                              " was not detected")
        return


class ProvisioningVersionNotFound(ProvisioningException):
    """ Exception raised when the version of an specific provisioning system
        could not be found in the local system.
    """ 

    def __init__(self, name):
        ProvisioningException.__init__(self,
                                        "The provisioning version for " + \
                                        name + " could not be found")
        return

class NoProvisioningFound(ProvisioningException):
    """ Exception raised when no supported provisioning system could be
        located in the system.
    """
    
    def __init__(self):
        ProvisioningException.__init__(self, "No supported provisioning "\
                                              "system was detected")
        return

class NodeListNotObtained(ProvisioningException):
    """ Exception raised when no supported provisioning system could be
        located in the system.
    """
    
    def __init__(self):
        ProvisioningException.__init__(self, "Could not obtain the list of "\
                                              "nodes of the cluster")
        return

class ServiceError(ProvisioningException):
    """ Exception raised if the provisioning layer has a problem managing
        a service.
    """

class DatabaseError(ProvisioningException):
    """ Exception raised if there is an error managing the provisioning
        database.
    """

class DeviceExistsError(ProvisioningException):
    """ Exception raised when a device tried to be added to the database
        already exist.
    """


class PackageError(ProvisioningException):
    """ Exception to be raised when there is problem handling packages """

    def __init__(self, desc):
        ProvisioningException.__init__(self, desc)
    

class PackageAddError(PackageError):
    """ Exception raised when there was an error installing a package """
    
    def __init__(self, group, packagename):
        error_desc = "Error adding package \"%s\" to nodegroup \"%s\"" % (packagename, group)

        PackageError.__init__(self, error_desc)
     

#############################################################################
#         Provisioning Classes                                              #
#############################################################################

class _Provisioning:
    """ This class is a pure virtual class that is only intended to be 
        an interface for the differente provisioning abstractions.
    """

    def __init__(self, name = None):
        if name is None:
            # If the name was not already provided the name will be the 
            # name of the class
            self.name = self.__class__.__name__
        else:
            self.name = name

        self.opsys = None
        self.version = None

        logger.debug('Probing for provisioning system ' + self.name + \
                    ' presence')
        self.probe()
        
        if self.version is None:
            self.version = "Undetermined"
        return        
    
    def __str__(self):
        return self.name + " version " + self.version

    def _get_nodes_from_command(self, nodelist_command):
        """ This method gets a nodelist from a command passed by parameter.
            The output of the command should be a list of nodes, each node in
            one line.
        """

        try:
            nodelist = recipeutils.execute_command(nodelist_command)
        except recipeutils.CommandException:
            raise NodeListNotObtained()            

        return nodelist 

    def associate_os(self, opsys):
        """ Associates the given OS with this provisioning system. """
        if not isinstance(opsys, _OSAL):
            raise ValueError("%s is not an Operating System Abstraction " \
                             "Layer object." % str(opsys))
        self.opsys = opsys

    def probe(self):
        """ Probes the provisioning system installed """
        raise NotImplementedError

    def get_nodes(self):
        """ Gets nodes list by using provisioning specific commands """
        raise NotImplementedError

    def get_profiles(self):
        """ Gets profiles by using provisioning specific commands """
        raise NotImplementedError

    def get_networks(self):
        """ Gets networks information from the database """
        raise NotImplementedError

    def get_interfaces(self):
        """ Gets networks interfaces from the database """
        raise NotImplementedError

    def get_private_net(self):
        """ Gets the private network """
        raise NotImplementedError

    def add_network(self, name, ip_addr, netmask, device, subnet='', desc='',
                    suffix=''):
        """ Adds a network to the database """
        raise NotImplementedError

    def add_device(self, dev, ip_addr=None, netmask=None, netname=None,
                   mac=None, module=None, name=None):
        """ Adds a device to the database. """
        raise NotImplementedError

    def remove_device(self, dev):
        """ Removes a device from the database """
        raise NotImplementedError

    def edit_device(self, dev, ip_addr=None, netmask=None, netname=None,
                    mac=None, module=None, name=None):
        """ Edits the device information in the database """
        raise NotImplementedError

    def configure_device(self, *args, **kwargs):
        """ Configures a network device. """
        raise NotImplementedError

    def ib_dev_configured(self, dev, ip_addr, netmask):
        """ Checks if ib device 'dev' is already configured with ip 'ip_addr'
            and netmask 'netmask'
        """
        raise NotImplementedError

    def setup_ib_files(self):
        """ Configures the necessary OS files for InfiniBand.
            Implemented by derived classes if necessary.
        """
        raise NotImplementedError

    def setup_ib_services(self):
        """ Configures, starts, stops, restarts InfiniBand and network
            services as needed for configuration.
            Implemented by derived classes if necessary.
        """
        raise NotImplementedError

    def network_configured(self, netname):
        """ Returns True if the network for current host has been added to
            the provisioning system database.
        """
        raise NotImplementedError

    def bring_up_device(self, device):
        """ Brings up a device """
        raise NotImplementedError

    def has_network_device(self, network_device):
        """ Returns True if the network device is enabled """
        raise NotImplementedError

    def link_network(self, network_device, node_group):
        """ Links a network device to a node group """
        raise NotImplementedError

    def sync_network(self, *args, **kwargs):
        """ Synchronizes the network configuration """
        raise NotImplementedError

    def sync(self, *args, **kwargs):
        """ Synchronizes the configuration """
        raise NotImplementedError

    def print_nodes(self):
        """ Prints the nodes found by get_nodes, one by line. """
        print '\n'.join(self.get_nodes())

    def component_installed(self, comp):
        """ Checks if the given component is installed. """
        raise NotImplementedError

    def add_script(self, *args, **kwargs):
        """ Abstract class to add scripts to be executed on nodes """
        raise NotImplementedError

    def copy_nodegroup(self, orig, copy):
        """ Abstract class to duplicate nodegroups """
        raise NotImplementedError

    def set_bkm_ethernet(self, drivers, modprobefile):
        """ Adds the parameter option to the driver in the modprobe file """
        raise NotImplementedError

    def fix_10gb_order(self, ifaces_10gb):
        """ Attempts to fix the 10gb interfaces order """
        raise NotImplementedError

    def has_group(self, group):
        """ validates if the node group is present in provisioning """
        raise NotImplementedError

    def get_network_from_ip(self, ip_addr):
        """ Returns the information of the network this ip belongs to.
            If it does not belong to any network registered by the
            provisioning it returns None
        """
        for net_info in self.get_networks():
            net_ip = net_info.get('network')
            mask_ip = net_info.get('subnet')

            net = ipaddr.IPv4Network('%s/%s' % (net_ip, mask_ip))
            ip = ipaddr.IPv4Address(ip_addr)

            if ip in net:
                return net_info

        return None

    def get_network_from_device(self, device):
        """ Returns the information of the network this device belongs to.
            If it does not belong to any network registered by the
            provisioning it returns None
        """
        for net_info in self.get_networks():
            if device == net_info.get('device'):
                return net_info

        return None

    def rebuild_images(self):
        """ Rebuilds the distro for including new rpms """
        raise NotImplementedError

#############################################################################
#         Provisioning System detection                                     #
#############################################################################

_supported_provisionings = list()


def get_provision():
    """ Returns a detected provisioning system by probing the registered
        supported provisioning systems.
    """

    provisioning_it = iter(_supported_provisionings)

    obj = None

    try: 
        while obj is None:

            try:
                provisioning = provisioning_it.next()
                obj = provisioning()
                logger.info('Provisioning system detected: ' + str(obj))
            except ProvisioningException, exception:
                logger.debug(str(exception))

    except StopIteration:
        #Iteration ended
        raise NoProvisioningFound()

    return obj

        
def register_provisioning(provisioning_system):
    """
    Register a new provisioning system.
    """
    if not issubclass(provisioning_system, _Provisioning):
        raise TypeError, "Invalid parameter"

    _supported_provisionings.append(provisioning_system)


def _test():
    """ Inline doctest. """
    import doctest
    doctest.testmod()


if __name__ == "__main__":
    _test()

