#!/usr/bin/python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.


""" Common functions and other utilities used by most scripts """

import commands
import exceptions
import ipaddr
import os
import re

# Internal Modules
from logger import logger

__version__ = "$Rev: 35275 $"

text_colors = { 'red': '\033[31m',
                'green': '\033[32m',
                'yellow': '\033[33m',
                'blue': '\033[34m',
                'pink': '\033[35m',
                'cyan': '\033[36m',
                'white': '\033[37m'}

##############################################################################
#         Exception Classes                                                  #
##############################################################################

class CommandException(exceptions.Exception):
    '''Exceptions that occurs for errors during command execution ''' 
    def __init__(self, status, reason):
        exceptions.Exception.__init__(self)
        self.status = status
        self.reason = reason
        return
      
    def __str__(self):
        return self.reason

class MultiPageException(exceptions.Exception):
    """ This exception occurs when multipage function could not show the file
    """
    def __init__(self, value):
	self.value = value
	exceptions.Exception.__init__(self)
        
    def __str__(self):
        return repr(self.value)

class SearchException(exceptions.Exception):
    """This exception occurs when there is an error searching a pattern in a 
    file"""
    def __init__(self, reason):
       self.reason = reason
       exceptions.Exception.__init__(self)

    def __str__(self):
        return repr(self.reason)
	
def execute_command(command, ignore_errors=False, print_output=False):
    '''Executes a command and returns the output of the command
       If ignore_errors is True, it will succeed even if return status is
       other than 0. Running command service status <service> with a stopped
       service returns with a non zero status!
       print_output states if output should be sent to stdout
    
    Example_1 - Normal execution:
    >>> execute_command('echo hello')
    ['hello']

    Example_2 - Fail execution:
    >>> execute_command('command_not_exist')
    Traceback (most recent call last):
    ...
    CommandException: sh: command_not_exist: command not found
    '''
    logger.debug('Execute command: ' + command)
    
    (status, output) = commands.getstatusoutput(command)

    if status is 0 or ignore_errors:
        logger.debug ('Command succeeded:')
        for line in output.split('\n'):
            logger.debug(line)
            if print_output:
                print line
    else:
        logger.debug ('Error:')
        for line in output.split('\n'):
            logger.debug(line)
            if print_output:
                print line
        raise CommandException (status, output)

    return output.split('\n')

def display_text(text, color='green', max_len=49):
    """ Centers and displays a text in an ASCII rectangle. Input can be a
        string, or a tuple. If it is a string, the function will split it in
        as many lines as needed. If it is a tuple, each item is a line string.
        The color parameter is green by default. None prints it with no color.
        max_len defines the maximum length of a text line. Defaults to 49.

        >>> text = "Generating ICR Configuration Board File"
        >>> display_text(text, color=None)
        <BLANKLINE>
        =====================================================
        =      Generating ICR Configuration Board File      =
        =====================================================
        <BLANKLINE>

        >>> text = "Infiniband Support was configured in Node Installer"
        >>> display_text(text, color=None)
        <BLANKLINE>
        =====================================================
        =     Infiniband Support was configured in Node     =
        =                     Installer                     =
        =====================================================
        <BLANKLINE>

        >>> display_text(("Infiniband Support was", \
                          "configured in Node Installer"), color=None)
        <BLANKLINE>
        =====================================================
        =              Infiniband Support was               =
        =           configured in Node Installer            =
        =====================================================
        <BLANKLINE>

        >>> display_text(("Infiniband Support was configured", \
                          "in Node Installer"), color=None, max_len=59)
        <BLANKLINE>
        ===============================================================
        =              Infiniband Support was configured              =
        =                      in Node Installer                      =
        ===============================================================
        <BLANKLINE>


    """
    def center_text(line):
        """ Centers a text in a new string of 'max_len' length. """
        spaces = max_len - len(line)
        hs1 = spaces / 2
        hs2 = spaces - hs1
        line = ' ' * hs1 + line + ' ' * hs2
        return line

    lines = []

    if isinstance(text, tuple):
        lines = list(text)
    elif isinstance(text, basestring):
        words = text.split()
        line = []
        length = 0
        for w in words:
            if (len(w) + length) < max_len:
                length += 1 + len(w)
                line.append(w)
            else:
                lines.append(" ".join(line))
                length = len(w) 
                line = [w]

        if line:
            lines.append(" ".join(line))
    else:
        raise TypeError('Text must be a string or a tuple')

    color = text_colors.get(color, '')
    no_color = ''
    if color:
        no_color = '\033[0m'

    print
    print color + "=" * (max_len + 4)
    for line in lines:
        print "= %s =" % center_text(line)
        logger.debug(line)
    print "=" * (max_len + 4) + no_color
    print

def search_pattern_in_file(pattern, filename):
    """Function to search a pattern in a file. Returns True if it is found"""
    search_pattern = False
    if os.path.isfile(filename):
        try:
            file = open(filename, 'r')
            for line in file:
                if re.search(pattern, line) != None:
                    search_pattern = True
            file.close()
        except exceptions.IOError, ex:
            logger.debug(str(ex))
            raise SearchException("Error manipulating file : %s"  % str(ex))
    else:
        msg = "File not found: %s" % (filename)
        logger.debug(msg)
        raise SearchException(msg)
    return search_pattern


def cprint(text, color=None):
    """ Prints the given string in the given color. """
    logger.debug(text)
    color = text_colors.get(color)

    if color:
        print color + text + '\033[0m'
    else:
        print text

def valid_ip_addr(ip):
    """Checks if an IP address is valid using ipaddr module
    >>> valid_ip_addr('132.23.1.54')
    True
    >>> valid_ip_addr('332.23.1.54')
    False
    >>> valid_ip_addr('2.23.1')
    False
    """

    try:
        ipaddr.IPv4Address(ip)
        return True
    except:
        return False

def valid_address(ip, addrtype='ip'):
    """ Checks if an address of a given type (IP by default) is valid.
    >>> valid_address('132.23.1.54')
    True
    >>> valid_address('132.23.1.54', 'netmask')
    False
    >>> valid_address('332.23.1.54')
    False
    >>> valid_address('255.255.255.128', 'netmask')
    True
    >>> valid_address('255.255.255.128', 'email')
    Traceback (most recent call last):
    ...
    ValueError: email is not a valid address type
    """

    valid = False
    if addrtype.lower() in ('ip', 'ipv4'):
        addrtype = 'IP'
    elif addrtype.lower() in ('nm', 'netmask', 'net mask'):
        addrtype = 'Netmask'
    else:
        raise ValueError("%s is not a valid address type" % addrtype)

    if addrtype == 'IP':
        valid = valid_ip_addr(ip)
    elif addrtype == 'Netmask':
        valid = valid_ip_netmask(ip)

    return valid


def valid_ip_netmask(ip):
    """Checks if an IP netmask address is valid using ipaddr module
    >>> valid_ip_netmask('255.255.255.128')
    True
    >>> valid_ip_netmask('255.255.255.0')
    True
    >>> valid_ip_netmask('255.255.255.64')
    False
    """

    try:
        ipaddr.IPv4Network('0.0.0.0/' + ip)
        return True
    except:
        return False

def ip_network(ip, netmask=''):
    """Returns the network an IP belongs to depending on its netmask
    >>> ip_network('192.168.0.1', '255.255.255.128')
    '192.168.0.0'
    >>> ip_network('192.168.0.200', '255.255.255.128')
    '192.168.0.128'
    >>> ip_network('192.168.0.1/255.255.255.128')
    '192.168.0.0'
    >>> ip_network('192.168.0.200/255.255.255.128')
    '192.168.0.128'
    >>> ip_network('192.168.0.200/25')
    '192.168.0.128'
    >>> ip_network('192.168.0.200/255.255.255.128', '255.255.255.128')
    Traceback (most recent call last):
    ...
    ValueError: Netmask parameter was given and netmask is also defined in the ip parameter
    """

    if netmask:
        if '/' not in ip:
            ip = '/'.join( (ip, netmask) )
        else:
            raise ValueError('Netmask parameter was given and netmask is '
                             'also defined in the ip parameter')


    return ipaddr.IPv4Network(ip).network.exploded

def next_subnet_first_host(ip, netmask):
    """ Given an ip and a netmask it returns the ip of the first host in
        the following subnet.
        >>> next_subnet_first_host('192.168.4.8', '255.255.255.0')
        '192.168.5.1'

        >>> next_subnet_first_host('192.168.4.8', '255.255.255.128')
        '192.168.4.129'

        >>> next_subnet_first_host('192.168.254.1', '255.255.255.254')
        '192.168.254.3'

        >>> next_subnet_first_host('192.168.254.1', '255.255.0.254')
        Traceback (most recent call last):
        ...
        NetmaskValueError: 255.255.0.254 is not a valid netmask

        >>> next_subnet_first_host('192.168.254.1', '255.255.128.0')
        '192.169.0.1'
    """
    net = ipaddr.IPv4Network("%s/%s" % (ip, netmask))
    basenet = int(ipaddr.IPv4Address(net.network))
    lastip = basenet + net.numhosts
    return ipaddr.IPv4Address(lastip + 1).exploded

def show_multipage_file(fname):
    """ This method show a multiple page file according the windows length
    Depends on less utility.
    """
    try:
        fhdlr = open(fname)
        fhdlr.close()
    except IOError, ex:
        raise MultiPageException(str(ex))

    cmd = 'more %s' % fname

    try:
        os.system(cmd)
    except exceptions.OSError, ex:
        raise MultiPageException(str(ex))

def _test():                
    """Inline doctest."""
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()


