#!/usr/bin/python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.


""" This module defines the method for Red Hat Enterprise Linux OS """

# System Modules
import re
import exceptions

# Internal Modules
from common.logger import logger
import common.oslayer as oslayer
from common.configrecipe import ParameterNotFound, RecipeConfiguration

__version__ = "$Rev: 37520 $"

#############################################################################
#            RHEL   Class Definition                                        #
#############################################################################

class RHEL(oslayer._OSAL):
    """ Adds support for the RHEL operating system """

    def __init__(self, forced = False):
        self.version = ''
        oslayer._OSAL.__init__(self, 'RHEL', forced)

    def probe(self):
        """ Probes the existence of the OS in the system by analyzing the
            presence and contents of certain files.
        """
        file_name = '/etc/redhat-release'
        os_pattern = '^Red Hat Enterprise Linux Server release (\d+(?:\.\d+)?) \((\w+)\)'

        try:
            _fd = open(file_name, 'r')
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise oslayer.OSNotFound(self.name)

        try:
            file_contents = _fd.readline()
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise oslayer.OSNotFound(self.name)

        matchs = re.match(os_pattern, file_contents)
        if matchs is None:
            logger.debug ('The expected string ' + os_pattern + 
                          ' was not found in ' + file_name)
            raise oslayer.OSNotFound(self.name)

        self.version = matchs.group(1)
        if self.version is '':
            logger.info("The version for " + self.name + " could not be found")

    def __str__(self):
        return self.name + " " + self.version

#############################################################################
#            RHEL 6.1   Class Definition                                    #
#############################################################################

class RHEL61(oslayer._OSAL):
    """ Adds support for the RHEL 6.1 operating system """

    def __init__(self, forced = False):
        self.version = ''
        
        oslayer._OSAL.__init__(self, 'RHEL', forced)

        # Define the infiniband services
        self.ib_opensm_service = "opensm"
        self.ib_openib_service = "rdma"

    def probe(self):
        """ Probes the existence of the OS in the system by analyzing the
            presence and contents of certain files.
        """
        file_name = '/etc/redhat-release'
        os_pattern = '^Red Hat Enterprise Linux Server release (6\.1) \((Santiago)\)'

        try:
            _fd = open(file_name, 'r')
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise oslayer.OSNotFound(self.name)

        try:
            file_contents = _fd.readline()
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise oslayer.OSNotFound(self.name)

        matchs = re.match(os_pattern, file_contents)
        if matchs is None:
            logger.debug ('The expected string ' + os_pattern + 
                          ' was not found in ' + file_name)
            raise oslayer.OSNotFound(self.name)

        self.version = matchs.group(1)
        if self.version is '':
            logger.info("The version for " + self.name + " could not be found")

    def __str__(self):
        return self.name + " " + self.version


    def set_conf_files(self, config):
        """ This method sets the OS configuration files """

        if isinstance(config, RecipeConfiguration):
            # Sets sysctl.conf file path
            try:
                self.SYSCTL_FILE = config.get_parameter(self._section,"SYSCTL")
            except ParameterNotFound:
                self.SYSCTL_FILE = "/etc/sysctl.conf"
            
            # Sets limits.conf file path
            try:
                self.LIMITS_FILE = config.get_parameter(self._section,"LIMITS")
            except ParameterNotFound:
                self.LIMITS_FILE = "/etc/security/limits.conf"

            # Sets modprobe.conf file path
            try:
                self.MODPRB_FILE = config.get_parameter(self._section,"MODPRB")
            except ParameterNotFound:
                self.MODPRB_FILE = "/etc/modprobe.d/modprobe.conf"    

            # Sets dat.conf file path
            try:
                self.DATCNF_FILE = config.get_parameter(self._section,"DATCNF")
            except ParameterNotFound:
                self.DATCNF_FILE = "/etc/rdma/dat.conf"

            # Sets ifcfg network scripts path and prefix
            try:
                self.IFCFG_FILE = config.get_parameter(self._section,"IFCFG")
            except ParameterNotFound:
                self.IFCFG_FILE = "/etc/sysconfig/network-scripts/ifcfg-"

            # Sets openib.conf file path
            try:
                self.OPENIB_FILE = config.get_parameter(self._section,"OPENIB")
            except ParameterNotFound:
                self.OPENIB_FILE = "/etc/rdma/rdma.conf"

            # Sets passwd file path
            try:
                self.PASSWD_FILE = config.get_parameter(self._section,"PASSWD")
            except ParameterNotFound:
                self.PASSWD_FILE = "/etc/passwd"
            
            # Sets the profile file path
            try:
                self.PROFILE_FILE = config.get_parameter(self._section,"PROFILE")
            except ParameterNotFound:
                self.PROFILE_FILE = "/etc/profile"
        else:
            error = 'RecipeConfiguration type expected but _%s_ received' % \
                str(type(config)).split("'")[1]
            logger.error(error)
            raise exceptions.TypeError(error)

if __name__ == "__main__":
    print "This file should not be run alone"
