""" This module adds support for the Rocks provisioning system. """
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.

import exceptions
import os
import shutil
import re
from xml.dom import minidom
from socket import gethostname
import oslayer
import ipaddr
from logger import logger
from common.provisioning import _Provisioning
from common.provisioning import ProvisioningException
from common.provisioning import ProvisioningNotFound
from common.provisioning import ProvisioningVersionNotFound
from common.provisioning import DatabaseError
from common.provisioning import DeviceExistsError
from common.provisioning import ServiceError
from common.provisioning import PackageAddError
import recipeutils as ru

__version__ = "$Rev: 37503 $"

#############################################################################
#     Main Rocks Class (all rocks classes should inherit from this one)     #
#############################################################################

class ROCKS(_Provisioning):
    """ Adds support for the Rocks provisioning system

    Example_1 - Register this provisioning
    >>> register_provisioning(ROCKS)

    Example_2 - Try to get the provisioning, expect raise an exception         
    because the provisioning could not be found
    >>> get_provision()
    Traceback (most recent call last):
    ...
        raise NoProvisioningFound()
    NoProvisioningFound: No supported provisioning system was detected
    """

    def __init__(self, name = None):
        self.version = ''
        self.opsys = None
        self.node_installer_group = None
        _Provisioning.__init__(self, name)
        return

    def probe(self):
        """ Probes if the provisioning is really Rocks  """
        raise NotImplementedError

    def get_nodes(self):
        """ Gets nodes list by using provisioning specific commands """
        raise NotImplementedError

    def configure_device(self, *args, **kwargs):
        """ Configures a network device.
            Just calls the OS abstraction layer method with the same params.
        """
        try:
            self.opsys.configure_device(*args, **kwargs)
        except Exception, ex:
            logger.debug(ex)
            raise ProvisioningException('Error configuring network device')

    def add_device(self, dev, ip_addr=None, netmask=None, netname=None,
                   mac=None, module=None, name=None):
        """ Adds a device to the database.
            Parameters:
            dev      -- network device, e.g. ib0, eth1
            ip_addr  -- ip address for the device
            netmask  -- subnet mask for the device
            netname  -- netname needed for rocks.
            mac      -- MAC address
            module   -- kernel module used
            name     -- hostname for the device
        """

        hostname = gethostname().split('.')[0]
        if not name and dev.startswith('ib'):
            name = hostname + '-ib'

        cmd = "rocks add host interface %s %s" % (hostname, dev)

        if ip_addr:
            cmd += " ip=%s" % ip_addr
        if netname:
            cmd += " subnet=%s" % netname
        if mac:
            cmd += " mac=%s" % mac
        if module:
            cmd += " module=%s" % module
        if name:
            cmd += " name=%s" % name

        try:
            ru.execute_command(cmd)
        except ru.CommandException, ex:
            logger.debug(ex)
            msg = 'interface \"%s\" exists' % dev
            if msg in ex.reason:
                raise DeviceExistsError(msg)
            raise DatabaseError('Error executing ' + cmd)

    def remove_device(self, dev):
        """ Removes a device from the database """

        hostname = gethostname().split('.')[0]
        cmd = "rocks remove host interface %s %s" % (hostname, dev)

        try:
            ru.execute_command(cmd)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise DatabaseError('Error executing ' + cmd)

    def edit_device(self, dev, ip_addr=None, netmask=None, netname=None,
                    mac=None, module=None, name=None):
        """ Edits the device information in the database
            Parameters:
            dev      -- network device, e.g. ib0, eth1
            ip_addr  -- ip address for the device
            netmask  -- subnet mask for the device
            netname  -- netname needed for rocks.
            mac      -- MAC address
            module   -- kernel module used
            name     -- hostname for the device
        """

        hostname = gethostname().split('.')[0]

        cmds = []
        base_cmd = "rocks set host interface "
        base_arg = "%s %s" % (hostname, dev)

        if ip_addr:
            cmds.append(base_cmd + " ip " + base_arg + " ip=%s" % ip_addr)
        if netname:
            cmds.append(base_cmd + " subnet " + base_arg + \
            " subnet=%s" % netname)
        if mac:
            cmds.append(base_cmd + " mac " + base_arg + " mac=%s" % mac)
        if module:
            cmds.append(base_cmd + " module " + base_arg + \
            " module=%s" % module)
        if name:
            cmds.append(base_cmd + " name " + base_arg + " name=%s" % name)

        for cmd in cmds:
            try:
                ru.execute_command(cmd)
            except ru.CommandException:
                raise DatabaseError('Error executing ' + cmd)


    def add_network(self, name, ip_addr, netmask, device, subnet='', desc='',
                    suffix=''):
        """ Adds a network to the database """

        if not subnet:
            net = ipaddr.IPv4Network('%s/%s' % (ip_addr, netmask))
            subnet = net.network.exploded

        cmd = "rocks add network %s subnet=%s netmask=%s" % \
            (name, subnet, netmask)
        try:
            ru.execute_command(cmd)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise DatabaseError('Error executing ' + cmd)

    def sync_network(self, *args, **kwargs):
        """ Synchronizes the network configuration """

        cmd = "rocks sync config"
        try:
            ru.execute_command(cmd)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise DatabaseError('Error executing ' + cmd)

        hostname = gethostname().split('.')[0]

        cmd = "rocks sync host network " + hostname
        try:
            ru.execute_command(cmd)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise DatabaseError('Error executing ' + cmd)

    def sync(self, *args, **kwargs):
        """ Synchronizes the configuration """

        cmd = "rocks sync config"
        try:
            ru.execute_command(cmd)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise DatabaseError('Error executing ' + cmd)
        
        cmd = "rocks sync users"
        try:
            ru.execute_command(cmd)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise DatabaseError('Error executing ' + cmd)

        # restart autofs service as the /home folder was updated in the
        # /etc/passwd file and it won't probably mount the required 
        # home folder immediately
        try:
            self.opsys.service_restart('autofs')
        except oslayer.ServiceError, ex:
            logger.debug(ex)
            raise ServiceError('Error restarting service autofs')
            

    def bring_up_device(self, device):
        """ ifup device """

        cmd = "ifup " + device
        try:
            ru.execute_command(cmd)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise ProvisioningException('Error bringing up ' + device)

    def has_network_device(self, network_device):
        """ Returns True if the network device is enabled """
        try:
            ru.execute_command("rocks list host interface | grep %s" % \
                               network_device)
            return True
        except ru.CommandException:
            return False

    def link_network(self, network_device, node_group):
        """ Links a network device to a node group """

    def network_configured(self, netname):
        """ Returns True if the network for current host has been added to
            Rocks+ database. False if not.
        """
        try:
            ru.execute_command("rocks list network | grep %s" % netname)
            return True
        except ru.CommandException:
            return False

        hostname = gethostname().split('.')[0]

        try:
            ru.execute_command("rocks list host interface %s | grep %s" % \
            (hostname, netname))
        except ru.CommandException:
            return False

        return True

    def get_networks(self):
        """ Gets networks information from the database """

        try:
            output = ru.execute_command('rocks list network')
        except ru.CommandException, ex:
            logger.debug(ex)
            raise DatabaseError('Error getting networks list')

        output = output[1:] # remove heading
        result = []

        for line in output:
            match = re.match('([^\s]+)\s+([^\s]+)\s+([^\s]+)', line)

            if match:
                net_id = match.group(1).rstrip(':')
                if net_id == 'private':
                    net_type = 'provision'
                elif net_id == 'public':
                    net_type = 'public'
                else:
                    net_type = ''
                result.append( {'id': net_id,
                                'device': '',
                                'network': match.group(2),
                                'subnet': match.group(3),
                                'description': '',
                                'type': net_type,
                                } )

        try:
            output = ru.execute_command('rocks list host interface')
        except ru.CommandException, ex:
            logger.debug(ex)
            raise DatabaseError('Error getting interfaces list')

        output = output[1:] # remove heading

        for line in output:
            match = re.match('([^\s]+)\s+([^\s]+)', line)

            if match:
                index = 0
                name = match.group(1)
                for net in result:
                    if name == net['id']:
                        net['device'] = match.group(2)
                        result[index] = net
                        break
                    index += 1

        return result

    def setup_ib_files(self):
        """ Configures the necessary OS files for InfiniBand.
            This step is not needed in ROCKS but is called in the script
        """
        pass


    def setup_ib_services(self):
        """ Configures, starts, stops, restarts InfiniBand and network
        services as needed for configuration.
        """
        if not self.opsys:
            logger.critical("Provisioning instance has no associated OS")
            raise Exception("Provisioning instance has no associated OS")
        
        try:
            if self.opsys.service_running(self.opsys.ib_opensm_service):
                self.opsys.service_stop(self.opsys.ib_opensm_service)
        except oslayer.ServiceError, ex:
            logger.debug(ex)
            msg = 'Error stopping service %s' % self.opsys.ib_opensm_service
            raise ServiceError(msg)
        
        try:
            self.opsys.service_restart(self.opsys.ib_openib_service)
        except oslayer.ServiceError, ex:
            logger.debug(ex)
            msg = 'Error restarting service %s' % self.opsys.ib_openib_service
            raise ServiceError(msg)
        
        try:
            self.opsys.service_enable(self.opsys.ib_opensm_service)
        except Exception, ex:
            raise ServiceError(ex)

        # Verify that openib could be loaded successfully
        if self.opsys.openibd_status() != 'loaded':
            msg = "Service %s could not be started" % self.opsys.ib_openib_service
            raise ServiceError(msg)
        
        try:
            self.opsys.service_restart(self.opsys.ib_opensm_service)
        except oslayer.ServiceError, ex:
            logger.debug(ex)
            msg = 'Error restarting service %s' % self.opsys.ib_opensm_service
            raise ServiceError(msg)

        # Verify that opensmd could be loaded
        if not self.opsys.service_running(self.opsys.ib_opensm_service):
            msg = "Service %s could not be started" % self.opsys.ib_opensm_service
            raise ServiceError(msg)
        
        try:
            self.opsys.service_restart('network')
        except oslayer.ServiceError, ex:
            logger.debug(ex)
            raise ServiceError('Error restarting network service')

    def ib_dev_configured(self, dev, ip_addr, netmask):
        """ Checks if ib device 'dev' is already configured with ip 'ip_addr'
            and netmask 'netmask'
        """

        ib0_info = self.opsys.device_info(dev)

        # Check if is up and with the specified ip and netmask
        if ib0_info.get('inet addr') != ip_addr or \
            ib0_info.get('Mask') != netmask:
            return False

        net = ipaddr.IPv4Network('%s/%s' % (ip_addr, netmask))

        # Check if is stored in database
        for net_info in self.get_networks():
            net_dev = net_info.get('device')

            if net_dev == dev:
                net_ip = net_info.get('network')
                mask_ip = net_info.get('subnet')
                if net_ip == net.network.exploded and mask_ip == netmask:
                    return True

        return False

    def component_installed(self, comp):
        """ Checks if the given component is installed. """

        cmd = "rocks list roll %s | tail -n1 | awk '{print $4}'" % comp

        try:
            out = ru.execute_command(cmd)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise DatabaseError('Error getting info for component ' + comp)

        if out[0].strip() == 'yes':
            return True

        return False
    
    def has_component(self, comp):
        """ Checks if the given component is available. """

        cmd = "rocks list roll %s" % comp
        output = None

        try:
            ru.execute_command(cmd)
            output = True
            logger.debug("Component %s found" % comp)
        except ru.CommandException, ex:
            logger.debug(ex)
            output = False
            logger.debug("Component %s not found" % comp)

        return output


    def add_script(self, script, nodegroup):
        """ This method add a script to the xml for nodes
            or executes the script if the specified node group
            is the front end
        """

        if nodegroup == self.node_installer_group:
            # In frontend the packages are just installed as general packages
            self._execute_frontend_script(script)
        else:
            # Create a new XML configuration file that will extend the 
            # current appliance configuration, adding a script to be executed
            # at first boot
            self._add_appliance_script(nodegroup, script)

    def set_bkm_ethernet(self, drivers, modprobefile):
        """ Adds the parameter option to the driver in the modprobe file """

        # Method vars
        _file = modprobefile
        _fd = None
        _regex = None
        _line = ''
        _lines = []

        # Parameter to configure
        param = "InterruptThrottleRate=0,3"

        for drv in drivers:
            # Set match variable in false
            _reg_match = False

            try:
                _fd = open(_file)
                _lines = _fd.readlines()
                _fd.close()
            except IOError, ex:
                raise ProvisioningException(str(ex))

            # Regular expression to match Interrupt Throttle Rate configuration
            # e.g.
            # options * InterruptThrottleRate=0,3 # match!

            _regex = re.compile("^options %s InterruptThrottleRate=0,3" % drv)

            try:
                _fd = open(_file, 'w')
            except IOError, ex:
                raise ProvisioningException(str(ex))

            for _line in _lines:
                if _regex.match(_line):
                    _reg_match = True
                _fd.write(_line)

            # Set configuration
            if not _reg_match:
                _fd.write('# InterruptThrottleRate configuration\n')
                _fd.write('options %s %s\n' % (drv, param))
                logger.info('- Ethernet BKM for %s configured' % drv )
            else:
                logger.info('- Ethernet BKM for %s already configured' % drv) 
            _fd.close()

    def get_private_net(self):
        """ Gets the private network """
        for net in self.get_networks():
            if net.get('type') == 'provision':
                return net

    def get_interfaces(self):
        """ Gets networks interfaces from the database """

        try:
            output = ru.execute_command('rocks list host interface')
        except ru.CommandException, ex:
            logger.debug(ex)
            raise DatabaseError('Error getting interfaces list')

        ifaces = []
        output = output[1:] # remove heading

        for line in output:
            match = re.match('([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)', line)

            if match:
                subnet = not match.group(1).startswith('---') and \
                         match.group(1) or None
                device = not match.group(2).startswith('---') and \
                         match.group(2) or None
                mac = not match.group(3).startswith('---') and \
                      match.group(3) or None
                ip = not match.group(4).startswith('---') and \
                     match.group(4) or None
                # netmask not used
                module = not match.group(6).startswith('---') and \
                         match.group(6) or None
                name = not match.group(7).startswith('---') and \
                       match.group(7) or None


                ifaces.append( {'subnet': subnet,
                                'device': device,
                                'mac': mac,
                                'ip': ip,
                                'module': module,
                                'name': name
                                } )

        return ifaces

    def fix_10gb_order(self, ifaces_10gb):
        """ Attempts to fix the 10gb interfaces order """

    def add_sshkeys(self, user_name):
        """ Add SSH keys for a user """
        logger.info("Generating SSH keys...")
        try:
            file_orig = '/etc/profile.d/ssh-key.sh'
            file_back = '/etc/profile.d/ssh-key.sh.orig'
            cmd = 'sed -i.orig -e \'s/\\(ssh-keygen -t rsa\\)/\\1 -b 2048 -f $HOME\/.ssh\/id_rsa -N ""/\''

            # Replace the script that generates the SSH keys
            ru.execute_command("%s %s" % (cmd, file_orig))

            # Log in the user to allow the script generates the SSH keys
            ru.execute_command("su - %s -c exit" % (user_name))

            # Restore the original script
            ru.execute_command("mv -f %s %s" % (file_back, file_orig))

        except ru.CommandException, ex:
            logger.critical("Failed creating SSH keys")
            logger.critical("ERROR: %s" % (ex))
            raise ru.CommandException(ex)

    def has_group(self, group):
        """ Returns true if the group is present in the provisioning system
        """
        if group == self.node_installer_group:
            hasgroup = True
        else:
            hasgroup = self.check_appliance(group)
        
        return hasgroup
            

    def check_appliance(self, appliance):
        """ Check that an appliance exists 
            rocks list appliance <appliance_name> return 0 if the appliance 
            was found, error code if could not find it or an error occurred
        """

        appliance_exists = None
        try:
            ru.execute_command("rocks list appliance %s" % (appliance)) 
            appliance_exists = True
        except ru.CommandException:
            appliance_exists = False
       
        return appliance_exists 


    def add_package(self, nodegroup, packagename):
        """ This method add a package to a nodegroup
            After adding a package rebuild the creation of the images/distros
        """


        if nodegroup == self.node_installer_group:
            # In frontend the packages are just installed as general packages
            
            self._add_frontend_package(packagename)
        else:
            # Create a new XML configuration file that will extend the 
            # current appliance configuration
            self._add_appliance_package(nodegroup, packagename)
                        

    def rebuild_images(self):
        """ Rebuilds the distro for including new rpms """
        distro_folder = "/export/rocks/install"
        try:
            ru.execute_command("cd %s && rocks create distro" % distro_folder)
        except ru.CommandException, err:
            raise ProvisioningException("Error rebuilding images, %s", str(err))



    def _add_frontend_package (self, packagename):
        """ Install a package in the front end"""        
        
        # The rationale of this is, if the package is not installed install it
        # but if the package is installed that means has_package return true,
        # don't do anything.
        if not self.opsys.has_package(packagename):
            try:
                self.opsys.install_package(packagename)
                logger.info("Package %s installed successfully in frontend" %\
                    packagename)
            except oslayer.PackageError, ex:
                
                logger.error("Error installing %s: %s"\
                                             % (packagename, str(ex)))
                raise PackageAddError(self.node_installer_group, packagename)

    def _add_package_to_extend_file(self, appliance, extend_file, package):
        """ Adds a package to the extend_file """
    
        try:     
            dom = minidom.parse(extend_file)
        except IOError, ex:
            raise PackageAddError(appliance, package)
    
        has_inserted_package = False
        packages_list = []
    
        for package_dom in dom.getElementsByTagName('package'):
            
            packages_list.append(package_dom.childNodes[0].data)
            
        
        if package not in packages_list:
            new_package_dom = dom.createElement(unicode('package'))
            
            text_dom = dom.createTextNode(unicode(package))
    
            new_package_dom.appendChild(text_dom)
    
             
            # By default add the package dom at the end
            reference_dom = dom.childNodes[0].childNodes[-2]        
    
            # search for the dom where the comment for adding packages below is
            for maindom in dom.childNodes[0].childNodes:
    
                if isinstance(maindom, minidom.Comment):
                    if maindom.data.rfind("<package></package>"):
                        
                        reference_dom = maindom.nextSibling.nextSibling
                        break
                        
            end_line_dom = dom.createTextNode(unicode("\n"))
            dom.childNodes[0].insertBefore(end_line_dom, reference_dom)
    
            # insert it inside Document Element kickstart (dom.childNodes[0])
        
            dom.childNodes[0].insertBefore(new_package_dom, end_line_dom)
    
    
            has_inserted_package = True
            logger.debug( "Inserted package %s in %s" % (package, extend_file) )
        else:
            logger.debug( "Package %s was already inserted in file %s" %\
                        (package, extend_file))
    
    
        if has_inserted_package:
    
            try:
                filed = open( extend_file, "w" )
            except IOError, ex:
                logger.error("Error writing file %s: %s" %\
                     (extend_file, str(ex))) 
                raise PackageAddError(appliance, package)
            
            dom.writexml(filed)
            
            filed.close()

    @property
    def extend_folder(self):
        """ Retrieves the folder were customizations are stored """
        return "/export/rocks/install/site-profiles/%s/nodes" \
                    % self.version


    def _create_extend_file (self, extend_folder, extend_file_path):
        """ Creates the extend file from a template
            Raises IOError
        """
        
        template = "skeleton.xml"
        template_path = extend_folder + "/" + template
        if os.path.isfile(template_path):
            shutil.copyfile(template_path, extend_file_path)
        else:
            raise IOError ("Template File %s does not exist", template_path)
            
            

    def _add_appliance_package (self, appliance, packagename):
        """Add a package to an appliance using the extend method.
           A new appliance is created by 
        """

        if not self.opsys.has_package_in_repo(packagename):
            logger.error("Package %s, is not available in repositories"\
                            % packagename)
            raise PackageAddError(appliance, packagename)

        extend_file = "extend-%s.xml" % appliance

        extend_file_path = self.extend_folder + "/" + extend_file

        if self.check_appliance(appliance):
            if os.path.isdir(self.extend_folder):
            # Create extend file if it doesn't exist
                if not os.path.isfile(extend_file_path):
                # File is not present so create one
                    try: 
                        self._create_extend_file(self.extend_folder, \
                                                 extend_file_path)
                    except IOError, ex:
                        msg = "Could not create extend file, %s" % str(ex)
                        logger.error(msg)
                        raise PackageAddError (appliance, packagename)
                                
                self._add_package_to_extend_file(appliance, \
                                extend_file_path, packagename)
            else:
                logger.error("Missing folder %s" % self.extend_folder)
        else:
            logger.error("Appliance does not exist")
            raise PackageAddError(appliance, packagename)


    def add_package_to_repo(self, package_file):
        """ Add an rpm file package to the repository """

        repo_folder = "/export/rocks/install/contrib/%s/x86_64/RPMS" \
            % self.version

        if not os.path.isdir(repo_folder):
            raise ProvisioningException("Repository folder %s not created" \
                % repo_folder)

        if isinstance(package_file, str) and \
            os.path.isfile(package_file):

            # Check that file was not already added

            file_name = os.path.basename(package_file)
            if not os.path.isfile(os.path.join(repo_folder, file_name)):
    
                try:
                    shutil.copy(package_file, repo_folder)
                except IOError, ex:
                    logger.debug(str(ex))
                    raise ProvisioningException("Could not add package " +
                        package_file + " to repository")
            else:
                logger.debug ("File %s already added to repository\n"\
                     % file_name)


    def update_repo(self):
        """ Updates repository of packages """

        self.rebuild_images()

        # As a workaround refresh the yum database. Otherwise
        # packages may not appear after adding them to the
        # repository
        self.opsys.refresh_repo()


    def _execute_frontend_script(self, script):
        """ Executes a script in the frontend """

        logger.info('Executing ' + script + ' in Frontend')

        if not os.path.isfile(script):
            raise ProvisioningException('Could not locate file %s' \
                    % script)

        os.system('chmod +x %s' % script)

        try:
            ru.execute_command(script)
        except ru.CommandException, ex:
            logger.debug(ex)
            raise ProvisioningException('Error executing %s.' % script)


    def _add_appliance_script(self, appliance, script):
        """ Adds the script to an appliance to be executed at first boot"""
    
        extend_file = "extend-%s.xml" % appliance

        extend_file_path = self.extend_folder + "/" + extend_file

        # open script file
        try:
            script_fd = open (script)
        except IOError, ex:
            logger.debug(str(ex))
            raise ProvisioningException("Could not open script file %s:%s" %\
                (script, str(ex)))

        try:
            # open the file and separate each line in different arrays
            script_contents = script_fd.read().split('\n')
        except IOError, ex:
            logger.debug(str(ex))
            msg = "Could not read the contents of file %s:%s" % \
                (script, str(ex))
            raise ProvisioningException(msg)

        script_fd.close()


        if self.check_appliance(appliance):
            if os.path.isdir(self.extend_folder):
            # Create extend file if it doesn't exist
                if not os.path.isfile(extend_file_path):
                # File is not present so create one
                    try: 
                        self._create_extend_file(self.extend_folder, \
                                                 extend_file_path)
                    except IOError, ex:
                        msg = ("Could not create extend file, %s" % str(ex))
                        raise ProvisioningException(msg)
                                
                self._add_script_to_extend_file(appliance, \
                                extend_file_path, script_contents,
                                os.path.basename(script) )
            else:
                logger.error("Missing folder %s" % self.extend_folder)

    
    def _fix_scaping(self, file):
        """ Solve an issue in the parser that replaces characters such
        as < with &gt:
        """
        
        try:
            fd = open(file)
        except IOError, ex:
            raise ProvisioningException("Error opening file %s:%s" % \
                (file, str(ex)))

        try:
            file_cont = fd.read()
        except IOError, ex:
            raise ProvisioningException("Error reading file %s:%s" % \
                 (file, str(ex)))
        
        fd.close()

        file_cont = file_cont.replace('&amp;','&').replace('&quot;', '"')\
                .replace('&apos;', '\'').replace('&gt;','>')\
                .replace('&lt;', '<')

        try:
            fd = open(file, 'w')
        except IOError, ex:
            raise ProvisioningException("Error opening file %s:%s" % \
                 (file, str(ex)))
        
        try:
            fd.write(file_cont)
        except IOError, ex:
            raise ProvisioningException("Error writing file %s:%s" % \
                 (file, str(ex)))
   
        fd.close()
    


    def _add_script_to_extend_file(self, appliance, extend_file, script, name):
        """ Adds the scripts contents to the extend_file """

        try:
            dom = minidom.parse(extend_file)
        except IOError, ex:
            raise ProvisioningException("appliance Error adding scripts: %s" \
                % str(ex))

        # create a signature for checking if the file was already added
        script_signature = "##### " + name + " #####"

        # assumes that the script is using the template so the post tag is 
        # already created

        post_dom = dom.getElementsByTagName('post')[0]

        already_updated = False

        for post_child_dom in post_dom.childNodes:
            if isinstance(post_child_dom, minidom.Text):
                # If the signature was located, do not update the file
                if post_child_dom.data.rfind(script_signature) != -1:
                    already_updated = True

        if not already_updated:

            text = "\n".join(["\n", script_signature, "\n".join(script), 
                              "\n\n"])
                            
            script_dom = dom.createTextNode(unicode(text)) 
                    
            post_dom.insertBefore(script_dom, post_dom.childNodes[0])     
                

            try:
                filed = open( extend_file, "w" )
            except IOError, ex:
                raise ProvisioningException("Error writing file %s: %s" %\
                    (extend_file, str(ex)))

            dom.writexml(filed)
            filed.close()

            self._fix_scaping(extend_file)

    def copy_nodegroup(self, orig, copy):
        """ Abstract class to duplicate nodegroups """
        # Still not implemented in Rocks
        raise NotImplementedError
    
    def get_profiles(self):
        """ Gets profiles by using provisioning specific commands """
        # Still not implemented in Rocks
        raise NotImplementedError

#############################################################################
#     ROCKS 5.3 (open source)                                               #
#############################################################################

class ROCKS53(ROCKS):
    """ Adds support for the Rocks 5.3 provisioning system
    """

    def __init__(self):
        self.file_name = '/etc/rocks-release'
        ROCKS.__init__(self, "Rocks")
        return

    def probe(self):
        """ Probes if the provisioning is really Rocks  """

        # Pattern for parsing the following strings:
        # Rocks release 5.3 (Rolled Tacos)

        prov_pattern = '^Rocks release (5.3).*'

        try:
            fileh = open(self.file_name, 'r')
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise ProvisioningNotFound(self.name)

        try:
            file_contents = fileh.readline()
        except exceptions.IOError, exception:
            logger.debug (str(exception))
            raise ProvisioningVersionNotFound(self.name)

        matchs = re.match(prov_pattern, file_contents)
        if matchs is None:
            logger.debug ('The expected string was not found in ' + self.file_name)
            raise ProvisioningVersionNotFound(self.name)

        self.version = matchs.group(1)
       
        # If the system has Rocks it should not have the component "support"
        # so if the component is found, raise an exception that this is
        # not the provisioning system Rocks
        if self.has_component("support"):
            raise ProvisioningNotFound(self.name)

        self.node_installer_group = "frontend"

#############################################################################
#     ROCKS PLUS 5.3                                                        #
#############################################################################

class ROCKSPLUS53(ROCKS):
    """ Adds support for the Rocks Plus provisioning system
    """

    def __init__(self):
        self.file_name = '/etc/rocks-release'
        ROCKS.__init__(self, "Rocks+ from Clustercorp.com")
        return

    def probe(self):
        """ Probes if the provisioning is really Rocks+  """

        # Pattern for parsing the following strings:
        # Rocks release 5.3 (Rolled Tacos)

        prov_pattern = '^Rocks release (5.3).*'

        try:
            fileh = open(self.file_name, 'r')
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise ProvisioningNotFound(self.name)

        try:
            file_contents = fileh.readline()
        except exceptions.IOError, exception:
            logger.debug (str(exception))
            raise ProvisioningVersionNotFound(self.name)

        matchs = re.match(prov_pattern, file_contents)
        if matchs is None:
            logger.debug ('The expected string was not found in ' + self.file_name)
            raise ProvisioningVersionNotFound(self.name)

        self.version = matchs.group(1)


        # If the system has Rocks Plus it should have the folder "rocksplus"
        # under /opt folder so if the folder is not found, raise an exception
        # hat this is not the provisioning system Rocks Plus
        if not os.path.isdir('/opt/rocksplus'):
            raise ProvisioningNotFound(self.name)
            
        self.node_installer_group = "frontend"


    def get_nodes(self):
        """ Gets nodes list by using provisioning specific commands """
        return self._get_nodes_from_command('dbreport nodes')

    def copy_nodegroup(self, orig, copy):
        """ Abstract class to duplicate nodegroups """
        # Still not implemented in Rocks
        raise NotImplementedError
    
    def get_profiles(self):
        """ Gets profiles by using provisioning specific commands """
        # Still not implemented in Rocks
        raise NotImplementedError

#############################################################################
#     ROCKS PLUS 5.4                                                        #
#############################################################################

class ROCKSPLUS54(ROCKS):
    """ Adds support for the Rocks Plus provisioning system
    """

    def __init__(self):
        self.file_name = '/etc/rocks-release'
        ROCKS.__init__(self, "Rocks+ from StackIQ")
        return

    def probe(self):
        """ Probes if the provisioning is really Rocks+  """

        # Pattern for parsing the following strings:
        # Rocks release 5.4 (Maverick)

        prov_pattern = '^Rocks release (5.4).*'

        #prov_pattern = '^Rocks release (\d+.\d+).*'

        try:
            fileh = open(self.file_name, 'r')
        except exceptions.IOError, exception:
            logger.debug( str(exception))
            raise ProvisioningNotFound(self.name)

        try:
            file_contents = fileh.readline()
        except exceptions.IOError, exception:
            logger.debug (str(exception))
            raise ProvisioningVersionNotFound(self.name)

        matchs = re.match(prov_pattern, file_contents)
        if matchs is None:
            logger.debug ('The expected string was not found in ' + self.file_name)
            raise ProvisioningVersionNotFound(self.name)

        self.version = matchs.group(1)

        # If the system has Rocks Plus it should have the folder "rocksplus"
        # under /opt folder so if the folder is not found, raise an exception
        # hat this is not the provisioning system Rocks Plus
        if not os.path.isdir('/opt/rocksplus'):
            raise ProvisioningNotFound(self.name)
            
        self.node_installer_group = "frontend"


    def get_nodes(self):
        """ Gets nodes list by using provisioning specific commands """
        cmd = "rocks report host icrnodes | awk {'print $1'}"
        return self._get_nodes_from_command(cmd)

    def copy_nodegroup(self, orig, copy):
        """ Abstract class to duplicate nodegroups """
        # Still not implemented in Rocks Plus
        raise NotImplementedError
    
    def get_profiles(self):
        """ Gets profiles by using provisioning specific commands """
        # Still not implemented in Rocks Plus
        raise NotImplementedError

        

if __name__ == "__main__":
    print "This file should not be run alone"
