#!/usr/bin/env python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.

""" Common classes for module objects """

import exceptions
import parameterlist
import configrecipe
from configrecipe import RecipeConfiguration
from logging import Logger
from provisioning import _Provisioning

__version__ = "$Rev: 35275 $"

class SectionError(exceptions.Exception):
    """ Base exception for section errors """
    
    def __init__(self, reason=None):
        exceptions.Exception.__init__(self)
        self.reason = reason
    
    def __str__(self):
        return self.reason 

class SectionSkipped(exceptions.Exception):
    """ The section was not executed due to a non critical issue """
     

class NoParametersError(SectionError):
    """ self._dict or self._section vars are not defined in the derived classes
    """
    def __init__(self, reason):
        SectionError.__init__(self, reason)
        

class ParameterNotFound(SectionError):
    """ The parameter requested by the user is invalid """

    def __init__(self, parameter_name):
        self.reason = "Parameter %s was not configured" % \
            parameter_name
        SectionError.__init__(self, self.reason)
       
    def __str__(self):
        return repr(self.reason)

class Section(object):
    """ Abstraction of section interfaces """

    def __init__(self, prov, logger, config):
        """ Initializes the section interface
        """

        if isinstance(logger, Logger):
            self._logger = logger
        else:
            error = '%s=%s is not an instance of Logger' % \
            (str(logger), type(logger))
            try:
                from logger import logger
                logger.error(error)
            except exceptions.ImportError:
                print "Failed to import logger"
            raise exceptions.TypeError(error)

        if isinstance(prov, _Provisioning):
            self._prov = prov
        else:
            error = '%s=%s is not an instance of _Provisioning' % \
            (str(prov), type(prov))
            self._logger.error(error)
            raise exceptions.TypeError(error)

        if isinstance(config, RecipeConfiguration):
            self._config = config
        else:
            error = '%s=%s is not an instance of RecipeConfiguration' % \
            (str(config), type(config))
            self._logger.error(error)
            raise exceptions.TypeError(error)

        self._dict = {}
        self._defaultdict = {}
        self._section = ''
        # Relative bash scripts directory. Should be used with INSTALL_DIR
        self._bash_scripts_d = 'lib/python/sections/bash_scripts'

    def get_parameter_list(self):
        """ This method return a list with all the variables that Config Recipe
        should understand. 
        """
        if self._section == '' or self._dict == {}:
            raise NoParametersError('Section or vars not defined.')
        else:
            params = parameterlist.ParameterList(self._section, self._dict)
            if self._defaultdict != {}:
                defparams = parameterlist.ParameterList('Default',
                                                        self._defaultdict)
                params.concat(defparams)

            return params

    def run(self):
        """ To be implemented by the derived class.
        """
        raise NotImplementedError()

    def section_configurator(self):
        """ Loads all the required parameters and validates them
        """
        raise NotImplementedError()

    def check_requirements(self):
        """All the checks that will be done before executing the scripts goes
        here"""
        raise NotImplementedError()


    def section_dependency_check(self):
        """All the checks that are needed in this section that should be solved
        in previous sections """
        raise NotImplementedError()
                     
    def get_param(self, var):
        """ This method retrieves a variable value from the configuration 
	object. The function succeeds only if the variable was previously
        defined in the default section or the recipe section's reserved
	configuration section

        """

        if self._dict.has_key(var):
        #Obtain a parameter in the recipe sections of the configuration section
            try:
                value = self._config.get_parameter(self._section,var)
            except configrecipe.ParameterNotFound:
                raise ParameterNotFound(var)
        elif self._defaultdict.has_key(var):
            try:
                value = self._config.get_parameter('Default', var)
            except configrecipe.ParameterNotFound:
                raise ParameterNotFound(var)
        else:
            raise ParameterNotFound(var)

        if value == 'NOT_REQUIRED':
            return None

        return value

    def get_multi_param(self, var):
        """ returns a list of dictionaries representing the parameters
            and its options found in the config file.
            if the config file has something like

        """

        if self._dict.has_key(var):
        #Obtain a parameter in the recipe sections of the configuration section
            try:
                value = self._config.get_multi_parameter(self._section,var)
            except configrecipe.ParameterNotFound:
                raise ParameterNotFound(var)
        elif self._defaultdict.has_key(var):
            try:
                value = self._config.get_multi_parameter('Default', var)
            except configrecipe.ParameterNotFound:
                raise ParameterNotFound(var)
                
        else:
            raise ParameterNotFound(var)

        if value == 'NOT_REQUIRED':
            return None

        return value

    
