#!/usr/bin/env python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.

""" Main """

from common import iniparameterparser
from common import fileparameterparser
from common import cliuserinterface
from common import userinterface
from common import parameterlist
from common.logger import logger
from common.logger import LOGFILE
from common import configrecipe
from common import sectionloader
from common.section import SectionError, SectionSkipped, ParameterNotFound
from common import detect
from common import recipeutils as ru
import sys
import exceptions

__version__ = "$Rev: 36605 $"

TOOL_NAME = "Intel(R) Cluster Ready recipe installer"
TOOL_DESCRIPTION =  "Configures a Cluster configuration supported by an " +\
                    "Intel(R) Cluster Ready Reference Implementation and "+\
                    "turns it to be Intel(R) Cluster Ready Compliant"
TOOL_VERSION = "0.1"

MAX_SECTIONS = 1000


def remove_undefined_params(argv, paramlist):
    ''' Removes undefined parameters in the argv
        and returns the new string
    '''
    if not isinstance(paramlist, parameterlist.ParameterList):
        raise exceptions.TypeError()

    new_argv = []
    new_argv.append(argv[0])

    long_opts = paramlist.get_all_longopts()

    for i in range(len(long_opts)):
        long_opt_defined = "--" + long_opts[i] + "="

        for j in range(len(argv) - 1):
            if argv[j+1].find(long_opt_defined) != -1:
                new_argv.append(argv[j+1])

    return new_argv

def get_section_long_name(section, section_name):
    ''' Retrieves the name of the section '''
    try:
        section_text = getattr(section, 'section_name', section_name)
    except AttributeError:
        section_text = section_name
    return section_text

def remove_force_os_param():
    """ Remove force os and retrieve the args in a correct way"""
    forced_os = None

    args = sys.argv[:]
    for arg in args:
        if '--force_os=' in arg:
            forced_os = arg.split('=')[-1]
            if forced_os:
                args.remove('--force_os=%s' % forced_os)
            else:
                logger.error('Error using --forced_os option')
                sys.exit(1)
            continue
    return args, forced_os

def log_command_line_arguments():
    """Logs the command line arguments passed to the script """
    logger.debug("Command: %s" % " ".join(sys.argv))

def log_file_config(file_config):
    """Logs the contents of the file configuration """
    logger.debug("Configuration File: Configuration file contents: ")

    file_contents = file_config.return_config_file_contents()
    for line in file_contents:
        logger.debug("Configuration File: %s" % line.rstrip("\n"))

    logger.debug("Configuration File: End of file configuration")
    


def main():
    """ Main method """
    default_section = "Default"
    cli_param_list = parameterlist.ParameterList(default_section, \
        {'INI_CONFIG_FILE' : ['ini_file',\
            'Specify configuration file to use for the recipe',False],\
        'EXECUTE_SECTION' : ['section',\
            'Instruct the installer to execute only the specified section'\
            ,False],\
        })


    param_list = parameterlist.ParameterList()

    #Section definitions for parameters list
    section_filesection = "Sections"
    section_variable = 'SECTIONS'

    section_param_list = parameterlist.ParameterList(section_filesection,
                                                     {section_variable: 
                                                      [None,
                                                       'List of sections',
                                                       True],})

    param_list.concat(section_param_list)

    # Check if --forced-os options was passed to command line and pass
    # it to detect_os method

    forced_os = None

    args, forced_os = remove_force_os_param()

    try:
        prov = detect.detect_provisioning(detect.detect_os(forced_os))
    except detect.DetectionError, ex:
        logger.critical(str(ex))
        sys.exit(1)
    
    recipe_conf = configrecipe.RecipeConfiguration()
    
    # Loading Intel(R) Cluster Ready recipe sections 

    section_classes = sectionloader.import_recipe_sections()
   
    if section_classes == {}:
        logger.critical("No sections were loaded, nothing to be done")
        sys.exit(1)
    
    #Stores the section object of each section
    #key is the section name
    #value is the section object
    recipe_sections = {}

    #Stores the parameters defined for each section
    #key is the section name
    #value is the param list 
    section_paramlist = {}
   
    #Has a list of sections to execute in order
    section_execute_list = []
    
    for section_name in section_classes.keys():
        try:
            recipe_sections[section_name] = section_classes[section_name]\
                (prov, logger, recipe_conf)
            section_paramlist[section_name] = recipe_sections[section_name].\
                get_parameter_list()

        except exceptions.Exception, ex:
            logger.critical("Error initializing section %s: %s" %
                            (section_name, ex))
            sys.exit(1)
            
    # Pre gather the configuration file to obtain the list of sections 
    try:
        #Remove help if present
        if '--help' in args:
            args.remove('--help')
        args = remove_undefined_params(args, cli_param_list)

        pre_ui = cliuserinterface.CliInterface( args, \
            cli_param_list, TOOL_NAME, TOOL_DESCRIPTION, TOOL_VERSION)
    except userinterface.UserInterfaceExit:
        sys.exit(1)
   
    try:
        config_filename = pre_ui.get_parameter(default_section, \
            'INI_CONFIG_FILE')
        
    except userinterface.UserInterfaceErrors:
        config_filename = configrecipe.get_default_file_config_name()
        if config_filename is None:
            logger.error("Missing configuration file")
            pre_ui.help()
            sys.exit(1)
        

    try:
        logger.info("Reference Design scripts Configuration file: %s" \
                   % (config_filename))
        file_config = iniparameterparser.IniParameterParser(config_filename)
    except fileparameterparser.FileNotFound:
        logger.error("Could not open file %s\n" % (config_filename))
        sys.exit(1)
    except fileparameterparser.ParsingError, ex:
        logger.error(str(ex))
        sys.exit(1)


    try:
        single_section = pre_ui.get_parameter(default_section,'EXECUTE_SECTION')
          
        try:
            param_list.concat(section_paramlist[single_section])
            section_execute_list.append(single_section)
        except exceptions.KeyError:
            logger.critical("Section %s was not found in system" \
                % (single_section))
            sys.exit(1)
            

    except userinterface.UserInterfaceErrors:

        # Gather enabled sections
        try:
            sections_names = file_config.get_multi(section_filesection,
                                                   section_variable)
        except fileparameterparser.ParameterNotFound, ex:
            msg = "The variable SECTION is required and must be defined in" + \
                " the configuration file."
            logger.critical(msg)
            logger.debug(str(ex))
            sys.exit(1)

        # due sections_names is returned as a list we create an iterator
        iter_sections_names = sections_names.__iter__()

        for section_name in iter_sections_names:
            # Transform from dictionary to string
            section_name = section_name['name']
            try:
                # As the section was found, try to load the parameters
                param_list.concat(section_paramlist[section_name])
                section_execute_list.append(section_name)
                
            except exceptions.KeyError:
                logger.critical("Section %s was not found in system" %
                                (section_name))
                sys.exit(1)
            except parameterlist.ParameterListError, ex:
                logger.critical("Error building parameters list: %s" %
                                (str(ex)))
                sys.exit(1)
            
    # Get OS specific parameters and concatenate with global param_list
    param_list.concat(prov.opsys.get_parameter_list())


    # Read again the command line parameters and remove --force_os if it's found
    args, forced_os = remove_force_os_param()

    # Parse the parameters passed in command line  
    try:
        cli_param_list.concat(param_list)
        user_int = cliuserinterface.CliInterface( args, \
            cli_param_list, TOOL_NAME, TOOL_DESCRIPTION, TOOL_VERSION)
    except userinterface.UserInterfaceExit:
        sys.exit(1)
    except userinterface.InvalidParameters:
        sys.exit(1)
    
    
    log_command_line_arguments()
    log_file_config(file_config)

    try:
        recipe_conf.configure(user_int, file_config, param_list)
    except configrecipe.ParameterNotConfigured, ex:
        logger.critical(str(ex))
        print "Please, configure the required parameter"
        sys.exit(1)
    except configrecipe.DuplicatedLongOpts, ex:
        logger.critical(str(ex))
        sys.exit(1)
        
    # Sets OS configuration files paths
    prov.opsys.set_conf_files(recipe_conf)
    
    logger.info("The following sections will be executed %s" % 
        section_execute_list)

    #Pre load all the sections parameters
    for section_name in section_execute_list:
        section = recipe_sections[section_name]

        section_text = get_section_long_name(section, section_name)

        try:
            section.section_configurator()
        except ParameterNotFound, ex:
            print "Error loading required parameter for %s: %s"\
                 % (section_text, str(ex))
            sys.exit(1)
        except NotImplementedError:
            #If this function was not defined in the section, it will
            #be assumed that it was not required
            pass

    # Declare a variable to store the skipped sections
    skipped_sections = []
    # Check that all sections requirements are met 
    for section_name in section_execute_list:
        section = recipe_sections[section_name]

        section_text = get_section_long_name(section, section_name)

        try:
            section.check_requirements()
        except SectionError, ex:
            print "The prerequisites of the section %s were not met: %s" \
                % (section_text, str(ex))
            sys.exit(1)
        except SectionSkipped, ex:
            print "The %s will be skipped. Check log for details." \
                % section_text
            _msg = "Section %s skipped due: %s" % (section_text, str(ex))
            logger.debug(_msg)
            skipped_sections.append(section_name)
        except NotImplementedError:
            #If this function was not defined in the section, it will
            #be assumed that it was not required
            pass

    # Remove skipped sections from the section execute list
    for section_skipped in skipped_sections:
        for section_name in section_execute_list:
            if section_skipped == section_name:
                section_execute_list.pop(section_execute_list.index(section_name))

    for section_name in section_execute_list:
        section = recipe_sections[section_name]

        # try to get section's longer name defined in its attribute section_name
        # if not found use its name (e.g. ICRUser)
        section_text = get_section_long_name(section, section_name)

        try:
            section.section_dependency_check()
        except SectionError, ex:
            print "The dependencies required for section %s were not met: %s"\
                             % (section_text, str(ex))
            sys.exit(1)
        except NotImplementedError:
            #If this function was not defined in the section, it will
            #be assumed that it was not required
            pass
        except SectionSkipped, ex:
            print "The dependencies required for section %s are already met" % \
                (section_text)
            logger.debug(str(ex))

        try:
            ru.display_text('Running ' + section_text)
            section.run()
            print "%s:" % section_text,
            ru.cprint('Success', color='green')
        except SectionSkipped, ex:
            print "%s:" % section_text,
            ru.cprint('Skipped', color='yellow')
        except SectionError, ex:
            logger.critical(str(ex))
            print "%s:" % section_text,
            ru.cprint('Failed', color='red')
            print "Find the log file at: %s" % LOGFILE
            sys.exit(1)

    logger.info("Configuration succeeded") 
    print "Find the log file at: %s" % LOGFILE


if __name__ == "__main__":

    main()
