#!/usr/bin/env python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.


""" This section sets up Intel(R) Cluster Checker configuration files
    and optionally executes it.
"""

# Custom imports
from common.oslayer import FileCopyException
from common.section import Section, SectionError, ParameterNotFound
import common.recipeutils as ru

# Python imports
import exceptions
import os
import re

__version__ = "$Rev: 37814 $"

# Script version
VERSION = '1.2'
# Values by default

class ICRCLCK(Section):
    """ Installer class """

    def __init__(self, prov, mlogger, config):
        """ Initialize the object 
        """
        Section.__init__(self, prov, mlogger, config)

        self.section_name = "Intel(R) Cluster Checker"

        self._section = "ICRCLCK"
        self._dict = {
            'intel-clck-path': ['iclckpath', 'Path to Intel(R) Cluster Checker',
                                False],
            'intel-clck-version':['iclckver', 'Version of Intel(R) Cluster' \
                                      ' Checker',False], 
            'intel-nodelist': ['nodelist', 'The nodelist for Intel(R) Cluster' \
                                   ' Checker execution',False],
            'intel-clck-options': ['iclckoptions', 'The Intel(R) Cluster ' \
                                       'Checker option to execute',False], 
            'intel-clck-binary': ['iclckbinary', 'The Intel(R) Cluster Checker'\
                                      ' binary',False],
            'intel-clck-logs-path': ['iclcklogpath', 'The Intel(R) Cluster' \
                                         ' Checker log paths',False],
            'intel-clck-config': ['iclckconfig', 'The Intel(R) Cluster Checker'\
                                      ' configuration file',False], 
            'intel-clck-head-list': ['iclckheadlist', 'The Intel(R) Cluster'\
                                         ' Checker head rpm list ',False], 
            'intel-clck-compute-list': ['iclckcomputelist', 'The Intel(R) '\
                                            'Cluster Checker compute node rpm' \
                                            ' list',False],
            'intel-clck-conf-path': ['iconfpath', 'The Intel(R) Cluster Ready' \
                                         ' configuration Path',False],
            'execute-intel-clck': ['executeclck', 'Set to True to enable the ' \
                                       'execution of Intel (R) Cluster Checker',
                                   False]}
        
        self._defaultdict = {
            'INSTALL_DIR': ['install_dir',
                            'Intel(R) Cluster Ready installation directory',
                            True],
            'CLCK_LIC_PATH': [None,
                              'Path of the Intel(R) Cluster Checker license',
                              True],
            'HEAD_AS_COMPUTE': ['head-as-compute',
                                'Option to configure the Head node also as ' \
                                    'Compute node', False],
            'SKIP_IB': ['no-infiniband', 'Do not configure Infiniband', False],}
        
        self._iclck_path = ''
        self._iclck_version = ''
        self._iclck_binary = ''
        self._iclck_log = ''
        self._iclck_options = ''
        self._iclck_nodelist = ''
        self._iclck_config = ''
        self._iclck_head_list = ''
        self._iclck_compute_list = ''
        self._iclck_path_conf = ''
        self._iclck_license_path = ''
        self._icr_path_conf = ''
        self._iclck_execute = False
        self._intel_license_path = "/opt/intel/licenses/" 
        self._iclck_head_as_compute = False
        self._iclck_head_as_compute_config = "_head-as-compute.xml"
        self.skip_ib = False

    def _check_dependencies(self):
        """ Checks for required Intel(R) Cluster Checker Dependencies
        """
        
        self._logger.info("Beginning Checking Intel(R) Cluster Checker " + \
                              " Config Dependencies  ")
        
        if self._iclck_head_as_compute == "True":
            self._iclck_config = self._iclck_config[0:-4] + \
                self._iclck_head_as_compute_config

        config = self._prov.opsys.get_full_path(self._icr_path_conf,
                                                    self._iclck_config)

        if not self._prov.opsys.check_file(config):
            raise SectionError("The %s config file is required" % \
                                   (self._iclck_config)) 
        head_list = self._prov.opsys.get_full_path(self._icr_path_conf,
                                                   self._iclck_head_list)
        if not self._prov.opsys.check_file(head_list):
            raise  SectionError("The %s config file is required" % \
                                    (self._iclck_head_list)) 
        compute_list = self._prov.opsys.get_full_path(self._icr_path_conf,
                                                      self._iclck_compute_list)
        if not self._prov.opsys.check_file(compute_list):
            raise  SectionError("The %s config file is required" % \
                                    (self._iclck_head_list)) 
        if not self._prov.opsys.check_folder(self._iclck_path_conf):
            self._logger.info("The path %s does not exist" % 
                              (self._iclck_path_conf))
            self._prov.opsys.create_dir(self._iclck_path_conf)
            
            self._logger.info("Finished Intel(R) Cluster Checker Config "+\
                                  "Dependencies")
        
    def load_param(self):
        """Load the parameter defines at __init__ 
            must be execture before the run method
        """
        
        self._logger.info("Loading parameters for Intel(R) Cluster Checker " +\
                          "Section ")
		
        # If the param is registered with required=False it shouldn't throw an
        # exception
        # Check all the assigns

        try:
            self._iclck_path = self.get_param('intel-clck-path')
            self._iclck_version = self.get_param('intel-clck-version')
            self._iclck_path = ("%s/%s/" % \
                                    (self._iclck_path,self._iclck_version))
        except ParameterNotFound, ex:
            raise SectionError(str(ex))
        
        try:
            self._iclck_binary = self.get_param('intel-clck-binary')
        except ParameterNotFound, ex:
            raise SectionError(str(ex))
        
        try:
            self._iclck_log = self.get_param('intel-clck-logs-path')
        except ParameterNotFound, ex:
            raise SectionError(str(ex))
        
        try:
            self._iclck_options = self.get_param('intel-clck-options')
        except ParameterNotFound, ex:
            raise SectionError(str(ex))
        try:
            self._iclck_nodelist = self.get_param('intel-nodelist')
        except ParameterNotFound, ex:
            raise SectionError(str(ex))

        try:
            self._iclck_config = self.get_param('intel-clck-config')
        except ParameterNotFound, ex:
            raise SectionError(str(ex))
                
        try:
            self._iclck_head_list = self.get_param('intel-clck-head-list')
        except ParameterNotFound, ex:
            raise SectionError(str(ex))
            
        try:
            self._iclck_compute_list = self.get_param('intel-clck-compute-list')
        except ParameterNotFound, ex:
            raise SectionError(str(ex))
        
        try:
            self._iclck_path_conf = self.get_param('intel-clck-conf-path')
        except ParameterNotFound, ex:
            raise SectionError(str(ex))
                
        try:
            self._icr_path_conf = self.get_param('INSTALL_DIR') + '/etc'
        except ParameterNotFound, ex:
            raise SectionError(str(ex))
        
        try:
            self._iclck_license_path = self.get_param('CLCK_LIC_PATH') 
        except ParameterNotFound, ex:
            raise SectionError(str(ex))
        
        try:
            self._iclck_head_as_compute = self.get_param('HEAD_AS_COMPUTE')
        except ParameterNotFound, ex:
            self._iclck_head_as_compute = False

        #Validation of the value
        if not self._iclck_head_as_compute in [ "True", "False" ]:
            self._iclck_head_as_compute = False

        try:
            execute_clck = self.get_param('execute-intel-clck')
            
            #if the variable was defined as true, turn the boolean flag to true
            #Otherwise, it will be considered false.
            if execute_clck == "True":
                self._iclck_execute = True
            else:
                self._iclck_execute = False
                
        except ParameterNotFound, ex:
            #if the variable was not defined, will not execute clck by default
            self._iclck_execute = False

        # Check if Infiniband is skipped or not
        try:
            self.skip_ib = self.get_param('SKIP_IB')
        except ParameterNotFound:
            # If not set default to False
            self.skip_ib = False
        if not self.skip_ib in ['True','False']:
            # Default to False if garbage is entered
            self.skip_ib = False
        
        self._logger.info("Finished the parameters load")
    
    def _configure_license (self):
        '''This method checks and configures the license file for CLCK'''
        info = 'Check and copy of Intel(R) Cluster Checker license file'
        self._logger.info(info)
        
        #Brings all the files in the path of the license
        try:
            files = os.listdir(self._iclck_license_path)
        except os.error, ex:
            raise SectionError("Error executing command: %s" % str(ex))
          
        #Filter the valid licenses  
        license_files = []
        for filed in files:
            if ".lic" in filed:
                license_path = self._prov.opsys.get_full_path(
                               self._iclck_license_path, filed)
                if self._check_license_clck(license_path) == True:
                    license_files.append(filed)
        
        #Verify the number of valid licenses on the path
        if len(license_files) > 1:
            self._logger.debug("There is more than one valid license file." + \
                               "Looking for the newest one.")
            date = []
            for licensed in license_files:
                license_path = self._prov.opsys.get_full_path(
                               self._iclck_license_path, licensed)
                date.append(os.stat(license_path).st_mtime)
            
            #Order the list of dates
            date.sort()               
            
            #Look for the newest date
            for licensed in license_files:
                license_path = self._prov.opsys.get_full_path(
                              self._iclck_license_path, licensed)
                if os.stat(license_path).st_mtime == date[len(date)-1]:
                    validated_license = licensed
                    msg = "Intel(R) Cluster Checker license file validated: %s"\
                           % (validated_license)
                    self._logger.debug(msg)

        elif len(license_files) == 0:
            msg = "No Intel(R) Cluster Checker valid license file "+ \
                  "found at: %s" % (self._iclck_license_path)
            self._logger.info(msg)
            raise SectionError (msg)
        elif len(license_files) == 1:
            validated_license = license_files[0]
            msg = "Intel(R) Cluster Checker license file validated: %s" \
                   % (validated_license) 
            self._logger.debug(msg)  
        
        #Check the folder /opt/intel/licenses exists
        if not self._prov.opsys.check_folder(self._intel_license_path):
            self._logger.debug("The path %s does not exist" % \
                                (self._intel_license_path))
            self._prov.opsys.create_dir(self._intel_license_path)

        #Copy the license file
        try:
            lic_file_orig_path = self._prov.opsys.get_full_path(
                self._iclck_license_path, validated_license)
            self._prov.opsys.safe_cp(lic_file_orig_path, 
                                     self._intel_license_path)
        except FileCopyException, ex:
            raise SectionError("Error copying Intel(R) Cluster Checker"+ \
                                "license file: %s" % \
                                (validated_license))


    def _check_license_clck(self, license_file):
        """Verifies that the Intel(R) Cluster Checker license is valid"""
        #Pattern for parsing the strings
        lic_pattern = "COMPONENTS=IClsValL"

        lic_validation = False

        try:
            lic_file = open(license_file, 'r')
            for line in lic_file:
                if re.search(lic_pattern, line) != None:
                    lic_validation = True
        except exceptions.IOError, ex:
            self._logger.debug( str(ex))
            raise SectionError("Error opening license file : %s"  % str(ex))
        
        try:
            lic_file.close()
        except exceptions.IOError, ex:
            self._logger.debug(str(ex))
            raise SectionError("Error closing license file : %s"  % str(ex))
        
        return lic_validation


    def run(self):
        """ Main method, it will executed Intel(R) Cluster Checker according to
            the configuration specified in the configuration file or
            command line parameters.
        """

        try:
            self.load_param()
            self._check_dependencies()
            self._configure_license()
            self._set_configfiles()
            self._set_nodelist()
            # Add Infiniband test to the Intel Cluster Checker(R) configuration
            # file.
            self._modify_ib_conf()

        except SectionError, ex:
            self._logger.debug(ex)
            raise SectionError("There is no proper configuration " + \
                              "to run Intel(R) Cluster Checker") 

        try:
            self._logger.info("Adding to env PATH %s" % self._iclck_path)
            os.environ['PATH'] = os.path.pathsep.join([self._iclck_path, 
            os.environ['PATH']]) 
        except KeyError:
            os.environ['PATH'] = self._iclck_path
        
        intel_license_path = ('%s:/opt/intel/licenses' % (os.environ['HOME']))

        os.environ['CLCK_LOG_DIRECTORY'] = self._iclck_log
        os.environ['INTEL_LICENSE_FILE'] = intel_license_path
        self._source_clck()
        
        if self._iclck_execute:
            try:
                self._logger.info("Starting the Intel(R) Cluster Checker " + \
                                   "Execution %s %s" % 
                (self._iclck_binary, self._iclck_options))
                output = ru.execute_command("%s %s " % (self._iclck_binary, 
                    self._iclck_options))
                self._parser_clck_out(output)
                self._sales_report()
            except ru.CommandException, ex:  
                self._parser_clck_out('FAIL')
                self._logger.critical("Failed to execute %s/%s %s" 
                    % (self._iclck_path, self._iclck_binary, \
                       self._iclck_options))
                raise SectionError("Intel(R) Cluster Checker execution " +\
                                   "failed: %s" % str(ex))

    def _set_configfiles(self):
        """ Make sym link for Intel(R) Cluster Checker Config files
			config.xml, head.list, compute.list
        """
        config_name = "config.xml"
        head_name = "head.list"
        compute_name = "compute-node.list"
        
        config = self._prov.opsys.get_full_path(self._icr_path_conf, 
                self._iclck_config)
        conf_link = self._prov.opsys.get_full_path(self._iclck_path_conf, 
                config_name)
        head = self._prov.opsys.get_full_path(self._icr_path_conf, 
                self._iclck_head_list)
        head_link = self._prov.opsys.get_full_path (self._iclck_path_conf, 
                head_name)
        compute = self._prov.opsys.get_full_path(self._icr_path_conf, 
                self._iclck_compute_list)
        compute_link = self._prov.opsys.get_full_path (self._iclck_path_conf, 	
                compute_name)

        self._prov.opsys.link_file_to(config, conf_link)
        self._prov.opsys.link_file_to(head, head_link)
        self._prov.opsys.link_file_to(compute, compute_link)


    def _sales_report(self):
        """ Execute the Sales report of Intel(R) Cluster checker 
        """
        iclck_pattern = "config" 
        file_tmp = self._prov.opsys.get_last_file_modified(self._iclck_log,
                                                           iclck_pattern)
        file_tmp = file_tmp.replace('.out','.xml')
        file_xml = self._prov.opsys.get_full_path (self._iclck_log, file_tmp)
        self._logger.info("Starting the Intel(R) Cluster Checker " + \
            "Sales Report %s --sales-report %s "
             % (self._iclck_binary, file_xml))
        output = ru.execute_command("%s --sales-report %s " \
             % (self._iclck_binary, file_xml))

        # Try to get the filename of the generated file. If not found
        # print all the output.
        out_file = ''
        for line in output:
            if line.startswith('Created file:'):
                out_file = line.replace('Created file:', '').strip()
                break
        if out_file == '':
            out_file = output

        ru.display_text("The Sales report was generated at %s" % (out_file))


    def _parser_clck_out(self, output):
        """Check the output from Intel(R) Cluster Checker and print the overall 
           result
        """
        flag = False
        line_to_print = ''
        color = "red"
        for line in output:
            if "has Succeeded." in line:
                flag = True
                line_to_print = line
        
        if flag:
            ru.display_text("Intel(R) Cluster Checker %s" % (line_to_print))
        else:
            text_to = ("See Intel(R) Cluster Checker logs at %s"\
                    % (self._iclck_log))
            ru.display_text(text_to, color)
        
    def _set_nodelist(self):
        """ This method generates the nodelist file need for Intel(R)
            Cluster Checker execution
        """
        
        _icr_compute_patthern = 'compute'
        self._logger.info("Generating the %s" % self._iclck_nodelist)
        
        mpfile = open(self._iclck_nodelist, 'w')
        
        nodes = self._prov.get_nodes()
        for node in nodes:
            if _icr_compute_patthern in node:
                mpfile.write("%s\n" % (node))
            else:
                if self._iclck_head_as_compute == "True":
                    mpfile.write("%s #type: head,compute group:head_icr\n" % \
                                     (node))
                else:
                    mpfile.write("%s # type: head\n" %(node))
        
        mpfile.close()      
        self._logger.info("The %s was generated" % self._iclck_nodelist)   
        
    def _source_clck(self):
        """ Set the Intel(R) Cluster Checker bash environment
        """
        try:
            self._logger.info("Setting the Intel(R) Cluster Checker " + \
            "Var environment %s" %
            (self._iclck_binary))
            ru.execute_command("source %s/clckvars.sh" % (self._iclck_path))
        except ru.CommandException:
            self._logger.critical("Intel(R) Cluster Checker failed to " + \
            "execute %s/%s %s" %
            (self._iclck_path, self._iclck_binary, self._iclck_options))

    def _roll_back(self):
        """ This method allow to uninstall previously installed rpms in a
        correct way (remove them from DB and nodes for example).
        """
        pass

    def _modify_ib_conf(self):
        """ This method allow to enable or disable Infiniband configuration
        from the Intel Cluster Checker configuration file.
        """

        config = self._prov.opsys.get_full_path(self._icr_path_conf,
                                                self._iclck_config)

        if self.skip_ib == 'True':
            # Turn off Infiniband global device
            cmd = 'sed -i.bak -e \'s/\\(<fabric\\s*name=.*Infiniband.*\\)on">'\
                '/\\1off">/\' %s' % config
            try:
                self._logger.debug("Turn off Infiniband global configuration")
                ru.execute_command(cmd)
            except ru.CommandException, ex:
                msg = 'Fail turning off Infiniband configuration: %s' % str(ex)
                raise SectionError(msg)

            # Comment ipoib module inclusion
            cmd = 'sed -i.bak -e \'s/\\(^\\s*<include_module>\\s*ipoib\\s*' \
                '<\/include_module>\\)/<!--\\1-->/\' %s' % config
            try:
                self._logger.debug("Commenting ipoib test module inclusion")
                ru.execute_command(cmd)
            except ru.CommandException, ex:
                msg = 'Fail to comment ipoib test module inclusion: %s' % \
                    str(ex)
                raise SectionError(msg)

            # Uncomment dat_conf module exclusion
            cmd = 'sed -i.bak -e \'s/^\\s*<!--\\(\\s*<exclude_module>\\s*' \
                'dat_conf\\s*<\/exclude_module>\\)\\s*-->/\\1/\' %s' % config
            try:
                self._logger.debug("Uncomment dat_conf test module exclusion")
                ru.execute_command(cmd)
            except ru.CommandException, ex:
                msg = 'Fail to uncomment dat_conf test module exclusion: %s' \
                    % str(ex)
                raise SectionError(msg)
        else:
            # Turn on Infiniband global device            
            cmd = 'sed -i.bak -e \'s/\\(<fabric\\s* name=.*Infiniband.*\\)' \
                'off">/\\1on">/\' %s' % config
            try:
                self._logger.debug("Turn on Infiniband global configuration")
                ru.execute_command(cmd)
            except ru.CommandException, ex:
                msg = 'Fail turning on Infiniband configuration: %s' % str(ex)
                raise SectionError(msg)

            # Uncomment ipoib module inclusion
            cmd = 'sed -i.bak -e \'s/^\\s*<!--\\(\\s*<include_module>\\s*ipoib'\
                '\\s*<\/include_module>\\)\\s*-->/\\1/\' %s' % config
            try:
                self._logger.debug("Uncomment ipoib test module inclusion")
                ru.execute_command(cmd)
            except ru.CommandException, ex:
                msg = 'Fail to uncomment ipoib test module inclusion: %s' % \
                    str(ex)
                raise SectionError(msg)

            # Comment dat_conf module exclusion
            cmd = 'sed -i.bak -e \'s/\\(^\\s*<exclude_module>\\s*dat_conf\\s*'\
                '<\/exclude_module>\\)/<!--\\1-->/\' %s' % config
            try:
                self._logger.debug("Comment dat_conf test module exclusion")
                ru.execute_command(cmd)
            except ru.CommandException, ex:
                msg = 'Fail to Comment dat_conf test module exclusion: %s' % \
                    str(ex)
                raise SectionError(msg)

SECTION_PROPERTIES = {'name': 'ICRCLCK', 'class': ICRCLCK}
TOOL_NAME = "Intel(R) Cluster Checker configuration section"

TOOL_DESCRIPTION = "Configure and run Intel(R) Cluster Checker tool" 
                    

if __name__ == "__main__":
    print "This file should not be run alone"
