#!/usr/bin/env python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.

""" This module define the class to install Intel(R) Cluster Checker
"""

# Custom imports
from common.section import Section, SectionError, ParameterNotFound, \
    SectionSkipped
from common.oslayer import UntarException, MissingParameterException, \
    FileNotFoundException
from common import recipeutils as ru

# Python imports
import re
import os
import tempfile

__version__ = "$Rev: 35275 $"

# Exceptions
class EULARejected(SectionError):
    """ This exception is raised when the EULA is rejected
    """
    def __init__(self, msg):
        SectionError.__init__(self, msg)

# Main class
class ICRCLCKInstall(Section):
    """This class is in charge of installing Intel(R) Cluster Checker
    """

    def __init__(self, mprov, mlogger, mconfig):
        """ Initialize the object """
        Section.__init__(self, mprov, mlogger, mconfig)

        self.section_name = "Intel(R) Cluster Checker automated installation"

        self._section = "ICRCLCKInstall"
        # Define custom configuration variables
        self._dict = {
            'CLCK_PKG_NAME': ['clck-pkg',
                              'Package name of Intel(R) Cluster Checker',
                            True],}

        # Define default configuration variables
        self._defaultdict = {
            'TEMP_DIR': [None, 'Path of the temporary dir', False],
            'CLCK_LIC_PATH': [None,
                              'Path of the Intel(R) Cluster Checker license',
                              True],}

        # Define global vars
        self._clck_pkg = None
        self._clck_idir = None
        self._temp_dir = None
        self._clck_tdir = None
        self._clckre = None
        self._clck_silent = None
        self._clck_eula = None
        self._clck_lic = None

    def section_configurator(self):
        """ Loads all the required parameters and validates them.
        """
        # Retrieve CLCK package name
        try:
            self._clck_pkg = self.get_param("CLCK_PKG_NAME")
        except ParameterNotFound, ex:
            raise SectionError(str(ex))

        # Retrieve OS temp folder
        try:
            self._temp_dir = self.get_param("TEMP_DIR")
        except ParameterNotFound, ex:
            self._temp_dir = tempfile.gettempdir()

        # Retrieve CLCK license path
        try:
            self._clck_lic = self.get_param("CLCK_LIC_PATH")
        except ParameterNotFound, ex:
            raise SectionError(str(ex))

        # Set CLCK temp folder
        self._clck_tdir = os.path.join(self._temp_dir,
                                       os.path.basename(self._clck_pkg))

        self._clck_tdir = self._clck_tdir.split('.tgz')[0]

        # Define regular expression to match CLCK information
        #
        # e.g.
        # l_clck_p_1.6.002.tgz
        # ^------------------ OS
        #        ^----------- Sprint or Production
        #          ^--------- Version
        #              ^----- Release
        _regex = re.compile("(?P<clck_os>\w)_clck_(?P<clck_type>\w)_" +
                            "(?P<clck_version>\d\.\d).(?P<clck_release>\d{3})")
        self._clckre = _regex.search(self._clck_pkg)

        # Set CLCK install directory
        self._clck_idir = os.path.join('/opt/intel/clck',
                                       self._clckre.groupdict()['clck_version'])

        # Define silent configuration file path
        self._clck_silent = os.path.join(self._clck_tdir,
                                         'SilentInstallConfigFile.ini')

        # Define CLCK EULA file path
        self._clck_eula = os.path.join(self._clck_tdir, 'clckEULA.txt')

    def check_requirements(self):
        """All the checks that will be done before executing the scripts
        goes here.
        """

        # Check if CLCK is already installed
        if self._check_clck_rpm():
            _msg = "Intel(R) Cluster Checker already installed"
            raise SectionSkipped(_msg)

    def section_dependency_check(self):
        """All the checks that are needed in this section that should be
        solved in previous sections """
        # icr_user should be executed to copy CLCK license to /opt/intel/license
        pass

    def run(self):
        """ Overridden method
        """
        
        # Decompress the package
        self._step_1()
        
        # Read and accept the EULA
        # EULA acceptance is no longer required during the installation
        # of this package in the scripts so it was commented out. 
        # EULA acceptance is done when the package is downloaded.
        # self._step_2()
        
        # Create the configuration for silent installation 
        self._step_3()
        
        # Set install directory
        self._step_4()
        
        # Set CLCK license path
        self._step_5()
        
        # Install Intel(R) Cluster Checker
        self._step_6()

    def _step_1(self):
        """ Decompress the Intel(R) Cluster Checker package

        # cd /tmp
        # tar xzf $CLCK_PKG

        """

        # Untar the tarball
        try:
            self._prov.opsys.untar_file(self._clck_pkg, self._temp_dir,
                                        compress='')
        except UntarException, ex:
            self._logger.error("Intel(R) Cluster Checker package not found.")
            raise SectionError(str(ex))

    def _step_2(self):
        """ Read and accept the EULA thru editing silent config file

        # cd $CLCK_PKG
        # less clckEULA.txt
        """

        # Sets EULA file path
        _eula_file = self._clck_eula

        # Sets silent install configuration file path
        _silent_conf = self._clck_silent

        # Sets CLCK version
        _clck_ver = self._clckre.groupdict()['clck_version']

        # Sets CLCK OS
        if self._clckre.groupdict()['clck_os'] == 'l':
            _clck_os = 'Linux'
        else:
            _clck_os = 'Unknow'

        # Sets CLCK release
        _clck_release = _clck_ver + self._clckre.groupdict()['clck_type'] \
                        + '-' + self._clckre.groupdict()['clck_release']

        # Define CLCK EULA warning
        _clck_warn = '''
--------------------------------------------------------------------------------
Please carefully read the following license agreement for
Intel(R) Cluster Checker %s for %s* version %s.
Prior to installing the software you will be asked to agree to the terms and
conditions of the following license agreement.
--------------------------------------------------------------------------------'''\
        % (_clck_ver, _clck_os, _clck_release)

        # Show EULA warning
        self._logger.info(_clck_warn)
        raw_input('Press ENTER to continue...')

        # Show EULA
        try:
            ru.show_multipage_file(_eula_file)
        except ru.MultiPageException, ex:
            _msg = "Error trying to show Intel(R) Cluster Checker EULA"
            self._logger.error(_msg)
            raise SectionError(str(ex))

        # Checks for EULA answer
        _eula_acceptance = None

        while _eula_acceptance != 'accept' and _eula_acceptance != 'reject':
            try:
                self._logger.info('Do you agree to be bound by the terms and '\
                                      + 'conditions of this license agreement?')
                _eula_acceptance = raw_input(
                    "Enter 'accept' to continue, 'reject' to exit: " +
                    " (accept/reject): ")
                self._logger.debug('User EULA answer: %s' % _eula_acceptance)
            except EOFError, ex:
                _msg = "An error occurred trying to read the answer"
                self._logger.error(_msg)
                raise SectionError(str(ex))

        # If EULA was rejected raise an exception
        if _eula_acceptance != 'accept':
            _msg = 'You must accept EULA to install Intel(R) Cluster Checker'
            raise EULARejected(_msg)

    def _step_3(self):
        """ Create the configuration for silent installation 
        
        Note: By making modifications in the silent install configuration file
        such that EULA=accept, you agree to the terms and conditions listed in
        clckEULA.txt.

        # sed -i -e 's/\(EULA=\)reject/\1accept/' $CLCK_SILENT_CONFIG
        # Configures silent install configuration file with EULA answer

        """

        # Sets silent install configuration file path
        _silent_conf = self._clck_silent

        try:
            self._prov.opsys.set_file_parameter(_silent_conf, 'EULA',
                                                'accept',
                                                uncomment=True)
        except MissingParameterException, ex:
            raise SectionError(str(ex))
        except FileNotFoundException, ex:
            raise SectionError(str(ex))


    def _step_4(self):
        """ Set the installation dir

        # sed -i -e 's/\(INSTALLDIR=\).*/\1$CLCK_INST_DIR/' $CLCK_SILENT_CONFIG

        """

        # Sets silent install configuration file path
        _silent_conf = self._clck_silent

        # Set CLCK install directory in silent configuration file
        try:
            self._prov.opsys.set_file_parameter(_silent_conf, 'INSTALLDIR',
                                                self._clck_idir, uncomment=True)
        except MissingParameterException, ex:
            raise SectionError(str(ex))
        except FileNotFoundException, ex:
            raise SectionError(str(ex))

    def _step_5(self):
        """ Set the license path

        # sed -i -e 's/\(LICENSEPATH=\).*/\1$CLCK_LIC_DIR/' $CLCK_SILENT_CONFIG

        """

        # Sets silent install configuration file path
        _silent_conf = self._clck_silent

        # Set CLCK install directory in silent configuration file
        try:
            self._prov.opsys.set_file_parameter(_silent_conf, 'LICENSEPATH',
                                                self._clck_lic, uncomment=True)
        except MissingParameterException, ex:
            raise SectionError(str(ex))
        except FileNotFoundException, ex:
            raise SectionError(str(ex))

    def _step_6(self):
        """ Install Intel(R) Cluster Checker

        # ./install.sh --silent=$CLCK_SILENT_CONFIG

        """
        # Sets silent install configuration file path
        _silent_conf = self._clck_silent

        # Run Intel(R) Cluster Checker unattended installation
        _cmd = os.path.join(self._clck_tdir,
                            'install.sh --silent=%s' % _silent_conf)

        self._logger.info('- Running Intel(R) Cluster Checker installer...')

        try:
            ru.execute_command(_cmd)
        except ru.CommandException, ex:
            self._logger.error("Error:")
            raise SectionError(str(ex))

    def _check_clck_rpm(self):
        """ Checks if the CLCK RPM package is installed
        """

        _clck_rpm = 'intel-clck-em64t-'
        _clck_rpm += self._clckre.groupdict()['clck_version']
        _clck_rpm += self._clckre.groupdict()['clck_type']
        _clck_rpm += '-' + self._clckre.groupdict()['clck_release']

        return self._prov.opsys.has_package(_clck_rpm)

    def __del__(self):
        """Clean temporary folders created by the section
        """

        # If temporary folder was created remove it
        if self._clck_tdir and self._prov.opsys.check_folder(self._clck_tdir):
            self._logger.debug("Removing temporal folder %s" % self._clck_tdir)

            for root, dirs, files in os.walk(self._clck_tdir, topdown=False):
                for name in files:
                    os.remove(os.path.join(root, name))
                for name in dirs:
                    os.rmdir(os.path.join(root, name))

            os.rmdir(self._clck_tdir)

SECTION_PROPERTIES = {'name': 'ICRCLCKInstall', 'class': ICRCLCKInstall}
TOOL_NAME = "Intel(R) Cluster Checker automated installation"
TOOL_DESCRIPTION = "This section installs the Intel(R) Cluster Checker tool \
without user intervention"

if __name__ == "__main__":
    print "This file should not be run alone"
