#!/usr/bin/env python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.

""" This script sets Ethernet Driver Parameter HPC BKM configuration
"""

# Python imports
import os
import re
import time

# Custom imports
from common.section import Section, SectionError
from common.provisioning import ProvisioningException
from common.oslayer import ServiceError, ModReloadError
from common.oslayer import FileCreationException, DirCreationException

__version__ = "$Rev: 37814 $"

class ICREthernet(Section):
    """Section to set Ethernet Driver Parameter HPC BKM configuration
    """
    
    def __init__(self, mprov, mlogger, mconfig):
        """ Initialize the object """
        Section.__init__(self, mprov, mlogger, mconfig)
        
        self.section_name = "Intel(R) Cluster Ready Ethernet BKM section"
        
        self._section = "ICREthernet"
        self._dict = {
            'BKM_ETHERNET_SCRIPT':  ['eth_script_name', 'name of the script '\
                                         'that should be executed on nodes.',
                                     True],
            'BKM_LIMITS_SCRIPT':  ['limits_script', 'name of the limits script'\
                                       ' to be executed on nodes.', True],
            'DRIVERS':  [None, 'List of drivers to check', True]
            }

        self._defaultdict = {
            'compute-icr-nodegroup': ['nodegroup', 'Intel(R) Cluster Ready ' \
                                          'compliant compute nodegroup name',
                                      True],
            'INSTALL_DIR': ['install_dir', 'Intel(R) Cluster Ready '\
                                'installation directory', True],}

        self._drivers = None
        self._modprobef = None
        self._limitsf = None
        self._ethernets = None
        self._limitss = None
        self._compute_ng = None
        self._install_dir = None

    def section_configurator(self):
        """ Loads all the required parameters and validates them.
        """
	
        # Ethernet drivers
        self._drivers = self.get_multi_param('DRIVERS')

        # modprobe.conf file
        self._modprobef = self._prov.opsys.MODPRB_FILE

        # limits.conf file
        self._limitsf = self._prov.opsys.LIMITS_FILE

        # icr_ethernet.sh script
        self._ethernets = self.get_param('BKM_ETHERNET_SCRIPT')

        # icr_limits.sh script
        self._limitss = self.get_param('BKM_LIMITS_SCRIPT')

        # Retrieve compute node nodegroup
        self._compute_ng = self.get_param('compute-icr-nodegroup')

        # Retrieve reference framework installation directory
        self._install_dir = self.get_param('INSTALL_DIR')

    def check_requirements(self):
        """All the checks that will be done before executing the scripts goes
        here.
        """

        # Check if icr_ethernet.sh script exist
        _file = os.path.join(self._install_dir, self._bash_scripts_d)
        _file = os.path.join(_file, self._ethernets)
        if not self._prov.opsys.check_file(_file):
            _msg = 'The file %s is required for this section' % _file
            raise SectionError(_msg)

        # Check if icr_limits.sh script exist
        _file = os.path.join(self._install_dir, self._bash_scripts_d)
        _file = os.path.join(_file, self._limitss)
        if not self._prov.opsys.check_file(_file):
            _msg = 'The file %s is required for this section' % _file
            raise SectionError(_msg)

        # Check if <PATH/modprobe.conf> file exists, if no create it
        if not self._prov.opsys.check_folder(os.path.dirname(self._modprobef)):
            # Create the dir modprobe.d
            try:
                self._prov.opsys.create_dir(os.path.dirname(self._modprobef))
                _msg = "Directory %s created" % os.path.dirname(self._modprobef) 
                self._logger.debug(_msg)
            except DirCreationException, ex:
                raise SectionError(ex)

        if not self._prov.opsys.check_file(self._modprobef):
            # Create the file in the location required
            try:
                self._prov.opsys.create_file(self._modprobef)
                _msg = "File %s created" % self._modprobef
                self._logger.debug(_msg)
            except FileCreationException, ex:
                raise SectionError(ex)
    
    def section_dependency_check(self):
        """All the checks that are needed in this section that should be solved
        in previous sections """
        pass
    
    def run(self):
        """ Overridden method
        """
        # Add BKMs to modprobe configuration file
        self._step_1()
        # Add BKMs to limits configuration file
        self._step_2()
        # Disable irqbalance service
        self._step_3()
        # Reload network service and ethernet module
        self._step_4()
        # Restart dhcpd and nfs services
        self._step_5()
        # Add BKMs to compute nodes
        self._step_6()
        # Synchronize configuration and rebuild compute nodes image
        self._step_7()

    
    def _step_1(self):
        """ Edit modprobe.conf file to configure Interrupt Throttle Rate for
        ethernet drivers.
        """

        # Local list of driver/s
        drivers = []
        drivers_ok = []
        drv = ''
        param = "InterruptThrottleRate"
        
        # Retrieve the drivers
        self._logger.info('- Retrieving driver list to configure')
        drivers = [ drv['name'] for drv in self._drivers ]
        
        for drv in drivers:
            # Retrieve parameters
            if self._prov.opsys.has_module_param(drv, param):
                drivers_ok.append(drv)

        # Edit modprobe.conf
        self._logger.info('Configuring drivers:')
        if drivers_ok:
            try:
                self._prov.set_bkm_ethernet(drivers_ok, self._modprobef)
            except ProvisioningException, ex:
                raise SectionError(str(ex))
        else:
            self._logger.info('No module has to be configured')

    def _step_2(self):
        """ Add BKMs to limits configuration file """

        # Local list of vars
        _file = self._limitsf
        _reg_match_1 = False
        _reg_match_2 = False

        self._logger.info('Configuring limits.conf:')

        try:
            _fd = open(_file)
            _lines = _fd.readlines()
            _fd.close()
        except IOError, ex:
            raise SectionError(str(ex))
        
        # Regular expression to match memlock configuration
        # e.g.
        # *     soft   memlock 2000000 # match!
        # *     hard   memlock 2000000 # match!
        
        _regex_1 = re.compile("^\*\s*soft\s*memlock\s*2000000")
        _regex_2 = re.compile("^\*\s*hard\s*memlock\s*2000000")
        
        try:
            _fd = open(_file, 'w')
        except IOError, ex:
            raise SectionError(str(ex))
        
        for _line in _lines:
            # Check soft memlock configuration
            if _regex_1.match(_line):
                _reg_match_1 = True
            # Check hard memlock configuration
            if _regex_2.match(_line):
                _reg_match_2 = True

            _fd.write(_line)

        # Set soft memlock configuration
        if not _reg_match_1:
            _fd.write('# soft memlock configuration\n')
            _fd.write('*     soft   memlock 2000000\n')
            self._logger.info('- soft memlock set to 2000000')
        else:
            self._logger.info('- soft memlock already configured')

        # Set hard memlock configuration
        if not _reg_match_2:
            _fd.write('# hard memlock configuration\n')
            _fd.write('*     hard   memlock 2000000\n')
            self._logger.info('- hard memlock set to 2000000')
        else:
            self._logger.info('- hard memlock already configured')

        _fd.close()


    def _step_3(self):
        """ Disable irqbalance service """

        self._logger.info('Configuring irqbalance service:')

        try:
            if self._prov.opsys.service_running('irqbalance'):
                self._prov.opsys.service_stop('irqbalance')
            self._logger.info('- service stopped')
        except ServiceError, ex:
            raise SectionError(str(ex))
            
        try:
            self._prov.opsys.service_disable('irqbalance')
            self._logger.info('- service disabled')
        except ServiceError, ex:
            raise SectionError(str(ex))

    def _step_4(self):
        """ Reload network service and ethernet module """

        self._logger.info('Configuring network service:')
       
        try:
            self._prov.opsys.service_stop('network')
            self._logger.info('- service stopped')
        except ServiceError, ex:
            raise SectionError(str(ex))

        # Local list of driver/s
        drivers = []
        drv = ''
        
        # Retrieve the drivers
        self._logger.info('Reloading network drivers:')
        drivers = [ drv['name'] for drv in self._drivers ]

        for drv in drivers:
            try:
                self._prov.opsys.reload_module(drv)
            except ModReloadError, ex:
                raise SectionError(str(ex))

        # Wait for 5 seconds for  ethernet drivers to finish loading
        time.sleep(5)

        try:
            self._prov.opsys.service_start('network')
            self._logger.info('- service started')
        except ServiceError, ex:
            raise SectionError(str(ex))


    def _step_5(self):
        """ Restart dhcpd and nfs services """

        self._logger.info('- Restarting dhcpd')

        try:
            self._prov.opsys.service_restart('dhcpd')
        except ServiceError, ex:
            raise SectionError(str(ex))

        self._logger.info('- Restarting nfs')

        try:
            self._prov.opsys.service_restart('nfs')
        except ServiceError, ex:
            raise SectionError(str(ex))

    def _step_6(self):
        """ Add BKMs to compute nodes """

        self._logger.info('Applying BKMs to compute nodes:')

        # Local variables
        _bash_scripts = self._bash_scripts_d

        _path_ethernet = os.path.join(self._install_dir, 
                                      _bash_scripts,
                                      self._ethernets)

        _path_limits = os.path.join(self._install_dir,
                                    _bash_scripts,
                                    self._limitss)

        # Edit modprobe.conf configuring script to set the drivers
        _file  = _path_ethernet

        # Set the string for variable replacement
        self._logger.info('- Retrieving drivers list to configure')
        drivers = [ drv['name'] for drv in self._drivers ]
        
        # Set the list to a string
        _drivers = ' '.join(drivers)

        self._logger.info('- Editing %s:' % _file)

        # Edit the icr_ethernet file
        try:
            _fd = open(_file)
            _lines = _fd.readlines()
            _fd.close()
        except IOError, ex:
            raise SectionError(str(ex))
        
        # Regular expression to match DRIVERS="FIND"
        # e.g.
        # #DRIVERS="FIND"  # don't match!
        # DRIVERS="FIND" # match!
        
        _regex = re.compile("^DRIVERS=.FIND.$")
        
        try:
            _fd = open(_file, 'w')
        except IOError, ex:
            raise SectionError(str(ex))
        
        for _line in _lines:
            # Check DRIVERS variable
            if _regex.match(_line):
                _line = _line.replace('FIND','%s' % _drivers)
                
            _fd.write(_line)

        _fd.close()


        # Add scripts to compute nodes
        try:
            self._prov.add_script(_path_ethernet, self._compute_ng)
            self._logger.info('- %s added' % self._ethernets)
        except ProvisioningException, ex:
            raise SectionError(str(ex))

        try:
            self._prov.add_script(_path_limits, self._compute_ng)
            self._logger.info('- %s added' % self._limitss)
        except ProvisioningException, ex:
            raise SectionError(str(ex))


    def _step_7(self):
        """ Synchronize configuration and rebuild compute nodes image """

        self._logger.info("- Updating provisioning repository")
        self._prov.update_repo()

    
SECTION_PROPERTIES = {'name': 'ICREthernet', 'class': ICREthernet}
TOOL_NAME = "Intel(R) Cluster Ready Ethernet BKM section"
TOOL_DESCRIPTION = "Configures ethernet network with BKM parameters"

if __name__ == "__main__":
    print "This file should not be run alone"
