#!/usr/bin/env python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.

""" This section configures the InfiniBand interfaces
"""

# Custom imports
from common.section import Section, SectionError, SectionSkipped
from common.section import ParameterNotFound
from common.oslayer import FileNotFoundException
from common.provisioning import ProvisioningException
import common.recipeutils as ru

# Python imports
import os
import time
import tempfile

__version__ = "$Rev: 37960 $"

# Execute BKMs scripts
def execute_script(script_name, script_title, logger):
    """ Execute one of the bash scripts """

    logger.info('Applying ' + script_title)

    path = __file__.split(os.path.sep)[:-1]
    script = os.path.sep.join( path + ['bash_scripts', script_name] )

    if not os.path.isfile(script):
        raise SectionError('%s is not a valid script file for %s' % \
                          (script, script_title))

    os.system('chmod +x %s' % script)

    try:
        ru.execute_command(script)
    except ru.CommandException, ex:
        logger.debug(ex)
        raise SectionError('Error applying %s.' % script_title)


class ICRIBConf(Section):
    """ Class to configure InfiniBand adapter.
    Currently support IPoIB configuration only.
    """

    def __init__(self, mprov, mlogger, mconfig):
        """ Initialize the object """
        Section.__init__(self, mprov, mlogger, mconfig)

        self.section_name = "Intel(R) Cluster Ready InfiniBand section"

        self._section = "ICRIBConf"

        # Recheck the BKM scripts functionality to implement in code
        self._dict = {'BKM_SYSCTL_SCRIPT':
                          ['sysctl_script', 'sysctl BKM script.',True],
                      'BKM_OFED_SCRIPT':
                          ['ofed_script', 'OFED BKM script.',True]}

        self._defaultdict = {
            'IB0_IP': ['ib0_ip', 'ib0 InfiniBand ip address', False],
            'IB0_NM': ['ib0_netmask', 'ib0 netmask address', False],
            'IB0_NETNAME': ['ib0_netname', 'ib0 netname', False],
            'compute-icr-nodegroup': ['nodegroup',
                                      'Intel(R) Cluster Ready compliant ' +
                                      'compute nodegroup name', True],
            'SKIP_IB': ['no-infiniband', 'Do not configure Infiniband', False],}

        self.ib_iface = 'ib0'
        self.eth_iface = 'eth0'
        self.ib0_ip = None
        self.ib0_nm = None
        self.ib0_netname = None
        self.ib0_subnet = None
        self.bkm_sysctl = None
        self.bkm_ofed = None
        self.compute_ng = None
        self.autoconfig = False
        self.skip_ib = False

    def section_configurator(self):
        """ Loads all the required parameters and validates them.
        """

        try:
            self.ib0_ip = self.get_param('IB0_IP')
        except ParameterNotFound:
            # Try to retrieve ip from the eth0 configuration
            eth0 = self._prov.opsys.device_info(self.eth_iface)
            eth0_ip_addr = eth0.get('inet addr', '')
            eth0_nm = eth0.get('Mask', '')

            if not ru.valid_address(eth0_ip_addr):
                raise SectionError('Error in %s (IP: %s)' % 
                                   (self.eth_iface, eth0_ip_addr))

            try:
                # Check for next subnet starting from eth0
                self.ib0_ip = ru.next_subnet_first_host(eth0_ip_addr,
                                                        eth0_nm)
            
                # Retrieve the subnet to check if it is used
                ip_network = self._prov.get_network_from_ip(self.ib0_ip)

                # Check if the subnet is in use
                while ip_network != None:
                    self._logger.debug("Network %s/%s already in use. Trying \
                                       next..." % (ip_network['network'],
                                                   ip_network['subnet']))
                    self.ib0_ip = ru.next_subnet_first_host(self.ib0_ip,
                                                            eth0_nm)
                    ip_network = self._prov.get_network_from_ip(self.ib0_ip)
            except Exception, ex:
                self._logger.debug(ex)
                _msg = "Could not automatically generate an ib0 address" + \
                    " from the eth0 network configuration. Please specify " + \
                    "the desired %s ip and netmask manually in the icr.ini" + \
                    " file." % self.ib_iface
                raise SectionError(_msg)
            # Set to True to tell that icr.ini configuration was overridden
            self.autoconfig = True

        try:
            self.ib0_nm = self.get_param('IB0_NM')
        except ParameterNotFound:
            # Try to retrieve netmask from the eth0 configuration
            self.ib0_nm = \
                self._prov.opsys.device_info(self.eth_iface).get('Mask', '')
            # Set to True to tell that icr.ini configuration was overridden
            self.autoconfig = True
            
        try:
            self.ib0_netname = self.get_param('IB0_NETNAME')
        except ParameterNotFound:
            # Set default name
            self.ib0_netname = 'ipoib'
            # Set to True to tell that icr.ini configuration was overridden
            self.autoconfig = True

        try:
            self.ib0_subnet = ru.ip_network(self.ib0_ip, self.ib0_nm)
        except ValueError:
            raise SectionError("Subnet address can not be calculated")

        try:
            self.bkm_sysctl = self.get_param('BKM_SYSCTL_SCRIPT')
        except ParameterNotFound:
            self._logger.debug("SYSCTL BKM script is not set.")

        try:
            self.bkm_ofed = self.get_param('BKM_OFED_SCRIPT')
        except ParameterNotFound:
            self._logger.debug("OFED BKM script is not set.")

        try:
            self.compute_ng = self.get_param('compute-icr-nodegroup')
        except ParameterNotFound:
            raise SectionError('Compute nodegroup is not defined')

        try:
            self.skip_ib = self.get_param('SKIP_IB')
        except ParameterNotFound:
            # If not set default to False
            self.skip_ib = False
        if not self.skip_ib in ['True','False']:
            # Default to False if garbage is entered
            self.skip_ib = False

    def check_requirements(self):
        """All the checks that will be done before executing the scripts goes
        here.
        """

        # Check if this section need to run or should be skipped
        if self.skip_ib == 'True':
            raise SectionSkipped("Infiniband configuration disable.")

        # Check if interface is already added into the database
        if self._prov.has_network_device(self.ib_iface):
            _msg = "Infiniband interface already added to database"
            self._logger.warning(_msg)

        # Check if interface is already configured with the provided config
        ib_configured = False
        try:
            ib_configured = self._prov.ib_dev_configured(self.ib_iface,
                                                         self.ib0_ip,
                                                         self.ib0_nm)
        except NotImplementedError, ex:
            self._logger.warning('Error checking if device is configured!')
            self._logger.debug(str(ex))

        if ib_configured:
            _msg = '%s device already configured with ip %s and netmask %s' % \
            (self.ib_iface, self.ib0_ip, self.ib0_nm)
            self._logger.warning(_msg)
            raise SectionSkipped('%s device already configured' % self.ib_iface)
        else:
            # Check if interface is configured with a different config
            dev_network = self._prov.get_network_from_device(self.ib_iface)
            if dev_network != None:
                raise SectionError('%s device is already configured with a \
different configuration than the provided one. The device and its network must \
be removed from the provisioning system database first.' % self.ib_iface)

    def section_dependency_check(self):
        """All the checks that are needed in this section that should be solved
        in previous sections """

        # Infiniband device check
        if len(self._prov.opsys.ib_devices()) == 0:
            _msg = "No supported InfiniBand adapter available."
            raise SectionSkipped(_msg)

        # Check that we have got valid addresses
        if not ru.valid_address(self.ib0_ip):
            _msg = "The IP address %s is not valid" % str(self.ib0_ip)
            raise SectionError(_msg)

        if not ru.valid_address(self.ib0_nm, addrtype='netmask'):
            _msg = "The netmask address %s is not valid" % str(self.ib0_nm)
            raise SectionError(_msg)

    def run(self):
        """ Overridden method
        """
        # Add Infiniband network to provisioning system
        self._step_1()
        # Configure head node Infiniband interface
        self._step_2()
        # Setup configuration files
        self._step_3()
        # Configure and start related services
        self._step_4()
        # Configure compute nodes Infiniband interface
        self._step_5()
        # Execute BKMs
        self._step_6()


    def _step_1(self):
        """ Add Infiniband network to provisioning system """

        # Tell user which configuration is used
        if self.autoconfig:
            self._logger.info("Automatic generated configuration...")
        else:
            self._logger.info("Using entered configuration values...")

        # Print information
        self._logger.info("Interface:  " + self.ib_iface)
        self._logger.info("Net name:   " + self.ib0_netname)
        self._logger.info("IP address: " + self.ib0_ip)
        self._logger.info("Netmask:    " + self.ib0_nm)
        self._logger.info("Subnet:     " + self.ib0_subnet)

        # Add the network
        try:
            self._prov.add_network(self.ib0_netname, self.ib0_ip,
                                   self.ib0_nm, self.ib_iface)
        except ProvisioningException, ex:
            self._logger.debug(ex)
            raise SectionError('Error adding network %s.' % self.ib0_netname)

    def _step_2(self):
        """ Configure head node Infiniband interface """

        # Add the network device.
        try:
            self._prov.add_device(dev=self.ib_iface,
                                  ip_addr=self.ib0_ip,
                                  netmask=self.ib0_nm,
                                  netname=self.ib0_netname)
        except ProvisioningException, ex:
            self._logger.debug(ex)
            raise SectionError('Error adding %s device to database.' % 
                               self.ib_iface)

        # Configure the network device.
        try:
            self._prov.configure_device(dev=self.ib_iface,
                                        ip_addr=self.ib0_ip,
                                        netmask=self.ib0_nm,
                                        netname=self.ib0_netname)
        except ProvisioningException, ex:
            self._logger.debug(ex)
            raise SectionError('Error configuring %s device' % self.ib_iface)

        # Write Infiniband operating system configuration file for ib1
        if self._prov.opsys.device_info('ib1'):
            self._logger.info("Configuring ib1 to be disabled on boot")
            # Generate the ib1 network configuration script.
            # This network will be configured to down by default
            try:
                self._prov.configure_device('ib1', disable=True)
            except ProvisioningException, ex:
                self._logger.debug(ex)
                raise SectionError('Error disabling ib1')

    def _step_3(self):
        """ Setup configuration files  """

        # Modify infiniband configuration files
        try:
            self._prov.setup_ib_files()
        except ProvisioningException, ex:
            self._logger.debug(ex)
            raise SectionError('Error setting up %s configuration files' % 
                               self.ib_iface)
        except FileNotFoundException, ex:
            self._logger.debug(ex)
            raise SectionError('Error setting up %s configuration files' % 
                               self.ib_iface)

        # Sync network
        try:
            self._prov.sync_network()
        except ProvisioningException, ex:
            self._logger.debug(ex)
            raise SectionError('Error synchronizing network configuration')

        # Bring up device
        try:
            self._prov.bring_up_device(self.ib_iface)
        except ProvisioningException, ex:
            self._logger.debug(ex)
            raise SectionError('Error bringing up device %s' % self.ib_iface)

    def _step_4(self):
        """ Configure and start related services  """

        # Registers and starts Infiniband services
        try:
            self._prov.setup_ib_services()
        except ProvisioningException, ex:
            self._logger.debug(ex)
            raise SectionError('Error setting up Infiniband services')


        # Validates that the network
        if not self._prov.network_configured(self.ib0_netname):
            raise SectionError("The %s interface could not be added to the" +
                               " provisioning system database" % self.ib_iface)

        # Validates that the ip address configured in ib0 is up and working
        conf_ip = None
        conf_ip = self._prov.opsys.device_info(self.ib_iface).get('inet addr')
        tries = 3

        # Wait some seconds and get info again in case we need to wait
        # for the device to get up
        while conf_ip != self.ib0_ip and tries > 0:
            time.sleep(5)
            self._logger.debug("Checking %s configuration (%s != %s) retrying."\
                                   % (self.ib_iface, conf_ip, self.ib0_ip))
            conf_ip = self._prov.opsys.device_info(self.ib_iface).get( \
                'inet addr')
            tries = tries - 1

        if conf_ip != self.ib0_ip:
            raise SectionError("%s could not be configured (%s != %s)" %
                               (self.ib_iface, conf_ip, self.ib0_ip))

    def _step_5(self):
        """ Configure compute nodes Infiniband interface """

        # Create a temporary file to host the configuration script
        _tmp_script = tempfile.mkstemp()[1]

        try:
            _fd = open(_tmp_script, 'w')
        except IOError, ex:
            raise SectionError(str(ex))

        # This lines configure IPoIB on compute nodes
        _fd.write('#!/bin/bash\n')
        _fd.write('cat %seth0 | grep -v HWADDR | grep -v MTU > %sib0\n' %
                  (self._prov.opsys.IFCFG_FILE, self._prov.opsys.IFCFG_FILE))
        _fd.write('sed -i -e s/eth0/ib0/ %sib0\n' % self._prov.opsys.IFCFG_FILE)

        # Remove host address form network address
        subnet = ''
        subnet = self.ib0_ip.split('.')[:-1]
        subnet = '.'.join(subnet)

        _fd.write('my_host=$(cat %sib0 | grep IPADDR | cut -d"." -f4)\n' % \
                      self._prov.opsys.IFCFG_FILE)

        _fd.write( \
            'sed -i -e "s/IPADDR=.*\(\..*\)/IPADDR=%s\.$my_host/" %sib0 \n' % \
                (subnet, self._prov.opsys.IFCFG_FILE ))

        _fd.write('\n')
        _fd.close()

        # Update again to leave extend-compute file updated
        self._logger.info("- Updating compute-node configuration")

        # Add a script to configure Infiniband in compute nodes
        try:
            #self._prov.add_script(_tmp_script, self.compute_ng)
            pass
        except ProvisioningException, ex:
            _msg = "The compute node configuration script could not be added"
            self._logger.critical(str(ex))
            raise SectionError(_msg)

        # Enable infiniband interface if available
        self._prov.link_network(self.ib_iface, self.compute_ng) 

        # Synchronize configuration
        self._prov.sync(self.compute_ng)
        self._prov.update_repo()

        # Remove temporary file created
        os.remove(_tmp_script)

    def _step_6(self):
        """ Execute BKMs """

        ### Execute and copy ofed bkm bash script
        if self.bkm_ofed != None:
            execute_script(self.bkm_ofed, 'ofed BKM', self._logger)

        if self.bkm_ofed != None and self.compute_ng != None:
            path = __file__.split(os.path.sep)[:-1]
            script = os.path.sep.join( path + ['bash_scripts', self.bkm_ofed] )
            try:
                self._prov.add_script(script, self.compute_ng)
            except ProvisioningException, ex:
                self._logger.debug(ex)
                raise SectionError('Error executing BKM script %s' %
                                   self.bkm_ofed)


        ### Execute and copy sysctl bash script
        if self.bkm_sysctl != None:
            execute_script(self.bkm_sysctl, 'sysctl script', self._logger)

        if self.bkm_sysctl != None and self.compute_ng != None:
            path = __file__.split(os.path.sep)[:-1]
            script = os.path.sep.join( path + ['bash_scripts', self.bkm_sysctl])
            try:
                self._prov.add_script(script, self.compute_ng)
            except ProvisioningException, ex:
                self._logger.debug(ex)
                raise SectionError('Error executing BKM script %s' %
                                   self.bkm_sysctl)


SECTION_PROPERTIES = {'name': 'ICRIBConf', 'class': ICRIBConf}
TOOL_NAME = "Intel(R) Cluster Ready InfiniBand section"
TOOL_DESCRIPTION = "Configures an InfiniBand network"

if __name__ == "__main__":
    print "This file should not be run alone"
