#!/usr/bin/env python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.

""" This section builds the Intel(R) Lan drivers
"""

# Python imports
import os
import tempfile
import re
import time

# Custom imports
from common.section import Section, SectionError
from common.provisioning import ProvisioningException
from common.oslayer import UnzipException, UntarException, ModUnloadError
from common.oslayer import ServiceError
import common.recipeutils as ru

class ICRLanDrvUpdate(Section):
    """ Section to build the Intel(R) Lan drivers package
    """

    def __init__(self, mprov, mlogger, mconfig):
        """ Initialize the object """
        Section.__init__(self, mprov, mlogger, mconfig)

        self.section_name = "Intel(R) Cluster Ready Lan Driver Update"

        self._section = "ICRLanDriver"

        self._dict = {
            'DRIVER_PATH':
                [None, 'Path to the folder where the lan drivers are located',
                 True],
            'DRIVER_FILE':
            [None, 'Name of the zip file were the lan drivers are located' \
                 , True]
            }

        self._defaultdict = {
            'compute-icr-nodegroup':
                ['nodegroup',
                 'Intel(R) Cluster Ready compliant compute nodegroup name',
                 True],}

        # Holds the full path to temporary folder
        self.tmp_folder = None
        # Holds the name of the installer node group
        self.installer_nodegroup = None
        # Holds the path to driver package
        self.driver_path = None
        # Holds the name of the driver package
        self.driver_file = None
        # Holds the full path to driver package
        self.driver_full_path = None
        # Holds the name of the compute node group
        self.compute_nodegroup = None
        # Holds the structure of drivers
        self.s_drivers = None

    def section_configurator(self):
        """ Loads all the required parameters and validates them.
        """

        # Temporal folder to work
        self.tmp_folder = ''

        # Installer node nodegroup
        self.installer_nodegroup = self._prov.node_installer_group

        # Driver path
        self.driver_path = self.get_param('DRIVER_PATH')

        # Driver package
        self.driver_file = self.get_param('DRIVER_FILE')

        # Driver full path
        self.driver_full_path = os.path.join(self.driver_path, self.driver_file)

        # Compute node group
        self.compute_nodegroup = self.get_param('compute-icr-nodegroup')

    def check_requirements(self):
        """All the checks that will be done before executing the scripts goes
        here.
        """
        # Define the list that contains the packages to install
        install_devel_packages = self._prov.opsys.make_os_developer()
        
        # Install the required packages
        if len(install_devel_packages) != 0:
            try:
                for package in install_devel_packages:
                    self._prov.add_package(self.installer_nodegroup, package)
            except ProvisioningException, ex:
                raise SectionError("Error adding %s package required for "
                    + "building the drivers" % package)
            try:
                self._prov.sync(self.installer_nodegroup)
            except ProvisioningException, ex:
                raise SectionError(str(ex))

        # check if driver package exist
        _file = self.driver_full_path
        if not self._prov.opsys.check_file(_file):
            _msg = 'The file %s is required for this section' % _file
            raise SectionError(_msg)

    def section_dependency_check(self):
        """All the checks that are needed in this section that should be solved
        in previous sections """
        pass

    def run(self):
        """ Overridden method
        """
        # Create temporal folders for file handling
        self._step_1()
        # Unzip drivers package
        self._step_2()
        # Untar tar ball driver file
        self._step_3()
        # Build drivers rpm and checks the versions
        self._step_4()
        # Add drivers to front-end and compute-nodes
        self._step_5()
        # Synchronize database
        self._step_6()
        # Reload affected modules and restart network service
        self._step_7()


    def _step_1(self):
        """ Create temporal folders for file handling """

        self._logger.info("- Creating temporary directory")

        try:
            self.tmp_folder = tempfile.mkdtemp()
        except OSError, ex:
            raise SectionError(str(ex))

    def _step_2(self):
        """ Unzip drivers package """

        try:
            self._prov.opsys.unzip_file(self.driver_full_path, self.tmp_folder)
        except UnzipException, ex:
            raise SectionError(str(ex))

    def _step_3(self):
        """ Untar tar ball driver file """

        driver_tar_ball = os.path.join(
            self.tmp_folder, self.driver_file.replace(".zip", ".tar.gz"))

        if not self._prov.opsys.check_file(driver_tar_ball):
            _msg = "Required file %s could not be extracted from %s" % \
                (driver_tar_ball,self.driver_full_path)

            raise SectionError(_msg)

        try:
            self._prov.opsys.untar_file(driver_tar_ball, self.tmp_folder)
        except UntarException, ex:
            raise SectionError(str(ex))

    def _step_4(self):
        ''' Create a hash with  a list of drivers from a folder:
            {
            "driver_name" : {
                    "version" : <version number>,
                    "file"  : <tar ball file>,
                    "installer_package" : <rpm package created from "file">
            }

            Create RPM files using hash information and checks the versions
        '''


        driver_folder = os.path.join(self.tmp_folder, "drivers")

        contents = os.listdir(driver_folder)
        self.s_drivers = {}

        # Create the hash
        for elem in contents:
            _regex = re.search('(.*)-(\d.*).tar.gz', elem)

            if _regex:
                self._logger.debug( "Found driver %s version %s" % \
                                        (_regex.group(1), _regex.group(2)))
                self.s_drivers[_regex.group(1)] = {'version': _regex.group(2),
                                                'file' : elem }

        # Build the RPM package from tar.gz file
        for driver in self.s_drivers.keys():
            package = self.s_drivers[driver]['file']
            
            if self.s_drivers[driver]['version'] in \
                    self._prov.opsys.get_module_version(driver) :
                msg = '- driver %s has the same version as installed' % driver
                self._logger.info(msg)
                # If the driver is already installed remove it from the list
                self.s_drivers.__delitem__(driver)
            else:
                self._logger.info("Creating package for %s" % (package))
                full_path = os.path.join(driver_folder, package)

                if not self._prov.opsys.check_file(full_path):
                    raise SectionError("Missing file %s\n" % full_path)

                #Change the ownership of the file
                try:
                    ru.execute_command("chown root.root " + full_path)
                except ru.CommandException, ex:
                    raise SectionError(str(ex))

                # Call RPM build tool
                try:
                    output = ru.execute_command("rpmbuild -tb " + full_path)
                except ru.CommandException, ex:
                    raise SectionError(str(ex))

                built_package = None

                # Check the output to determine if the package was created
                for line in output:
                    _reg_ex = re.search('Wrote: (.*\.rpm)', line)
                    if _reg_ex:
                        built_package = _reg_ex.group(1)
                        if not self._prov.opsys.check_file(built_package):
                            msg = "Built package not found in system %s" \
                                % built_package
                            raise SectionError(msg)

                if not built_package:
                    raise SectionError("No package built for %s" % package)  

                self.s_drivers[driver]['installer_package'] = built_package

    def _step_5(self):
        """ Add drivers to front-end and compute-nodes """

        # Local vars
        drivers = self.s_drivers

        # Add RPM packages to repository
        if drivers is not None and drivers != {}:
            self._logger.info('Adding built packages to repository:')
            for driver in drivers.keys():
                try:
                    driver_package = drivers[driver]['installer_package']
                    self._prov.add_package_to_repo(driver_package)
                except ProvisioningException, ex:
                    raise SectionError(str(ex))
                except KeyError, ex:
                    self._logger.debug(str(ex))
                    raise SectionError("Could not find driver package for %s" %\
                        driver)

            # Updating the repository
            self._logger.info("- Updating provisioning repository")
            try:
                self._prov.update_repo()
            except ProvisioningException, ex:
                raise SectionError("Could not update repository")

            self._logger.info('Adding packages to node groups:')
            
            # Adding packages to node groups
            for driver in drivers.keys():
                try:
                    self._logger.info("- driver  %s added to %s" % 
                                      (driver, self.installer_nodegroup))
                    self._prov.add_package(self.installer_nodegroup, driver)
                except ProvisioningException, ex:
                    raise SectionError(str(ex))

                try:
                    self._logger.info("- driver  %s added to %s" % 
                                      (driver, self.compute_nodegroup))
                    self._prov.add_package(self.compute_nodegroup, driver)
                except ProvisioningException, ex:
                    raise SectionError(str(ex))

    def _step_6(self):
        """ Synchronize configuration """

        # Synchronize installer node group
        try:
            self._prov.sync(self.installer_nodegroup)
        except ProvisioningException, ex:
            raise SectionError(str(ex))

        # Synchronize compute node group
        try:
            self._prov.sync(self.compute_nodegroup)
        except ProvisioningException, ex:
            raise SectionError(str(ex))


    def _step_7(self):
        """ Reload affected modules and restart network service """

        self._logger.info('Configuring network service:')

        try:
            self._prov.opsys.service_stop('network')
            self._logger.info('- service stopped')
        except ServiceError, ex:
            raise SectionError(str(ex))

        # Local list of driver/s
        drivers = []
        drv = ''

        # Retrieve the name of the drivers
        for drv in self.s_drivers.keys():
            drivers.append(drv)

        # Check if there are any driver to configure
        self._logger.info('Reloading network drivers:')

        for drv in drivers:
            try:
                self._prov.opsys.unload_module(drv)
            except ModUnloadError, ex:
                raise SectionError(str(ex))
        
        # Wait for 5 seconds for ethernet drivers to finish loading
        time.sleep(5)

        try:
            self._prov.opsys.service_start('network')
            self._logger.info('- service started')
        except ServiceError, ex:
            raise SectionError(str(ex))

        # Reload dhcpd
        try:
            self._prov.opsys.service_restart('dhcpd')
        except ServiceError, ex:
            self._logger.debug(ex)
            raise SectionError('Error restarting dhcpd service')

        # Reload nfs
        try:
            self._prov.opsys.service_restart('nfs')
        except ServiceError, ex:
            self._logger.debug(ex)
            raise SectionError('Error restarting NFS service')


    def __del__(self):
        ''' Removes temporal folder created '''

        if self.tmp_folder:
            self._logger.debug("Removing temporal folder %s" % self.tmp_folder)

            for root, dirs, files in os.walk(self.tmp_folder, topdown=False):
                for name in files:
                    os.remove(os.path.join(root, name))
                for name in dirs:
                    os.rmdir(os.path.join(root, name))

            os.rmdir(self.tmp_folder)

SECTION_PROPERTIES = {'name': 'ICRLanDrvUpdate', 'class': ICRLanDrvUpdate}
TOOL_NAME = "Intel(R) Cluster Ready Lan Driver update"
TOOL_DESCRIPTION = "Section to build the Intel(R) lan drivers package"

if __name__ == "__main__":
    print "This file should not be run alone"
