#!/usr/bin/env python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.

""" This section compile, configure, and install OFED package
"""

# Custom imports
from common.section import Section, SectionError, SectionSkipped
from common.section import ParameterNotFound
from common.oslayer import OSALException, ServiceError
from common.provisioning import ProvisioningException
from common import recipeutils as ru

# Python imports
import ConfigParser
import tempfile
import time
import os
import re

__version__ = "$Rev: 36168 $"

class ICROFED(Section):
    """Section to build, configure, and install OFED package
    """

    def __init__(self, mprov, mlogger, mconfig):
        """ Initialize the object """
        Section.__init__(self, mprov, mlogger, mconfig)

        self.section_name = "Intel(R) Cluster Ready OFED configuration"

        self._section = "ICROFED"
        self._dict = {
            'PKG_PATH': [None,
                         'Path to folder where the OFED package is located',
                         True],
            'PKG_NAME': [None, 'Name of the OFED package file', True],
            }

        self._defaultdict = {
            'IB0_IP': ['ib0_ip', 'ib0 InfiniBand ip address', False],
            'IB0_NM': ['ib0_netmask', 'ib0 netmask address', False],
            'IB0_NETNAME': ['ib0_netname', 'ib0 netname', False],
            'compute-icr-nodegroup': ['nodegroup', 'Intel(R) Cluster Ready ' +
                                      'compliant compute nodegroup name', True],
            'SKIP_IB': ['no-infiniband', 'Do not configure Infiniband', False],}

        self._ib0_ip = None
        self._ib0_nm = None
        self._ofed_pkg = None
        self._ofed_old_pkg = None
        self._tmp_folder = None
        self._ofed_conf = 'ofed-all.conf'
        self._yum_conf = '/etc/yum.conf'
        self._sshd_conf = '/etc/sysconfig/sshd'
        self._compute_ng = None
        self.skip_ib = False

    def section_configurator(self):
        """ Loads all the required parameters and validates them.
        """

        # IPoIB configuration
        self._ib0_ip = self.get_param('IB0_IP')
        self._ib0_nm = self.get_param('IB0_NM')

        # Retrieve compute node nodegroup
        self._compute_ng = self.get_param('compute-icr-nodegroup')

        # Retrieve OFED package name and location
        self._ofed_pkg = os.path.join(self.get_param('PKG_PATH'),
                                      self.get_param('PKG_NAME'))

        # Create a temporary directory for package decompression
        self._tmp_folder = tempfile.mkdtemp()

        # Due some issue with the OFED uninstall utility some packages should
        # be removed before the installation.
        # Set the OFED related packages to remove
        self._ofed_old_pkg = ("mpich_mlx", "ibtsal", "openib", "opensm",
                              "opensm-devel", "opensm-libs", "mpi_ncsa",
                              "mpi_osu", "thca", "ib-osm", "osm", "diags",
                              "ibadm", "ib-diags", "ibgdiag", "ibdiag",
                              "ib-management", "ib-verbs", "ib-ipoib", "ib-cm",
                              "ib-sdp", "ib-dapl", "udapl", "udapl-devel",
                              "libdat", "libibat", "ib-kdapl", "ib-srp",
                              "ib-srp_target", "oiscsi-iser-support",
                              "libipathverbs", "libipathverbs-devel", "libehca",
                              "libehca-devel", "dapl", "dapl-devel", "libibcm",
                              "libibcm-devel", "libibcommon",
                              "libibcommon-devel", "libibmad", "libibmad-devel",
                              "libibumad", "libibumad-devel", "ibsim",
                              "ibsim-debuginfo", "libibverbs",
                              "libibverbs-devel", "libibverbs-utils",
                              "libipathverbs", "libipathverbs-devel",
                              "libmthca", "libmthca-devel", "libmlx4",
                              "libmlx4-devel", "libsdp", "librdmacm",
                              "librdmacm-devel", "librdmacm-utils",
                              "openib-diags", "openib-mstflint",
                              "openib-perftest", "openib-srptools",
                              "openib-tvflash", "openmpi", "openmpi-devel",
                              "openmpi-libs", "ibutils", "ibutils-devel",
                              "ibutils-libs", "libnes", "libnes-devel")

        try:
            self.skip_ib = self.get_param('SKIP_IB')
        except ParameterNotFound:
            # If not set default to False
            self.skip_ib = False
        if not self.skip_ib in ['True','False']:
            # Default to False if garbage is entered
            self.skip_ib = False

    def check_requirements(self):
        """All the checks that will be done before executing the scripts goes
        here.
        """
        # Check if this section need to run or should be skipped
        if self.skip_ib == 'True':
            raise SectionSkipped("Infiniband configuration disable.")

        if not self._prov.opsys.check_file(self._ofed_pkg):
            _msg = "Error: %s is required to install OFED" % self._ofed_pkg
            raise SectionError(_msg)

    def section_dependency_check(self):
        """All the checks that are needed in this section that should be solved
        in previous sections """
        pass

    def run(self):
        """ Overridden method
        """
        # Check section dependencies
        self.section_dependency_check()

        # Configure the section
        self.section_configurator()

        # Check section requirements
        self.check_requirements()

        # Remove existing MPI/OFED packages
        self._step_1()

        # Configure OFED installer
        self._step_2()

        # Build OFED packages
        self._step_3()

        # Update YUM configuration
        self._step_4()

        # Configure OFED related services
        self._step_5()

        # Add OFED packages to compute nodes
        self._step_6()

        # Configure SSHD to avoid RLIMIT MEMLOCK issues
        self._step_7()

    def _step_1(self):
        """Remove existing MPI/OFED things and install missing dependencies """

        self._logger.info("- Removing old MPI/OFED packages")

        # Read each of the packages and remove it
        for pname in self._ofed_old_pkg:
            self._logger.debug("Checking package: %s" % str(pname))
            if self._prov.opsys.has_package(pname):
                try:
                    self._logger.debug("Removing package: %s" % str(pname))
                    self._prov.opsys.remove_package(pname)
                except OSALException, ex:
                    raise SectionError(str(ex))

    def _step_2(self):
        """ Configure OFED installer """

        self._logger.info("- Creating OFED configuration file")

        # Decompress the OFED package
        self._prov.opsys.untar_file(self._ofed_pkg, self._tmp_folder)

        # Directory name created after decompression
        _ofed_dir = os.path.basename(os.path.splitext(self._ofed_pkg)[0])
        _ofed_dir = os.path.join(self._tmp_folder, _ofed_dir)

        # Create ofed.conf
        _cmd = os.path.join(_ofed_dir, 'install.pl')
        _cmd = _cmd + ' --all --build32 --print-available'
        try:
            ru.execute_command(_cmd)
        except ru.CommandException, ex:
            raise SectionError(str(ex))

        # Check if the file was created
        _file = os.path.join(_ofed_dir, self._ofed_conf)
        self._prov.opsys.check_file(_file)

        # Remove debuginfo entries from ofed-all.conf
        try:
            _fd = open(_file)
            _flines = _fd.readlines()
            _fd.close()
        except IOError, ex:
            raise SectionError(str(ex))

        # Regular expression to match debuginfo packages
        # e.g.
        # dapl=y # non-match
        # dapl-debuginfo=y # match!

        _regex = re.compile("^[\w-]*-debuginfo=y")

        try:
            _fd = open(_file, 'w')
        except IOError, ex:
            raise SectionError(str(ex))

        for _line in _flines:
            if not _regex.match(_line):
                _fd.write(_line)

        _fd.close()

        # Install flex package to allow building of opensm and opensm-libs
        self._prov.opsys.install_package("flex")

    def _step_3(self):
        """ Build OFED and install it into frontend"""

        self._logger.info("- Building and installing OFED packages")

        # Directory name created after decompression
        _ofed_dir = os.path.basename(os.path.splitext(self._ofed_pkg)[0])
        _ofed_dir = os.path.join(self._tmp_folder, _ofed_dir)

        # Check if the ofed-all.conf file was created
        _file = os.path.join(_ofed_dir, self._ofed_conf)
        self._prov.opsys.check_file(_file)

        # Run the OFED installer using custom ofed-all.conf
        _cmd = os.path.join(_ofed_dir, 'install.pl')
        _cmd = _cmd + ' -c %s' % self._ofed_conf
        try:
            ru.execute_command(_cmd)
        except ru.CommandException, ex:
            raise SectionError(str(ex))

    def _step_4(self):
        """ Update YUM to prevent updating OFED packages """

        self._logger.info("- Updating YUM configuration")

        _file = self._yum_conf

        # OFED installed packages needs to be removed from being updated
        # by YUM updates to avoid miss-function of Infiniband fabric

        _pkg2exclude = 'kernel\* wordpress tentakel compat-dapl compat-dapl-devel compat-dapl-devel-static compat-dapl-utils dapl dapl-devel dapl-devel-static dapl-utils ib-bonding ibsim ibutils infiniband-diags infinipath-psm infinipath-psm-devel kernel-ib kernel-ib-devel libcxgb3 libcxgb3-devel libibcm libibcm-devel libibmad libibmad-devel libibmad-static libibumad libibumad-devel libibumad-static libibverbs libibverbs-devel libibverbs-devel-static libibverbs-utils libipathverbs libipathverbs-devel libmlx4 libmlx4-devel libmthca libmthca-devel-static libnes libnes-devel-static librdmacm librdmacm-devel librdmacm-utils libsdp libsdp-devel mpi-selector mpitests_mvapich2_gcc mpitests_mvapich_gcc mpitests_openmpi_gcc mstflint mvapich2_gcc mvapich_gcc ofed-docs ofed-scripts openmpi_gcc opensm opensm-devel opensm-libs opensm-static perftest qperf rds-tools scsi-target-utils sdpnetstat srptools tgt compat-dapl-static dapl-static'

        # Check existence of YUM configuration file
        if self._prov.opsys.check_file(_file):
            _config = ConfigParser.SafeConfigParser()
            try:
                _fd = open(_file)
            except IOError, ex:
                raise SectionError(str(ex))

            _config.readfp(_fd)
            _fd.close()

            try:
                _config.set('main', 'exclude', _pkg2exclude)
            except ConfigParser.NoSectionError, ex:
                self._logger.debug(str(ex))
                self._logger.info("Warning! yum.conf not updated.")
            try:
                _fd = open(_file, 'w')
            except IOError, ex:
                raise SectionError(str(ex))
            _config.write(_fd)
            _fd.close()

        else:
            self._logger.info("Warning! No yum.conf file found.")

    def _step_5(self):
        """ Configure OFED related services """

        self._logger.info("- Setting services configurations")

        # Unload cxgb3i to avoid issues stopping openibd service
        self._prov.opsys.unload_module('cxgb3i')

        # Reload all related services
        try:
            self._prov.opsys.service_stop('opensmd')
            time.sleep(5)
        except ServiceError, ex:
            self._logger.debug("Error stopping opensmd: %s" % str(ex))

        try:
            self._prov.opsys.service_stop('openibd')
            time.sleep(5)
        except ServiceError, ex:
            self._logger.debug("Error stopping openibd: %s" % str(ex))

        try:
            self._prov.opsys.service_enable('opensmd')
            time.sleep(5)
        except ServiceError, ex:
            _msg = "Error enabling opensmd service: %s" % str(ex)
            raise SectionError(_msg)

        try:
            self._prov.opsys.service_enable('openibd')
            time.sleep(5)
        except ServiceError, ex:
            _msg = "Error enabling openibd service: %s" % str(ex)
            raise SectionError(_msg)

        try:
            self._prov.opsys.service_start('openibd')
            time.sleep(5)
        except ServiceError, ex:
            _msg = "Error starting openibd service: %s" % str(ex)
            raise SectionError(_msg)

        try:
            self._prov.opsys.service_start('opensmd')
            time.sleep(5)
        except ServiceError, ex:
            _msg = "Error starting opensmd service: %s" % str(ex)
            raise SectionError(_msg)

    def _step_6(self):
        """ Configure OFED packages for compute nodes """

        self._logger.info("- Adding OFED packages to provisioning repository")

        # Add OFED packages to provisioning repository
        _drivers_added = []

        # Directory name created after decompression
        _ofed_dir = os.path.basename(os.path.splitext(self._ofed_pkg)[0])
        _ofed_dir = os.path.join(self._tmp_folder, _ofed_dir)

        # Look for rpms
        _rpms_dir = os.path.join(_ofed_dir, 'RPMS')
        for root, dirs, files in os.walk(_rpms_dir, topdown=False):
            for name in files:
                _pkg = os.path.join(root, name)
                _pkg_ext = os.path.splitext(_pkg)[1]
                if _pkg_ext == '.rpm':
                    # Add package to repository
                    try:
                        self._prov.add_package_to_repo(_pkg)
                    except ProvisioningException, ex:
                        raise SectionError(str(ex))
                    except KeyError, ex:
                        raise SectionError(str(ex))

                    # Add package to extend compute file
                    # This is the list that should be used to add the packages
                    # to compute nodes but due an issue a workaround must to be
                    # used. See the following comment.
                    #_pkg_name = os.path.splitext(name)[0]
                    #_drivers_added.append(_pkg_name)

        self._logger.info("- Updating provisioning repository")
        self._prov.update_repo()

        self._logger.info("- Adding OFED packages to compute nodes")

        # The following packages have to be added to compute nodes in order to
        # get OFED installed on it. This is a WORKAROUND because there is an
        # issue adding all packages to the YUM repository.
        _drivers_added = ("compat-dapl", "compat-dapl-devel", "dapl",
                          "dapl-devel", "dapl-devel-static", "dapl-utils",
                          "ib-bonding", "ibsim", "ibutils", "infiniband-diags",
                          "kernel-ib", "kernel-ib-devel", "libcxgb3",
                          "libcxgb3-devel", "libibcm", "libibcm-devel",
                          "libibmad", "libibmad-devel", "libibmad-static",
                          "libibumad", "libibumad-devel", "libibumad-static",
                          "libibverbs", "libibverbs-devel",
                          "libibverbs-devel-static", "libibverbs-utils",
                          "libipathverbs", "libmlx4", "libmlx4-devel",
                          "libmthca", "libmthca-devel-static", "libnes",
                          "libnes-devel-static", "librdmacm", "librdmacm-devel",
                          "librdmacm-utils", "libsdp", "libsdp-devel",
                          "mpi-selector", "mpitests_mvapich2_gcc",
                          "mpitests_mvapich_gcc", "mpitests_openmpi_gcc",
                          "mstflint", "mvapich2_gcc", "mvapich_gcc",
                          "ofed-docs", "ofed-scripts", "openmpi_gcc", "opensm",
                          "opensm-devel", "opensm-libs", "opensm-static",
                          "perftest", "qperf", "rds-tools", "sdpnetstat",
                          "srptools")

        for _driver in _drivers_added:
            try:
                self._prov.add_package(self._compute_ng, _driver)
            except ProvisioningException, ex:
                raise SectionError(str(ex))

        # Update again to leave extend-compute file updated
        self._logger.info("- Updating compute-node configuration")
        self._prov.update_repo()

    def _step_7(self):
        """ Configure sshd to avoid RLIMIT MEMLOCK issues """

        self._logger.info("- Configuring sshd to avoid RLIMIT MEMLOCK issues")

        _file = self._sshd_conf
        _reg_match = False

        # Modify sshd sysconfig configuration file
        if self._prov.opsys.check_file(_file):
            try:
                _fd = open(_file)
                _flines = _fd.readlines()
                _fd.close()
            except IOError, ex:
                raise SectionError(str(ex))

            # Regular expression to match ulimit configuration
            # e.g.
            # ulimit * # match!

            _regex = re.compile("^ulimit \w*")

            try:
                _fd = open(_file, 'w')
            except IOError, ex:
                raise SectionError(str(ex))

            for _line in _flines:
                if _regex.match(_line):
                    _reg_match = True
                _fd.write(_line)

            # Set configuration
            if not _reg_match:
                _fd.write('# Fix for RLIMIT MEMLOCK issue\n')
                _fd.write('ulimit -l unlimited\n')
            _fd.close()
        else:
            try:
                _fd = open(_file, 'w')
            except IOError, ex:
                raise SectionError(str(ex))

            # Set configuration
            _fd.write('# Fix for RLIMIT MEMLOCK issue\n')
            _fd.write('ulimit -l unlimited\n')
            _fd.close()

        # Set the configuration for the current session
        try:
            ru.execute_command('ulimit -l unlimited')
        except ru.CommandException, ex:
            raise SectionError(str(ex))

        # Create a temporary file to host the script
        _tmp_script = tempfile.mkstemp()[1]

        try:
            _fd = open(_tmp_script, 'w')
        except IOError, ex:
            raise SectionError(str(ex))

        # Set configuration

        _fd.write('#!/bin/bash\n')
        _fd.write('\n')
        _fd.write('sshd=/etc/sysconfig/sshd\n')
        _fd.write('\n')
        _fd.write('if [ -f $sshd ]\n')
        _fd.write('    then\n')
        _fd.write('    cat $sshd | grep "^ulimit -l unlimited" -q\n')
        _fd.write('    if [ $? -ne 0 ]\n')
        _fd.write('        then\n')
        _fd.write('        echo "# Fix for RLIMIT MEMLOCK issue" >> $sshd\n')
        _fd.write('        echo "ulimit -l unlimited" >> $sshd\n')
        _fd.write('    fi\n')
        _fd.write('else\n')
        _fd.write('    echo "# Fix for RLIMIT MEMLOCK issue" > $sshd\n')
        _fd.write('    echo "ulimit -l unlimited" >> $sshd\n')
        _fd.write('fi\n')
        _fd.write('\n')

        # This line removes 32bit OFED library
        _fd.write('yum erase libibumad-1.2.3-1.el5\n')
        _fd.write('yum erase opensm-libs-3.2.6-2.el5\n')
        _fd.write('\n')

        _fd.close()

        # Add a script to configure sshd in compute nodes
        self._prov.add_script( _tmp_script, self._compute_ng)

        # Update again to leave extend-compute file updated
        self._logger.info("- Updating compute-node configuration")
        self._prov.update_repo()

        # Remove temporary file created
        os.remove(_tmp_script)

    def __del__(self):
        ''' Removes temporal folder created '''

        if self._tmp_folder:
            self._logger.debug("Removing temporal folder %s" % self._tmp_folder)
            for root, dirs, files in os.walk(self._tmp_folder, topdown=False):
                for name in files:
                    os.remove(os.path.join(root, name))
                for name in dirs:
                    os.rmdir(os.path.join(root, name))

            os.rmdir(self._tmp_folder)


SECTION_PROPERTIES = {'name': 'ICROFED', 'class': ICROFED}
TOOL_NAME = "Intel(R) Cluster Ready OFED configuration"
TOOL_DESCRIPTION = "Build, configure, and install an OFED package"

if __name__ == "__main__":
    print "This file should not be run alone"
