#!/usr/bin/env python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.
#

""" This script Installs the Intel(R) Cluster Runtimes package
    in the system 
"""

# Python imports
import exceptions
import os
import re
import tempfile

# ICR imports
import common.recipeutils as ru
from common.section import Section, SectionError, SectionSkipped
from common.oslayer import UntarException
from common.provisioning import ProvisioningException

__version__ = "$Rev: 35275 $"

# Exceptions


# Main class

class ICRRuntimesInstall(Section):
    """ Section to install Intel(R) Cluster Runtimes     
    """

    def __init__(self, prov, logger, conf):
        """ Initializes variables. """
        Section.__init__(self, prov, logger, conf)

        self.section_name = "Intel(R) Cluster Runtimes installation section"

        # Definitions of variable names for the config parser
        self._section = 'ICRRuntimesInstall'
        self._dict = {
            'RUNTIMES_PKG_PATH': [None,
                        'The runtimes package file location', True],
            'RUNTIMES_PKG_FILE': [None,
                           'Runtimes package file name', True]}

        self._defaultdict = {'compute-icr-nodegroup': \
                ['nodegroup', 'Intel(R) Cluster Ready compliant ' +\
                'compute nodegroup name', True]}

        # List of rpm files contained in the Intel Cluster Runtimes tar 
        # ball package
        self.package_list = {}

        self.runtimes_path = ''
        self.runtimes_file = ''
        self.compute_group = ''
        self.installer_group = ''


        self.tmp_folder = ''
        self.runtimes_tmp_folder = ''
        self.packages_tmp_rpm_folder = ''
        self.runtimes_full_path = ''


    def section_configurator(self):
        """ Loads all the required parameters and validates them
        """
        
        self.runtimes_path = self.get_param('RUNTIMES_PKG_PATH')
        self.runtimes_file = self.get_param('RUNTIMES_PKG_FILE')
        self.compute_group = self.get_param('compute-icr-nodegroup')

        self.runtimes_full_path = os.path.join(self.runtimes_path, 
            self.runtimes_file)

        self.installer_group = self._prov.node_installer_group

    def check_requirements(self):
        """All the checks that will be done before executing the scripts goes
        here"""
        
        if not self._prov.opsys.check_file(self.runtimes_full_path):
            raise SectionError("Missing file %s\n" % self.runtimes_full_path)
       

    def section_dependency_check(self):
        """All the checks that are needed in this section that should be solved
        in previous sections """
        pass


    def _get_packages(self, path):
        """ Gets a list of rpm packages from a folder
        """

        contents = os.listdir(path)
        package_hash = {}

        for elem in contents:
            match = re.search('(intel-.*)-(\d+\..*)\.(x86_64|i486|i386|noarch)'
                '\.rpm', elem)

            if match:
                self._logger.debug( "Found package %s version %s arch %s" % \
                    (match.group(1), match.group(2), match.group(3)))
                package_hash[match.group(1) + "." + match.group(3)] ={
                        'version': match.group(2),
                                            'file' : elem,
                                            'arch' : match.group(3)
                                        }

        return package_hash


    def run(self):
        """ Overridden method 
        """
        # Temporally executes the section_configurator and check_requirements

        # Create a temporary folder
        self._step_1()
        
        # Uncompress the tar ball file containing the Intel(R) Cluster
        # Runtimes
        self._step_2()
        
        # Read and accept the EULA's located at the eula folder
        # EULA acceptance is no longer required during the installation
        # of this package in the scripts so it was commented out. 
        # EULA acceptance is done when the package is downloaded.
        # self._step_3()
        
        # Copy packages to the repository
        self._step_4()
        
        # Install the packages on the frontend and add them to the 
        # compute node 
        self._step_5()


    def _step_1(self):
        """ Create a temporary folder """
        self._logger.debug("Creating directory folder")

        self.tmp_folder = tempfile.mkdtemp()

        if self.tmp_folder:
            self._logger.debug("Created temporal folder %s", self.tmp_folder)

        else:
            self._logger.debug("Could not create temporal folder")
            raise SectionError("Could not create temporal folder")


    def _step_2(self):
        """Uncompress the tar ball file containing the Intel(R) Cluster 
        Runtimes
        """
        
        try:
            self._prov.opsys.untar_file(self.runtimes_full_path , 
                self.tmp_folder)
        except UntarException, ex:
            raise SectionError(str(ex))

        
        self.runtimes_tmp_folder = self.tmp_folder + "/" +\
             self.runtimes_file.rstrip(".tar.gz")

        # Check that the folder where the runtimes was decompressed was created
        if not os.path.isdir(self.runtimes_tmp_folder):
            raise SectionError("Missing folder %s" % self.runtimes_tmp_folder)


        self.packages_tmp_rpm_folder = self.runtimes_tmp_folder + '/rpm'

        # Change the rpm file privileges so that they can be installed 
        # with the yum database
        try:
            ru.execute_command("chmod a+r " + self.packages_tmp_rpm_folder + 
                "/*")
        except ru.CommandException, ex:
            raise SectionError(str(ex))



        self.package_list = self._get_packages(self.packages_tmp_rpm_folder)

    
    def _step_3(self):
        """ Read and accept the EULA's located at the eula folder """


        eula_folder = self.runtimes_tmp_folder + "/eula"

        # Create master EULA that contains all the other EULA's
        eula_file = self._create_master_eula(eula_folder)

        # Show the EULA
        try:
            ru.show_multipage_file(eula_file)
        except ru.MultiPageException, ex:
            raise SectionError(str(ex))
            

        print "\n\n\n"
        print ("Do you agree to be bound by the terms and conditions of this "
            "license agreement?")

        answer = ""

        while answer != "accept" and answer != "reject":
         
            answer = raw_input("Enter 'accept' to continue, "
                "'reject' to quit the installation: ")

        if answer != "accept":
            self._logger.debug("Intel(R) Cluster Runtimes Eula's rejected")
            raise SectionError("Intel(R) Cluster Runtimes EULA was not "
                "accepted") 


        self._logger.debug("Intel(R) Cluster Runtimes Eula's accepted")




    def _step_4(self):
        """ Copy packages to the repository """ 

        if self.package_list is not None:
            for package in self.package_list.keys():
                try:
                    package_file = self.package_list[package]['file']
                    self._logger.debug("Adding package %s to repo"\
                        % (package_file))

                    self._prov.add_package_to_repo(self.packages_tmp_rpm_folder
                        + "/" + package_file)

                except ProvisioningException, ex:
                    raise SectionError(str(ex))
                except exceptions.KeyError, ex:
                    raise SectionError("Could not find file package %s" %\
                        package)

                self._logger.info("Added package %s, version %s, arch %s to "\
                    "repository" % (package, 
                    self.package_list[package]['version'],
                    self.package_list[package]['arch']))

            self._logger.info("Updating repository, this may take some time")

            try:
                self._prov.update_repo()
            except ProvisioningException, ex:
                raise SectionError("Could not update repository")

    def _step_5(self):
        """ Install the packages on the frontend and add them to the computes
        """

        self._add_packages_to_group(self.package_list, 
            self.installer_group)

        #For installing the packages in the compute node use a first boot 
        #script
        script = self._create_first_boot_script(self.package_list)
        try:
            self._prov.add_script(script, self.compute_group)
        except Exception, ex:
            self._logger.error(ex)
            raise SectionError('Error copying first boot script %s for ' \
                              'nodegroup %s' % (script,\
                              str(self.compute_group)))

        try:
            self._prov.rebuild_images()
        except NotImplementedError:
            #Do not do nothing if rebuild images not implemented. Assume it is
            #not necessary
            pass
            
    
    def _create_master_eula( self, eula_folder):
        """ Creates the master EULA that contains all the individual 
        Eulas contained in the folder. Returns the master eula name 
        """

        eula_list = os.listdir(eula_folder)

        eula_header = """
--------------------------------------------------------------------------------
Please carefully read each of the following license agreements.  Prior to
installing the software you will be asked to agree to the terms and conditions
of the following license agreements.
--------------------------------------------------------------------------------
        """
        
        master_eula_file = eula_folder + "/master_eula.txt"


        try:
            file_desc = open(master_eula_file, "w")
        except IOError, ex:
            msg = "Cannot create master eula file %s: %s"\
                % (master_eula_file, str(ex))
            raise SectionError(msg)

        # Add EULA instructions
        file_desc.write(eula_header)

        for eula_file in eula_list:
            eula_path = eula_folder + "/" + eula_file

            # Add file header
            file_desc.write("\n-----------------------------------------------"
                "---------------------------------\n")
            file_desc.write(eula_file)
            file_desc.write("\n-----------------------------------------------"
                "---------------------------------\n")
            
            
            try:
                fd_eula = open(eula_path)
            except IOError, ex:
                msg = "Cannot create eula file %s: %s"\
                                % (eula_path, str(ex))
                raise SectionError(msg)

            # Copy Eula to master eula
            for line in fd_eula.readlines():
                file_desc.write(line)

            fd_eula.close()

        file_desc.close()

        return master_eula_file        

    
    def _add_packages_to_group(self, packages, nodegroup):
        """ Adds the rpm packages to a nodegroup
        """
        if packages is not None and nodegroup is not None:
            print "Add packages to %s" % nodegroup
            
            list_pkg = packages.keys()
            list_pkg.sort()
            list_pkg.reverse()
            
            for package in list_pkg:
                package_name = package 
                try:
                    self._prov.add_package(nodegroup, package_name)
                except ProvisioningException, ex:
                    raise SectionError(str(ex))
 
            try:
                self._prov.sync(nodegroup)
            except ProvisioningException, ex:
                raise SectionError(str(ex))

    def _create_first_boot_script(self, packages):
        """ This function creates a first boot script for installing the 
            the packages with yum. This is required as the rpm files are
            not checking dependencies and some dependencies are not met so
            the packages are not installing correctly
        """
        script_full_path = None
        if packages is not None:
            self._logger.debug("Creating post install script for "
                "installing Intel Cluster Runtimes files ")

            list_pkg = packages.keys()
            list_pkg.sort()
            list_pkg.reverse()
            script_contents = ""
            script_full_path = self.tmp_folder + "/runtimes_install.sh"

            for package in list_pkg:
                package_name = package
                script_contents += "yum install %s\n" % package_name

            try:
                script_fd = open(script_full_path, "w")
            except IOError, ex:
                error_string = "Could not create first boot script: %s"\
                                % str(ex)
                raise SectionError(error_string)

            try:
                script_fd.write(script_contents)
            except IOError, ex:
                error_string = "Could not create first boot script: %s"\
                                % str(ex)
                raise SectionError(error_string)

            script_fd.close()

        return script_full_path 
            
    
    def __del__(self):
        """ Removes temporal folder created """

        if self.tmp_folder and os.path.isdir(self.tmp_folder):
            self._logger.debug("Removing temporal folder %s" % self.tmp_folder)

            for root, dirs, files in os.walk(self.tmp_folder, topdown=False):
                for name in files:
                    os.remove(os.path.join(root, name))
                for name in dirs:
                    os.rmdir(os.path.join(root, name))

            os.rmdir(self.tmp_folder)



SECTION_PROPERTIES = {'name': 'ICRRuntimesInstall', 
                    'class': ICRRuntimesInstall}
TOOL_NAME = "Intel(R) Cluster Runtimes installation"
TOOL_DESCRIPTION = "Installs Intel(R) Cluster Runtimes in compute nodes and"\
    " head node"

