#!/usr/bin/env python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.
#

""" This script checks components and requirements.
"""

import sys

import common.recipeutils as ru
from common.logger import set_conshdlr_emit
from common.section import Section, SectionError
from common.detect import detect_provisioning

__version__ = "$Rev: 37814 $"

# Script version
VERSION = '1.0'

class ICRSanity(Section):
    """ Section to check components and requirements.
    """

    def __init__(self, prov, logger, conf):
        """ Initializes variables. """
        Section.__init__(self, prov, logger, conf)

        self.section_name = "Intel(R) Cluster Ready Sanity checker"
        self._components = []
        self._services = []

        # Definitions of variable names for the config parser
        self._section = 'ICRSanity'
        self._dict = {'COMPONENTS': ['components',
        'The number of components that will be fetched from the config file ' \
        'of the form COMP<N>=name[:option=value]*' , True],
                      'SERVICES': ['services',
        'The number of components that will be fetched from the config file ' \
        'of the form SERV<N>=name[:option=value]*' , True],
        }

    def run(self):
        """ Overridden method 'run'.
        """

        success = True

        # local references to this attributes to make shorter calls
        prov = self._prov
        opsys = prov.opsys

        try:
            self._components = self.get_multi_param('COMPONENTS')
        except Exception, ex:
            error = 'Error getting components: %s' % ex
            self._logger.error(error)
            raise SectionError(error)

        try:
            self._services = self.get_multi_param('SERVICES')
        except Exception, ex:
            error = 'Error getting services: %s' % ex
            self._logger.error(error)
            raise SectionError(error)


        for component in self._components:
            comp_name = component['name']
            if prov.component_installed(comp_name):
                self._logger.info("%s is installed" % comp_name)
            else:
                success = False
                set_conshdlr_emit(lambda x: ru.cprint(x, color='red'))
                self._logger.error("%s is NOT installed" % comp_name)
                set_conshdlr_emit()

        for service in self._services:
            serv_name = service['name']
            serv_onboot = service.get('onboot', '').lower()
            serv_running = service.get('running', '').lower()

            self._logger.info('Checking service %s' % serv_name)
 
            # get the status only if necessary
            if serv_onboot in ('yes', 'no'):
                try:
                    init_status = opsys.service_init_status(serv_name)
                except Exception, ex:
                    raise SectionError('Error getting init status for '\
                                      'service %s: %s' % (serv_name, ex))

            if serv_onboot == 'yes':
                # check that is on in at least levels 3,4,5
                if not ( init_status[3] == 'on' and init_status[4] == 'on' \
                         and init_status[5] == 'on' ):
                    set_conshdlr_emit(lambda x: ru.cprint(x, color='red'))
                    self._logger.error('Service %s must be configured to ' \
                                 'be started at boot time.' % serv_name)
                    set_conshdlr_emit()
                    success = False
            elif serv_onboot == 'no':
                # check that is off in at least levels 3,4,5
                if not ( init_status[3] == 'off' and init_status[4] == 'off' \
                         and init_status[5] == 'off' ):
                    set_conshdlr_emit(lambda x: ru.cprint(x, color='red'))
                    self._logger.error('Service %s must not be configured ' \
                                 'to start at boot time.' % serv_name)
                    set_conshdlr_emit()
                    success = False

            serv_wanted_status = (serv_running == 'yes') and 'running' or \
                                 (serv_running == 'no') and 'stopped' or ''

            if serv_wanted_status:
                serv_is_running = opsys.service_running(serv_name)
                if (serv_is_running == True and \
                    serv_wanted_status != 'running') \
                    or (serv_is_running == False and \
                        serv_wanted_status != 'stopped'):
                    set_conshdlr_emit(lambda x: ru.cprint(x, color='red'))
                    self._logger.error('Service %s is not %s' % \
                    (serv_name, serv_wanted_status))
                    set_conshdlr_emit()
                    success = False

        if not success:
            raise SectionError('Sanity check failed. Check logger output ' \
                              'for info.')


SECTION_PROPERTIES = {'name': 'ICRSanity', 'class': ICRSanity}
TOOL_NAME = "Intel(R) Cluster Ready Sanity checker"
TOOL_DESCRIPTION = "Checks for installed components and running services"


if __name__ == "__main__":
    print "This file should not be run alone"
