#!/usr/bin/env python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.

""" This section installs the necessary libraries on a cluster defined in the
configuration file
"""

from common.section import Section, SectionError
from common.section import ParameterNotFound
from common import recipeutils as ru
from common import provisioning
import os 

__version__ = "$Rev: 37814 $"

class ComponentNotAdded(Exception):
    """ A component could not be added into a nodegroup """

class ICRSetup(Section):
    """ Installer class """

    def __init__(self, mprov, mlogger, mconfig):
        """ Initialize the object """
        Section.__init__(self, mprov, mlogger, mconfig)

        self.section_name = "Intel(R) Cluster Ready Compliance Setup"

        self._section = "ICRSetup"
        self._dict = {'installer-nodegroup':
                          ['installer-ng','installer node group name',True],
                      'compute-nodegroup':
                          ['compute-ng','compute node group name',False],
                      'ICR_SETUP_SCRIPT':  [None , 'name of the icr script ' \
                              'that contains ICR customizations to execute' \
                              ' on nodes.', False],
                      }

        self._defaultdict = {'compute-icr-nodegroup': \
            ['nodegroup', 'Intel(R) Cluster Ready compliant ' +\
            'compute nodegroup name', True],
            
        }

        self._nodegroups = []
        self._ng_installer = ''
        self._ng_compute = ''
        self._ng_icr = ''


    def _define_param(self):
        """ """
        # Getting the values here to allow set the other vars in the _dict
        self._ng_installer = self.get_param('installer-nodegroup')
        self._nodegroups.append(self._ng_installer)
        
        try:
            #optional parameter
            self._ng_compute = self.get_param('compute-nodegroup')
            self._nodegroups.append(self._ng_compute)
        except ParameterNotFound:
            self._ng_compute = None
            
        
        self._ng_icr = self.get_param('compute-icr-nodegroup')
        self._nodegroups.append(self._ng_icr)

        # Define this variables on runtime
        self._dict['%s-RPMs' % self._ng_installer] = \
            [None,'quantity of rpms to install',False]
        self._dict['%s-COMPs' % self._ng_installer] = \
            [None,'quantity of components to install',False]
        
        if self._ng_compute:
            self._dict['%s-RPMs' % self._ng_compute] = \
                [None,'quantity of rpms to install',False]
            self._dict['%s-COMPs' % self._ng_compute] = \
                [None,'quantity of components to install',False]
            
        self._dict['%s-RPMs' % self._ng_icr] = \
            [None,'quantity of rpms to install',False]
        self._dict['%s-COMPs' % self._ng_icr] = \
            [None,'quantity of components to install',False]


        try:
            self._icr_setup_script = self.get_param('ICR_SETUP_SCRIPT')
        except ParameterNotFound:
            self._icr_setup_script = None


    def run(self):
        """ Main method
        """
        
        self._define_param()

        
        if self._ng_compute != None:
            if not self._prov.has_group(self._ng_compute):
                self._logger.critical("Nodegroup %s not found!" % \
                                    self._ng_compute)
                raise SectionError("Nodegroup %s not found!" % \
                                    self._ng_compute)

        if not self._prov.has_group(self._ng_installer):
            raise SectionError ("The head node's group is not " + \
                                "present in the provisioning system")
        
        if not self._prov.has_group(self._ng_icr):
        
            ##############################################################
            #       Make a copy of the original compute-nodegroup        #
            ##############################################################
                
            
            if self._ng_compute != None:
                self._logger.info("Copying nodegroup %s to %s" 
                                  % (self._ng_compute, self._ng_icr))
                try:
                    self._prov.copy_nodegroup(self._ng_compute, self._ng_icr)
                except ru.CommandException:
                    self._logger.critical("The %s nodegroup already exists" 
                                          % self._ng_icr)
            else:
                self._logger.critical("Please define a valid icr " + \
                                      "compute nodegroup")
                raise SectionError ("Missing valid compute node group")

        ##############################################################
        #       Allow to be read-write the /home share folder        #
        ##############################################################

        self._set_exports()

        ##############################################################
        #    Install selected components into selected nodegroups    #
        ##############################################################

        self._install_components()

        ##############################################################
        #     Install missing packages into installer-nodegroup      #
        ##############################################################
        
        self._install_packages() 
        
        ##############################################################
        #     Execute customization script in head node and add      #
        #           add it to be executed in compute nodes           #
        ##############################################################


        if self._icr_setup_script:
        # if the icr setup script was defined, execute the script in the
        # head node and the in the 
            
            path = __file__.split(os.path.sep)[:-1]
            icr_script = os.path.sep.join( path + ['bash_scripts',\
                 self._icr_setup_script] )
            
            for ngrp in self._nodegroups:

                try:
                    self._prov.add_script(icr_script, ngrp)
                except provisioning.ProvisioningException, ex:
                    self._logger.error(ex)
                    raise SectionError('Error adding script %s to ' \
                                      'nodegroup %s: %s' % (icr_script,  \
                                        str(ngrp), str(ex)))

        # Sync the configurations made and run the init_script
        self._prov.sync(self._nodegroups)
        
        # Init selected components
        try:
            init_scripts = self.get_param('init-scripts')

            init_scripts = init_scripts.split(',')
            for init_script in init_scripts:
                self._prov.init_script(init_script)

        except:
            # If the init-scripts are not defined don't do anything
            pass
        

        try:
            self._prov.rebuild_images()
        except NotImplementedError:
            #Do not do nothing if rebuild images not implemented. Assume it is
            #not necessary
            pass


        ######################
        #     END OF RUN     #
        ######################


    def _install_components(self):
        """ Install selected components into nodegroups """
        
        # Retrieve the components
        for ngrp in self._nodegroups:
            try:
                components = [ comps['name'] for comps in
                               self.get_multi_param("%s-COMPs" % ngrp) ]
            except ParameterNotFound:
                self._logger.debug("No additional components defined in the " + \
                        "%s-COMPs parameter in the configuration file" % ngrp)
                
                components = None
                
            
            
            if components == None or len(components) == 0:
                self._logger.debug("No component to install into %s" % ngrp)
            else:
                ngid = self._prov.get_group_id(ngrp)
                for component in components:
                    cid = self._prov.get_component_id(component)                
                    # Check if it was already added
                    if self._prov.ng_has_comp(ngid, cid):
                        self._logger.info("The component %s is already added "
                                          "to %s" % (str(component),str(ngrp)))
                    else:
                        self._logger.info("Adding \"%s\" to \"%s\"" 
                                          % (component, ngrp))
                        # Add the component
                        try:
                            self._prov.add_component_to_group(cid, ngid)
                        except Exception:
                            self._logger.error("Failed to add %s to %s"
                                               % (component, ngrp))
                            raise ComponentNotAdded("Failed to add %s to %s"
                                                    % (component, ngrp))

    def _install_packages(self):
        """ Install selected packages into nodegroups """

        for nodegroup in self._nodegroups:
            try:
                packages = []
                packages = [ rpms['name'] for rpms in
                             self.get_multi_param("%s-RPMs" % nodegroup) ]
            except ParameterNotFound:
                self._logger.debug("No additional packages defined in " + \
                        "the %s-RPMs parameter in the configuration file" \
                        % nodegroup)
                
                packages = []
                
            if len(packages) == 0:
                self._logger.debug("No packages to install into %s" % nodegroup)
            else:
                for package in packages:
                    try:
                        self._prov.add_package(nodegroup, package)
                    except provisioning.PackageError, ex:
                        raise SectionError(str(ex))
    
    def _set_exports(self):
        """ Set the appropriate permissions on the /etc/exports file """
        self._logger.info("Setting no_root_squash to nfs exported folders")
        try:
            ru.execute_command("sed -i -e 's/,async)/,async,no_root_squash) " 
                               "#Modified by Intel(R) Cluster Ready installer/'"
                               " /etc/exports")
        except ru.CommandException, error:
            self._logger.error('"/etc/exports"  file could not be modified')
            raise SectionError(error)

        try:
            ru.execute_command("exportfs -arv")
        except ru.CommandException, error:
            self._logger.error('"exportfs -arv" failed to run')
            raise SectionError(error)


SECTION_PROPERTIES = {'name': 'ICRSetup', 'class': ICRSetup}

if __name__ == "__main__":
    print "This file should not be run alone"
