#!/usr/bin/env python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.
#

""" This script Configures the Sun Grid Engine (SGE) resource manager to be 
used with Intel MPI library.
"""

# Python imports
import re
import exceptions

# ICR imports
import common.recipeutils as ru
from common.section import Section, SectionError, SectionSkipped

__version__ = "$Rev: 35275 $"

# Exceptions

class CreatingParallelEnvironment(SectionError):
    """ Error trying to create a parallel environment into SGE
    """
    def __init__(self, value):
        self.value = value
        SectionError.__init__(self)
        
    def __str__(self):
        return repr(self.value)

class ImportingParallelEnvironment(SectionError):
    """ Error trying to create a parallel environment into SGE
    """
    def __init__(self, value):
        self.value = value
        SectionError.__init__(self)
        
        def __str__(self):
            return repr(self.value)

class CreatingQueueTemplate(SectionError):
    """ Error trying to create a template from SGE
    """
    def __init__(self, value):
        self.value = value
        SectionError.__init__(self)
        
        def __str__(self):
            return repr(self.value)

class ModifyingQueue(SectionError):
    """ Error trying to modify a queue from SGE
    """
    def __init__(self, value):
        self.value = value
        SectionError.__init__(self)
        
        def __str__(self):
            return repr(self.value)

class CreatingJobExample(SectionError):
    """ Error trying to create a job submission script for SGE
    """
    def __init__(self, value):
        self.value = value
        SectionError.__init__(self)
        
        def __str__(self):
            return repr(self.value)


# Main class

class ICRSGEConf(Section):
    """ Section to create a new parallel environment using Intel MPI library    
    """

    def __init__(self, prov, logger, conf):
        """ Initializes variables. """
        Section.__init__(self, prov, logger, conf)

        self.section_name = "Intel(R) Cluster Ready resource manager section"

        # Definitions of variable names for the config parser
        self._section = 'ICRSGEConf'
        self._dict = {
            'PE_NAME': ['pe_name',
                        'Parallel environment name used in Sun Grid Engine ',
                        True],
            'ALLOC_RULE': [None,
                           'Allocation rule of slots when submitting jobs using\
 Sun Grid Engine', True],
            'SGE_ROLL':  [None, 'Sun Grid Engine roll name', True]}

        self._defaultdict = {
            'INSTALL_DIR': ['install_dir', 'Intel(R) Cluster Ready '\
                                'installation directory', True],}

        self._pe_name = ''
        self._queue_name = ''
        self._alloc_rule = ''
        self._sge_roll = ''
        self._install_dir = ''
        self._icr_job = ''
        self._icr_pe = ''
        self._icr_queue = ''


    def section_configurator(self):
        """ Loads all the required parameters and validates them
        """

        self._pe_name = self.get_param('PE_NAME')

        self._queue_name = 'all.q'

        self._alloc_rule = self.get_param('ALLOC_RULE')
        
        self._sge_roll = self.get_param('SGE_ROLL')

        self._install_dir = self.get_param('INSTALL_DIR')

        self._icr_job = self._install_dir + '/etc/impi_job.template'

        self._icr_pe = self._install_dir + '/etc/impi_pe.template'

        self._icr_queue = self._install_dir + '/etc/impi_queue.template'

    def check_requirements(self):
        """All the checks that will be done before executing the scripts goes
        here"""

        self._logger.info('Checking for "%s" roll' % self._sge_roll)
        if not self._prov.component_installed(self._sge_roll):
            raise SectionSkipped('%s roll is not installed' % self._sge_roll)

    def section_dependency_check(self):
        """All the checks that are needed in this section that should be solved
        in previous sections """
        pass

    def run(self):
        """ Overridden method 
        """

        # Creates the parallel environment for Intel(R) MPI Library
        self._step_1()
        # Add the parallel environment created at step 1 into SGE
        self._step_2()
        # Associate parallel environment into the main queue
        self._step_3()
        # Creates an example submission file
        self._step_4()

    def _step_1(self):
        """ Create the parallel environment for Intel(R) MPI Library.
        """
        # Configuration contents of the parallel environment
        parallel_template = """
pe_name            %s
slots              9999
user_lists         NONE
xuser_lists        NONE
start_proc_args    NONE
stop_proc_args     NONE
allocation_rule    $%s
control_slaves     FALSE
job_is_first_task  FALSE
urgency_slots      min
accounting_summary FALSE
""" % (self._pe_name, self._alloc_rule)

        # Write configuration to file
        try:
            fhl = open(self._icr_pe,'w')
            fhl.writelines(parallel_template)
            fhl.close()
        except exceptions.IOError, ex:
            msg = 'Error creating parallel environment template file: %s'\
                % self._icr_pe + str(ex)
            raise CreatingParallelEnvironment(msg)

    def _step_2(self):
        """ Add the parallel environment created at step 1 into SGE"""

        try:
            cmd = 'qconf -Ap %s' % self._icr_pe
            ru.execute_command(cmd)
        except ru.CommandException, ex:
            msg = 'Error trying to import parallel environment "%s" into SGE' \
                % self._pe_name + str(ex)
            raise ImportingParallelEnvironment(msg)

    def _step_3(self):
        """ Associate parallel environment into the main queue"""

        # Read the default queue configuration and save it to a file
        try:
            cmd = 'qconf -sq %s > %s' % (self._queue_name, self._icr_queue)
            ru.execute_command(cmd)
        except ru.CommandException, ex:
            msg = 'Error trying to retrieve "%s" queue information' \
                % self._queue_name + str(ex)
            raise CreatingQueueTemplate(msg)

        # Read the file contents to allow editing it
        try:
            fhl = open(self._icr_queue)
        except exceptions.IOError, ex:
            msg = 'Error trying to open "%s":' % self._icr_queue + str(ex)
            raise SectionError(msg)
        file_contents = fhl.readlines()
        fhl.close()

        new_line = ''
        # Compile regular expression to match the variable to edit
        pattern = re.compile("(pe_list\s*)(.*)")

        # Open the file to change the variable value
        try:
            fhl = open(self._icr_queue, 'w')
        except exceptions.IOError, ex:
            msg = 'Error trying to open "%s":' % self._icr_queue + str(ex)
            raise SectionError(msg)

        modified = False

        # Read each line of the file. If it matchs the pattern edit if not
        # leave it as is
        for line in file_contents:
            matched = pattern.match(line)
            
            if matched:
                new_line = matched.group() + ' %s' % self._pe_name
                line = line.replace(line.rstrip('\n'), new_line)
                modified = True
            
            try:
                fhl.write(line)
            except exceptions.IOError, ex:
                msg = 'Error trying to open "%s":' % self._icr_queue + str(ex)
                raise SectionError(msg)
        fhl.close()

        # Check if the variable was modified. If not advise user
        if modified == False:
            msg = 'The parallel environment %s could not be added to queue: %s'\
                % (self._pe_name, self._queue_name)
            self._logger.warning(msg)

        # Configure the queue with the modified file as template
        try:
            cmd = 'qconf -Mq %s' % self._icr_queue
            ru.execute_command(cmd)
        except ru.CommandException, ex:
            msg = 'Error trying to modify "%s" queue' % self._queue_name +\
                str(ex)
            raise ModifyingQueue(msg)
        

    def _step_4(self):
        """ Creates an example submission file """ 
        # Configuration contents of the parallel environment
        submission_template = \
"""#!/bin/bash
#
# Submission template file
#
# Job name
#$ -N <job_name>
#
# Parallel environment
#$ -pe %s <num_of_processes>
#
# Pass env vars to process
#$ -V

<mpirun_absolute_name> -r ssh -np <num_of_process> <app_absolute_name>

""" % self._pe_name

        # Write configuration to file
        try:
            fhl = open(self._icr_job,'w')
            fhl.writelines(submission_template)
            fhl.close()
        except exceptions.IOError, ex:
            msg = 'Warning: job example file may be uncomplete.' + \
            'Please check %s:' % self._icr_job + str(ex)
            self._logger.info(msg)
       

SECTION_PROPERTIES = {'name': 'ICRSGEConf', 'class': ICRSGEConf}
TOOL_NAME = "Intel(R) Cluster Ready SGE configuration"
TOOL_DESCRIPTION = "Configures a parallel environment for SGE to allow running\
jobs using Intel(R) MPI Library"

