#!/usr/bin/env python
# Copyright 2009-2011 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents
# related to the source code ("Material") are owned by Intel
# Corporation or its suppliers or licensors. Title to the Material
# remains with Intel Corporation or its suppliers and licensors. The
# Material may contain trade secrets and proprietary and confidential
# information of Intel Corporation and its suppliers and licensors,
# and is protected by worldwide copyright and trade secret laws and
# treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you by
# disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.
#

""" This script Configures the user to be used in by Intel(R) Cluster Checker.
"""

import common.recipeutils as ru
from common.section import Section, SectionError, ParameterNotFound
from common.oslayer import OSALException
from common.provisioning import ProvisioningException

__version__ = "$Rev: 37814 $"

# Script version
VERSION = '1.0'
# Values by default

class ICRUser(Section):
    """ Section to create a new user and configure it to execute cluster checker
    
    """

    def __init__(self, prov, logger, conf):
        """ Initializes variables. """
        Section.__init__(self, prov, logger, conf)

        self.section_name = "Intel(R) Cluster Ready user setup section"
        self.modprobefile = None

        # Definitions of variable names for the config parser
        self._section = 'ICRUser'
        self._dict = {'ICR_USERNAME': ['user',
        'User name of the user used to execute Intel(R) Cluster Checker' \
        , False],
        'USER_PATH': [None,
        'Adds the line to the PATH of the user' \
        , False]}

        self._defaultdict = {
                      'MPIVARS':  [None, 'Path to the environmental variable '\
        'scripts for Intel(R) MPI ', True],
                      'CLCKVARS':  [None, 'Path to the environmental variable '\
        'scripts for Intel(R) Cluster Checker ', False],
                      'MKLVARS': [None, 'Path to the environmental variable ' \
        'scripts for Intel(R) MKL ', True],
                      'CCEVARS': [None, 'Path to the environmental variable ' \
        'scripts for Intel(R) CCE ', True], 
                      'FCEVARS': [None, 'Path to the environmental variable ' \
        'scripts for Intel(R) FCE ', True]}

        self._mpd_file = ".mpd.conf"
        self.mpivars = ""
        self.mklvars = ""
        self.clckvars = ""
        self.ccevars = ""
        self.fcevars = ""
        self.username = ""
        self._user_home_folder = ""


    def section_configurator(self):
        """ Loads all the required parameters and validates them
        """

        self.mpivars = self.get_param('MPIVARS')

        self.mklvars = self.get_param('MKLVARS')

        self.ccevars = self.get_param('CCEVARS')

        self.fcevars = self.get_param('FCEVARS')
        
        try:        
            self.clckvars = self.get_param('CLCKVARS')
        except ParameterNotFound, ex:
            self.clckvars = None               
      
        self.username = self.get_param('ICR_USERNAME')

        
        # If the param is registered with required=False it shouldn't throw 
        # an exception

        try:
            self.username = self.get_param('ICR_USERNAME')
        except Exception, ex:
            raise  SectionError(str(ex))

        # If the param is registered with required=False it shouldn't throw 
        # an exception
        
        try:
            self.user_path = self.get_param('USER_PATH')
        except SectionError, ex:
            self._logger.debug("USER_PATH not defined, ignoring parameter")
            self.user_path = None


    def check_requirements(self):
        """All the checks that will be done before executing the scripts goes
        here"""

        pass

    def section_dependency_check(self):
        """All the checks that are needed in this section that should be solved
        in previous sections """
        if not self._prov.opsys.check_file(self.mpivars):
            raise SectionError("Missing file %s\n" % self.mpivars)
        
        if not self._prov.opsys.check_file(self.mklvars):
            raise SectionError("Missing file %s\n" % self.mklvars)
       
        if not self._prov.opsys.check_file(self.ccevars):
            raise SectionError("Missing file %s\n" % self.ccevars)

        if not self._prov.opsys.check_file(self.fcevars):
            raise SectionError("Missing file %s\n" % self.fcevars)
        
        if self.clckvars:
            if not self._prov.opsys.check_file(self.clckvars):
                raise SectionError("Missing file %s\n" % self.clckvars)

        # Checks the existence dat.conf file in the path defined 
        if not self._prov.opsys.check_file(self._prov.opsys.DATCNF_FILE):
            raise SectionError("Missing file %s\n" 
                                % self._prov.opsys.DATCNF_FILE)
        
    def run(self):
        """ Overridden method 
        """

        # Create the Linux user that has all the Intel(R) Cluster Ready
        # specific configuration
        self._step_1()

        # Create the SSH keys for the Intel(R) Cluster Ready Linux user
        self._step_2()

        # Create an mpd configuration file in the user's home directory for
        # running Intel(R) MPI test modules executed from Intel(R) Cluster 
        # Checker
        self._step_3()

        # Update the bashrc file to add Intel(R) products to the path
        self._step_4()

        # Synchronize the Provisioning System database to spread the
        # creation of the user to the compute nodes 
        self._step_5()


    def _step_1(self):
        """ Create the Linux user that has all the Intel(R) Cluster Ready
        specific configuration
        """

        if self._prov.opsys.has_user(self.username):
            self._logger.debug("The user %s was already created " \
                                   % (self.username))
        else:
            try:
                self._prov.opsys.add_user(self.username)
            except OSALException, ex:
                raise SectionError(str(ex))
        
        

    def _step_2(self):
        """ Create the SSH keys for the Intel(R) Cluster Ready Linux user """
        
        try:
            self._user_home_folder = self._prov.opsys.get_home(self.username) \
                    + "/"
        except OSALException, ex:
            raise SectionError('Could not obtain home directory for user'\
                ' %s: %s' % (self.username, str(ex)))
        
        # Checks for ssh keys
        if self._prov.opsys.has_sshkeys(self.username):
            self._logger.warn("The user %s has the SSH keys generated" \
                                  % (self.username))
        else:
            try:
                self._prov.add_sshkeys(self.username)
            except ProvisioningException, ex:
                raise SectionError(str(ex))

    def _step_3(self):
        """ Create an mpd configuration file in the user's home directory for
        running Intel(R) MPI test modules executed from Intel(R) Cluster 
        Checker """
        # Create file .mpd.conf
        mpd_path = self._user_home_folder + self._mpd_file

        if not self._prov.opsys.check_file(mpd_path):
            try:
                file_d = open(mpd_path,'w')
            except Exception, ex:
                self._logger.critical("Could not create user mpd file: " \
                    + str(ex))
                raise SectionError(str(ex))

            file_d.write("secretword=anything\n")
            file_d.write("password=anything\n")

            file_d.close()

        #UPdate bash RC file

    def _step_4(self):
        """ Update the bashrc file to add Intel(R) products to the path"""

        mpd_path = self._user_home_folder + self._mpd_file
        bashrc_file = self._user_home_folder + ".bashrc"
        profile_file = self._prov.opsys.PROFILE_FILE
        
        if self.user_path:
            try:
                ru.execute_command("echo \"PATH=\\$PATH:" + self.user_path \
                    + "\" >> " + bashrc_file )
            except ru.CommandException, ex:
                raise SectionError(ex)
        
        # Adds mpivars, mklvars, clckvars, ccevars and fcevars to 
        # the icr user environment    

        try:
            ru.execute_command("echo source " + self.mpivars + "  >> " + \
                bashrc_file )
        except ru.CommandException, ex:
            raise SectionError(ex)
        
        try:
            ru.execute_command("echo source " + self.mklvars + "  >> " + \
                bashrc_file )
        except ru.CommandException, ex:
            raise SectionError(ex)

        try:
            ru.execute_command("echo source " + self.ccevars + "  >> " + \
                bashrc_file )
        except ru.CommandException, ex:
            raise SectionError(ex)

        try:
            ru.execute_command("echo source " + self.fcevars + "  >> " + \
                bashrc_file )
        except ru.CommandException, ex:
            raise SectionError(ex)
        
        # Adds support for dat.conf file location
        try:
            cmd = "echo export DAT_OVERRIDE=" + self._prov.opsys.DATCNF_FILE + \
                  "  >> " + bashrc_file
            ru.execute_command(cmd)
        except ru.CommandException, ex:
            raise SectionError(ex)

        if self.clckvars:
            try:
                pattern = "source " + self.clckvars
                if not ru.search_pattern_in_file( pattern, profile_file):
                    ru.execute_command("echo source " + self.clckvars + \
                    "  >> " + profile_file )
            except ru.CommandException, ex:
                raise SectionError(str(ex))
            except ru.SearchException, ex:
                raise SectionError(str(ex))

        #Change the ownership of the file
        try:
            ru.execute_command("chown " + self.username + "." + self.username\
               + " "  + mpd_path)
        except ru.CommandException, ex:
            raise SectionError(str(ex))

        #Change the file privileges so that only the current user can use it
        try:
            ru.execute_command("chmod 600 " + mpd_path)
        except ru.CommandException, ex:
            raise SectionError(str(ex))

    def _step_5(self):
        """ Synchronize the Provisioning System database to spread the
        creation of the user to the compute nodes """
        try:
            self._prov.sync()
        except ProvisioningException, ex:
            raise SectionError("Unable to sync database: %s" % str(ex))




SECTION_PROPERTIES = {'name': 'ICRUser', 'class': ICRUser}

TOOL_NAME = "Intel(R) Cluster Ready user setup section"
TOOL_DESCRIPTION = "Configures a user to support Intel(R) Cluster Checker" \
    + "executions"

if __name__ == "__main__":
    print "This file should not be run alone"
